@@ -1,17 +1,22 @@
 #!/usr/bin/env bash
 
+saveenv=$-
+set -ex
+
+
 # devstack plugin for octavia
 
 GET_PIP_CACHE_LOCATION=/opt/stack/cache/files/get-pip.py
 
 function octavia_install {
-    setup_develop $OCTAVIA_DIR
+    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
+        setup_develop $OCTAVIA_DIR redis
+    else
+        setup_develop $OCTAVIA_DIR
+    fi
     if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
         if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
-            if [[ ${DISTRO} =~ "rhel7" ]]; then
-                # Installing qemu would bring in the default OS qemu package,
-                # which is too old for Pike and later releases
-                # See https://review.openstack.org/#/c/438325 for details
+            if [[ ${DISTRO} =~ (rhel|centos) ]]; then
                 install_package qemu-kvm
             else
                 install_package qemu
@@ -29,13 +34,26 @@ function octaviaclient_install {
     fi
 }
 
+function octavia_lib_install {
+    if use_library_from_git "octavia-lib"; then
+        git_clone_by_name "octavia-lib"
+        setup_dev_lib "octavia-lib"
+        export DIB_REPOLOCATION_octavia_lib=${GITDIR["octavia-lib"]}
+        export DIB_REPOREF_octavia_lib=$(git --git-dir="${GITDIR["octavia-lib"]}/.git" log -1 --pretty="format:%H")
+    else
+        pip_install_gr octavia-lib
+    fi
+}
+
 function install_diskimage_builder {
     if use_library_from_git "diskimage-builder"; then
         GITREPO["diskimage-builder"]=$DISKIMAGE_BUILDER_REPO_URL
         GITDIR["diskimage-builder"]=$DISKIMAGE_BUILDER_DIR
         GITBRANCH["diskimage-builder"]=$DISKIMAGE_BUILDER_REPO_REF
         git_clone_by_name "diskimage-builder"
-        setup_dev_lib "diskimage-builder"
+        setup_dev_lib -bindep "diskimage-builder"
+    else
+        pip_install -r $OCTAVIA_DIR/diskimage-create/requirements.txt
     fi
 }
 
@@ -49,6 +67,16 @@ function build_octavia_worker_image {
     # set up diskimage-builder if we need to
     install_diskimage_builder
 
+    # Pull in DIB local elements if they are defined in devstack
+    if [ -n "$DIB_LOCAL_ELEMENTS" ]; then
+        export DIB_LOCAL_ELEMENTS=$DIB_LOCAL_ELEMENTS
+    fi
+
+    # Pull in the option to install nftables in the amphora
+    if [ -n "$DIB_OCTAVIA_AMP_USE_NFTABLES" ]; then
+        export DIB_OCTAVIA_AMP_USE_NFTABLES=$DIB_OCTAVIA_AMP_USE_NFTABLES
+    fi
+
     # pull the agent code from the current code zuul has a reference to
     if [ -n "$DIB_REPOLOCATION_pip_and_virtualenv" ]; then
         export DIB_REPOLOCATION_pip_and_virtualenv=$DIB_REPOLOCATION_pip_and_virtualenv
@@ -74,9 +102,37 @@ function build_octavia_worker_image {
     if [[ ${OCTAVIA_AMP_IMAGE_SIZE:+1} ]] ; then
     export PARAM_OCTAVIA_AMP_IMAGE_SIZE='-s '$OCTAVIA_AMP_IMAGE_SIZE
     fi
-    if ! [ -f $OCTAVIA_AMP_IMAGE_FILE ]; then
-        $OCTAVIA_DIR/diskimage-create/diskimage-create.sh $octavia_dib_tracing_arg -o $OCTAVIA_AMP_IMAGE_FILE ${PARAM_OCTAVIA_AMP_BASE_OS:-} ${PARAM_OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID:-} ${PARAM_OCTAVIA_AMP_IMAGE_SIZE:-}
+
+    # Use the infra pypi mirror if it is available
+    if [[ -e /etc/ci/mirror_info.sh ]]; then
+        source /etc/ci/mirror_info.sh
     fi
+    if [[ ${NODEPOOL_PYPI_MIRROR:+1} ]]; then
+        if [[ ${DIB_LOCAL_ELEMENTS:+1} ]]; then
+            export DIB_LOCAL_ELEMENTS="${DIB_LOCAL_ELEMENTS} pypi"
+        else
+            export DIB_LOCAL_ELEMENTS='pypi'
+        fi
+        export DIB_PYPI_MIRROR_URL=$NODEPOOL_PYPI_MIRROR
+        export DIB_PYPI_MIRROR_URL_1=$NODEPOOL_WHEEL_MIRROR
+        export DIB_PIP_RETRIES=0
+    fi
+
+    if ! [ -f $OCTAVIA_AMP_IMAGE_FILE ]; then
+        local dib_logs=/var/log/dib-build
+        if [[ -e ${dib_logs} ]]; then
+            sudo rm -rf ${dib_logs}
+        fi
+        sudo mkdir -m755 ${dib_logs}
+        sudo chown $STACK_USER ${dib_logs}
+        $OCTAVIA_DIR/diskimage-create/diskimage-create.sh -l ${dib_logs}/$(basename $OCTAVIA_AMP_IMAGE_FILE).log $octavia_dib_tracing_arg -o $OCTAVIA_AMP_IMAGE_FILE ${PARAM_OCTAVIA_AMP_BASE_OS:-} ${PARAM_OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID:-} ${PARAM_OCTAVIA_AMP_IMAGE_SIZE:-}
+    fi
+
+    if ! [ -f $OCTAVIA_AMP_IMAGE_FILE ]; then
+        echo "Diskimage-builder failed to create the amphora image. Aborting."
+        exit 1
+    fi
+
     upload_image file://${OCTAVIA_AMP_IMAGE_FILE} $TOKEN
 
 }
@@ -143,12 +199,12 @@ function _cleanup_octavia_apache_wsgi {
 
 function _start_octavia_apache_wsgi {
     if [[ "$WSGI_MODE" == "uwsgi" ]]; then
-        run_process o-api "$OCTAVIA_BIN_DIR/uwsgi --ini $OCTAVIA_UWSGI_CONF"
+        run_process o-api "$(which uwsgi) --ini $OCTAVIA_UWSGI_CONF"
         enable_apache_site octavia-wsgi
     else
         enable_apache_site octavia
-        restart_apache_server
     fi
+    restart_apache_server
 }
 
 function _stop_octavia_apache_wsgi {
@@ -157,54 +213,94 @@ function _stop_octavia_apache_wsgi {
         stop_process o-api
     else
         disable_apache_site octavia
-        restart_apache_server
     fi
+    restart_apache_server
 }
 
 function create_octavia_accounts {
     create_service_user $OCTAVIA
 
-    # Increase the service account secgroups quota
+    # Increase the octavia account secgroups quota
     # This is imporant for concurrent tempest testing
-    openstack quota set --secgroups 100 $SERVICE_PROJECT_NAME
+    openstack quota set --secgroups 100 $OCTAVIA_PROJECT_NAME
+    openstack quota set --secgroup-rules 1000 $OCTAVIA_PROJECT_NAME
 
-    local octavia_service=$(get_or_create_service "octavia" \
+    octavia_service=$(get_or_create_service "octavia" \
         $OCTAVIA_SERVICE_TYPE "Octavia Load Balancing Service")
 
-    if [[ "$WSGI_MODE" == "uwsgi" ]]; then
+    if [[ "$WSGI_MODE" == "uwsgi" ]] && [[ "$OCTAVIA_NODE" == "main" ]] ; then
+        get_or_create_endpoint $octavia_service \
+            "$REGION_NAME" \
+            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/$OCTAVIA_SERVICE_TYPE"
+    elif [[ "$WSGI_MODE" == "uwsgi" ]]; then
         get_or_create_endpoint $octavia_service \
             "$REGION_NAME" \
-            "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE" \
-            "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE" \
             "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE"
     else
         get_or_create_endpoint $octavia_service \
             "$REGION_NAME" \
-            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
-            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/" \
             "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/"
     fi
 }
 
+function install_redis {
+    if is_fedora; then
+        install_package redis
+    elif is_ubuntu; then
+        install_package redis-server
+    elif is_suse; then
+        install_package redis
+    else
+        exit_distro_not_supported "redis installation"
+    fi
+
+    start_service redis
+
+}
+
+function uninstall_redis {
+    if is_fedora; then
+        uninstall_package redis
+    elif is_ubuntu; then
+        uninstall_package redis-server
+    elif is_suse; then
+        uninstall_package redis
+    fi
+
+    stop_service redis
+
+}
+
 function octavia_configure {
 
     sudo mkdir -m 755 -p $OCTAVIA_CONF_DIR
     safe_chown $STACK_USER $OCTAVIA_CONF_DIR
 
+    sudo mkdir -m 700 -p $OCTAVIA_RUN_DIR
+    safe_chown $STACK_USER $OCTAVIA_RUN_DIR
+
     if ! [ -e $OCTAVIA_CONF ] ; then
         cp $OCTAVIA_DIR/etc/octavia.conf $OCTAVIA_CONF
     fi
 
+    if ! [ -e $OCTAVIA_AUDIT_MAP ] ; then
+        cp $OCTAVIA_DIR/etc/audit/octavia_api_audit_map.conf.sample $OCTAVIA_AUDIT_MAP
+    fi
+
     # Use devstack logging configuration
     setup_logging $OCTAVIA_CONF
+    iniset $OCTAVIA_CONF DEFAULT debug $ENABLE_DEBUG_LOG_LEVEL
 
     # Change bind host
-    iniset $OCTAVIA_CONF DEFAULT bind_host $SERVICE_HOST
+    iniset $OCTAVIA_CONF api_settings bind_host $(ipv6_unquote $SERVICE_HOST)
+    iniset $OCTAVIA_CONF api_settings api_handler queue_producer
 
     iniset $OCTAVIA_CONF database connection "mysql+pymysql://${DATABASE_USER}:${DATABASE_PASSWORD}@${DATABASE_HOST}:3306/octavia"
-
+    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
+        iniset $OCTAVIA_CONF task_flow persistence_connection "mysql+pymysql://${DATABASE_USER}:${DATABASE_PASSWORD}@${DATABASE_HOST}:3306/octavia_persistence"
+    fi
     # Configure keystone auth_token for all users
-    configure_auth_token_middleware $OCTAVIA_CONF octavia
+    configure_keystone_authtoken_middleware $OCTAVIA_CONF octavia
 
     # Ensure config is set up properly for authentication as admin
     iniset $OCTAVIA_CONF service_auth auth_url $OS_AUTH_URL
@@ -220,7 +316,9 @@ function octavia_configure {
     # Setting other required default options
     iniset $OCTAVIA_CONF controller_worker amphora_driver ${OCTAVIA_AMPHORA_DRIVER}
     iniset $OCTAVIA_CONF controller_worker compute_driver ${OCTAVIA_COMPUTE_DRIVER}
+    iniset $OCTAVIA_CONF controller_worker volume_driver  ${OCTAVIA_VOLUME_DRIVER}
     iniset $OCTAVIA_CONF controller_worker network_driver ${OCTAVIA_NETWORK_DRIVER}
+    iniset $OCTAVIA_CONF controller_worker image_driver ${OCTAVIA_IMAGE_DRIVER}
     iniset $OCTAVIA_CONF controller_worker amp_image_tag ${OCTAVIA_AMP_IMAGE_TAG}
 
     iniuncomment $OCTAVIA_CONF health_manager heartbeat_key
@@ -229,22 +327,11 @@ function octavia_configure {
     iniset $OCTAVIA_CONF house_keeping amphora_expiry_age ${OCTAVIA_AMP_EXPIRY_AGE}
     iniset $OCTAVIA_CONF house_keeping load_balancer_expiry_age ${OCTAVIA_LB_EXPIRY_AGE}
 
-    iniset $OCTAVIA_CONF DEFAULT api_handler queue_producer
-
     iniset $OCTAVIA_CONF DEFAULT transport_url $(get_transport_url)
 
     iniset $OCTAVIA_CONF oslo_messaging rpc_thread_pool_size 2
     iniset $OCTAVIA_CONF oslo_messaging topic octavia_prov
 
-    # TODO(nmagnezi): Remove this when neutron-lbaas gets deprecated
-    # Setting neutron request_poll_timeout
-    iniset $NEUTRON_CONF octavia request_poll_timeout 3000
-    if [[ "$WSGI_MODE" == "uwsgi" ]]; then
-        iniadd $NEUTRON_CONF octavia base_url "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE"
-    else
-        iniadd $NEUTRON_CONF octavia base_url "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/"
-    fi
-
     # Uncomment other default options
     iniuncomment $OCTAVIA_CONF haproxy_amphora base_path
     iniuncomment $OCTAVIA_CONF haproxy_amphora base_cert_dir
@@ -286,26 +373,49 @@ function octavia_configure {
     if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] || [ $OCTAVIA_NODE == 'api' ]; then
         recreate_database_mysql octavia
         octavia-db-manage upgrade head
+
+        if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
+            recreate_database_mysql octavia_persistence
+            octavia-db-manage upgrade_persistence
+        fi
     fi
 
     if [[ -a $OCTAVIA_CERTS_DIR ]] ; then
         rm -rf $OCTAVIA_CERTS_DIR
     fi
 
+    # amphorav2 required redis installation
+    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
+        install_redis
+    fi
+
     if [[ "$(trueorfalse False OCTAVIA_USE_PREGENERATED_CERTS)" == "True" ]]; then
         cp -rfp ${OCTAVIA_PREGENERATED_CERTS_DIR} ${OCTAVIA_CERTS_DIR}
     else
-        source $OCTAVIA_DIR/bin/create_certificates.sh $OCTAVIA_CERTS_DIR $OCTAVIA_DIR/etc/certificates/openssl.cnf
+        pushd $OCTAVIA_DIR/bin
+        source create_dual_intermediate_CA.sh
+        mkdir -p ${OCTAVIA_CERTS_DIR}/private
+        chmod 700 ${OCTAVIA_CERTS_DIR}/private
+        cp -p etc/octavia/certs/server_ca.cert.pem ${OCTAVIA_CERTS_DIR}/
+        cp -p etc/octavia/certs/server_ca-chain.cert.pem ${OCTAVIA_CERTS_DIR}/
+        cp -p etc/octavia/certs/server_ca.key.pem ${OCTAVIA_CERTS_DIR}/private/
+        cp -p etc/octavia/certs/client_ca.cert.pem ${OCTAVIA_CERTS_DIR}/
+        cp -p etc/octavia/certs/client.cert-and-key.pem ${OCTAVIA_CERTS_DIR}/private/
+        popd
     fi
 
-    iniset $OCTAVIA_CONF haproxy_amphora client_cert ${OCTAVIA_CERTS_DIR}/client.pem
-    iniset $OCTAVIA_CONF haproxy_amphora server_ca ${OCTAVIA_CERTS_DIR}/ca_01.pem
-    iniset $OCTAVIA_CONF certificates ca_certificate ${OCTAVIA_CERTS_DIR}/ca_01.pem
-    iniset $OCTAVIA_CONF certificates ca_private_key ${OCTAVIA_CERTS_DIR}/private/cakey.pem
-    iniset $OCTAVIA_CONF certificates ca_private_key_passphrase foobar
+    iniset $OCTAVIA_CONF certificates ca_certificate ${OCTAVIA_CERTS_DIR}/server_ca.cert.pem
+    iniset $OCTAVIA_CONF certificates ca_private_key ${OCTAVIA_CERTS_DIR}/private/server_ca.key.pem
+    iniset $OCTAVIA_CONF certificates ca_private_key_passphrase not-secure-passphrase
+    iniset $OCTAVIA_CONF controller_worker client_ca ${OCTAVIA_CERTS_DIR}/client_ca.cert.pem
+    iniset $OCTAVIA_CONF haproxy_amphora client_cert ${OCTAVIA_CERTS_DIR}/private/client.cert-and-key.pem
+    iniset $OCTAVIA_CONF haproxy_amphora server_ca ${OCTAVIA_CERTS_DIR}/server_ca-chain.cert.pem
+
+    # Controller side symmetric encryption, not used for PKI
+    iniset $OCTAVIA_CONF certificates server_certs_key_passphrase insecure-key-do-not-use-this-key
 
     if [[ "$OCTAVIA_USE_LEGACY_RBAC" == "True" ]]; then
-        cp $OCTAVIA_DIR/etc/policy/admin_or_owner-policy.json $OCTAVIA_CONF_DIR/policy.json
+        cp $OCTAVIA_DIR/etc/policy/admin_or_owner-policy.yaml $OCTAVIA_CONF_DIR/policy.yaml
     fi
 
     # create dhclient.conf file for dhclient
@@ -323,14 +433,19 @@ function octavia_configure {
     if [ $OCTAVIA_NODE == 'main' ]; then
         configure_octavia_api_haproxy
         # make sure octavia is reachable from haproxy
-        iniset $OCTAVIA_CONF DEFAULT bind_port ${OCTAVIA_HA_PORT}
-        iniset $OCTAVIA_CONF DEFAULT bind_host 0.0.0.0
+        iniset $OCTAVIA_CONF api_settings bind_port ${OCTAVIA_HA_PORT}
+        iniset $OCTAVIA_CONF api_settings bind_host 0.0.0.0
     fi
     if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ] ; then
         # make sure octavia is reachable from haproxy from main node
-        iniset $OCTAVIA_CONF DEFAULT bind_port ${OCTAVIA_HA_PORT}
-        iniset $OCTAVIA_CONF DEFAULT bind_host 0.0.0.0
+        iniset $OCTAVIA_CONF api_settings bind_port ${OCTAVIA_HA_PORT}
+        iniset $OCTAVIA_CONF api_settings bind_host 0.0.0.0
     fi
+
+    # set default graceful_shutdown_timeout to 300 sec (5 minutes)
+    # TODO(gthiemonge) update this value after persistant taskflow commits are
+    # merged
+    iniset $OCTAVIA_CONF DEFAULT graceful_shutdown_timeout 300
 }
 
 function create_mgmt_network_interface {
@@ -342,8 +457,8 @@ function create_mgmt_network_interface {
     MGMT_PORT_ID=$(openstack port create --security-group lb-health-mgr-sec-grp --device-owner Octavia:health-mgr --host=$(hostname) -c id -f value --network lb-mgmt-net $PORT_FIXED_IP octavia-health-manager-$OCTAVIA_NODE-listen-port)
     MGMT_PORT_MAC=$(openstack port show -c mac_address -f value $MGMT_PORT_ID)
 
-    # TODO(johnsom) This gets the IPv4 address, should be updated for IPv6
-    MGMT_PORT_IP=$(openstack port show -f value -c fixed_ips $MGMT_PORT_ID | awk '{FS=",| "; gsub(",",""); gsub("'\''",""); for(i = 1; i <= NF; ++i) {if ($i ~ /^ip_address/) {n=index($i, "="); if (substr($i, n+1) ~ "\\.") print substr($i, n+1)}}}')
+    MGMT_PORT_IP=$(openstack port show -f yaml -c fixed_ips $MGMT_PORT_ID | awk -v IP_VER=$SERVICE_IP_VERSION '{FS=",|";gsub(",","");gsub("'\''","");for(line = 1; line <= NF; ++line) {if ($line ~ /^.*- ip_address:/) {split($line, word, " ");if ((IP_VER == "4" || IP_VER == "") && word[3] ~ /\./) print word[3];if (IP_VER == "6" && word[3] ~ /:/) print word[3];} else {split($line, word, " ");for(ind in word) {if (word[ind] ~ /^ip_address=/) {split(word[ind], token, "=");if ((IP_VER == "4" || IP_VER == "") && token[2] ~ /\./) print token[2];if (IP_VER == "6" && token[2] ~ /:/) print token[2];}}}}}')
+
     if function_exists octavia_create_network_interface_device ; then
         octavia_create_network_interface_device o-hm0 $MGMT_PORT_ID $MGMT_PORT_MAC
     elif [[ $NEUTRON_AGENT == "openvswitch" || $Q_AGENT == "openvswitch" ]]; then
@@ -360,7 +475,34 @@ function create_mgmt_network_interface {
         die "Unknown network controller. Please define octavia_create_network_interface_device"
     fi
     sudo ip link set dev o-hm0 address $MGMT_PORT_MAC
-    sudo iptables -I INPUT -i o-hm0 -p udp --dport 5555 -j ACCEPT
+
+    # Check if the host is using nftables, an alternative to iptables
+    if [ -x "$(sudo bash -c 'command -v nft')" ]; then
+        sudo nft add table inet octavia
+        sudo nft add chain inet octavia o-hm0-incoming { type filter hook input priority 0\;}
+        sudo nft flush chain inet octavia o-hm0-incoming
+        # Note: Order is important here and using counter here as this is
+        # devstack for testing.
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" counter log drop
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" meta l4proto ipv6-icmp counter accept
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" udp dport $OCTAVIA_HM_LISTEN_PORT counter accept
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" udp dport $OCTAVIA_AMP_LOG_ADMIN_PORT counter accept
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" udp dport $OCTAVIA_AMP_LOG_TENANT_PORT counter accept
+        sudo nft insert rule inet octavia o-hm0-incoming iifname "o-hm0" ct state related,established accept
+    else
+        if [ $SERVICE_IP_VERSION == '6' ] ; then
+            # Allow the required IPv6 ICMP messages
+            sudo ip6tables -I INPUT -i o-hm0 -p ipv6-icmp -j ACCEPT
+            sudo ip6tables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_HM_LISTEN_PORT -j ACCEPT
+            sudo ip6tables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_ADMIN_PORT -j ACCEPT
+            sudo ip6tables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_TENANT_PORT -j ACCEPT
+        else
+            sudo iptables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_HM_LISTEN_PORT -j ACCEPT
+            sudo iptables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_ADMIN_PORT -j ACCEPT
+            sudo iptables -I INPUT -i o-hm0 -p udp --dport $OCTAVIA_AMP_LOG_TENANT_PORT -j ACCEPT
+        fi
+    fi
+
 
     if [ $OCTAVIA_CONTROLLER_IP_PORT_LIST == 'auto' ] ; then
         iniset $OCTAVIA_CONF health_manager controller_ip_port_list $MGMT_PORT_IP:$OCTAVIA_HM_LISTEN_PORT
@@ -371,29 +513,52 @@ function create_mgmt_network_interface {
     iniset $OCTAVIA_CONF health_manager bind_ip $MGMT_PORT_IP
     iniset $OCTAVIA_CONF health_manager bind_port $OCTAVIA_HM_LISTEN_PORT
 
+    iniset $OCTAVIA_CONF amphora_agent admin_log_targets "${MGMT_PORT_IP}:${OCTAVIA_AMP_LOG_ADMIN_PORT}"
+    iniset $OCTAVIA_CONF amphora_agent tenant_log_targets "${MGMT_PORT_IP}:${OCTAVIA_AMP_LOG_TENANT_PORT}"
+    # Setting these here as the devstack rsyslog configuration expects
+    # these values.
+    iniset $OCTAVIA_CONF amphora_agent user_log_facility 0
+    iniset $OCTAVIA_CONF amphora_agent administrative_log_facility 1
+
 }
 
 function build_mgmt_network {
     # Create network and attach a subnet
-    OCTAVIA_AMP_NETWORK_ID=$(openstack network create lb-mgmt-net -f value -c id)
-    OCTAVIA_AMP_SUBNET_ID=$(openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET --allocation-pool start=$OCTAVIA_MGMT_SUBNET_START,end=$OCTAVIA_MGMT_SUBNET_END --network lb-mgmt-net lb-mgmt-subnet -f value -c id)
-
-    # Create security group and rules
-    openstack security group create lb-mgmt-sec-grp
-    openstack security group rule create --protocol icmp lb-mgmt-sec-grp
-    openstack security group rule create --protocol tcp --dst-port 22 lb-mgmt-sec-grp
-    openstack security group rule create --protocol tcp --dst-port 9443 lb-mgmt-sec-grp
-    if [ $IPV6_ENABLED == 'true' ] ; then
-        openstack security group rule create --protocol icmpv6 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
-        openstack security group rule create --protocol tcp --dst-port 22 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
-        openstack security group rule create --protocol tcp --dst-port 9443 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
+    openstack network create lb-mgmt-net
+    if [ $SERVICE_IP_VERSION == '6' ] ; then
+        openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET_IPV6 --allocation-pool start=$OCTAVIA_MGMT_SUBNET_IPV6_START,end=$OCTAVIA_MGMT_SUBNET_IPV6_END --network lb-mgmt-net --ip-version 6 --ipv6-address-mode slaac --ipv6-ra-mode slaac lb-mgmt-subnet
+        # SLAAC needs a router on the subnet to advertise the prefix.
+        openstack router create lb-mgmt-router
+        openstack router add subnet lb-mgmt-router lb-mgmt-subnet
+    else
+        openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET --allocation-pool start=$OCTAVIA_MGMT_SUBNET_START,end=$OCTAVIA_MGMT_SUBNET_END --network lb-mgmt-net lb-mgmt-subnet
     fi
 
     # Create security group and rules
+    # Used for the amphora lb-mgmt-net ports
+    openstack security group create lb-mgmt-sec-grp
+    if [ $SERVICE_IP_VERSION == '6' ] ; then
+        openstack security group rule create --protocol ipv6-icmp --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
+        openstack security group rule create --protocol tcp --dst-port 22 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
+        openstack security group rule create --protocol tcp --dst-port 9443 --ethertype IPv6 --remote-ip ::/0 lb-mgmt-sec-grp
+    else
+        openstack security group rule create --protocol icmp lb-mgmt-sec-grp
+        openstack security group rule create --protocol tcp --dst-port 22 lb-mgmt-sec-grp
+        openstack security group rule create --protocol tcp --dst-port 9443 lb-mgmt-sec-grp
+    fi
+
+    # Create security group and rules
+    # Used for the health manager port
     openstack security group create lb-health-mgr-sec-grp
-    openstack security group rule create --protocol udp --dst-port $OCTAVIA_HM_LISTEN_PORT lb-health-mgr-sec-grp
-    if [ $IPV6_ENABLED == 'true' ] ; then
+    if [ $SERVICE_IP_VERSION == '6' ] ; then
+        openstack security group rule create --protocol ipv6-icmp --ethertype IPv6 --remote-ip ::/0 lb-health-mgr-sec-grp
         openstack security group rule create --protocol udp --dst-port $OCTAVIA_HM_LISTEN_PORT --ethertype IPv6 --remote-ip ::/0 lb-health-mgr-sec-grp
+        openstack security group rule create --protocol udp --dst-port $OCTAVIA_AMP_LOG_ADMIN_PORT --ethertype IPv6 --remote-ip ::/0 lb-health-mgr-sec-grp
+        openstack security group rule create --protocol udp --dst-port $OCTAVIA_AMP_LOG_TENANT_PORT --ethertype IPv6 --remote-ip ::/0 lb-health-mgr-sec-grp
+    else
+        openstack security group rule create --protocol udp --dst-port $OCTAVIA_HM_LISTEN_PORT lb-health-mgr-sec-grp
+        openstack security group rule create --protocol udp --dst-port $OCTAVIA_AMP_LOG_ADMIN_PORT lb-health-mgr-sec-grp
+        openstack security group rule create --protocol udp --dst-port $OCTAVIA_AMP_LOG_TENANT_PORT lb-health-mgr-sec-grp
     fi
 }
 
@@ -402,19 +567,9 @@ function configure_lb_mgmt_sec_grp {
     iniset ${OCTAVIA_CONF} controller_worker amp_secgroup_list ${OCTAVIA_MGMT_SEC_GRP_ID}
 }
 
-function configure_octavia_tempest {
-    # Load the amp_boot_network_list to tempest.conf and copy to tree
-
-    # TODO (ptoohill): remove check when tempest structure merges
-    if ! [ $OCTAVIA_TEMPEST == 'disabled' ] ; then
-        iniset $TEMPEST_CONFIG controller_worker amp_boot_network_list [$1]
-        cp $TEMPEST_CONFIG $OCTAVIA_TEMPEST_DIR/etc
-    fi
-}
-
 function create_amphora_flavor {
     # Pass even if it exists to avoid race condition on multinode
-    openstack flavor create --id auto --ram 1024 --disk ${OCTAVIA_AMP_IMAGE_SIZE:-2} --vcpus 1 --private m1.amphora -f value -c id || true
+    openstack flavor create --id auto --ram 1024 --disk ${OCTAVIA_AMP_IMAGE_SIZE:-2} --vcpus 1 --private m1.amphora -f value -c id --property hw_rng:allowed=True || true
     amp_flavor_id=$(openstack flavor show m1.amphora -f value -c id)
     iniset $OCTAVIA_CONF controller_worker amp_flavor_id $amp_flavor_id
 }
@@ -430,18 +585,33 @@ function configure_octavia_api_haproxy {
     NODES=(${OCTAVIA_NODES//,/ })
 
     for NODE in ${NODES[@]}; do
-       DATA=(${NODE//:/ })
-       NAME=$(echo -e "${DATA[0]}" | tr -d '[[:space:]]')
-       IP=$(echo -e "${DATA[1]}" | tr -d '[[:space:]]')
-       echo "   server octavia-${NAME} ${IP}:${OCTAVIA_HA_PORT} weight 1" >> ${OCTAVIA_CONF_DIR}/haproxy.cfg
+        DATA=(${NODE//:/ })
+        NAME=$(echo -e "${DATA[0]}" | tr -d '[[:space:]]')
+        IP=$(echo -e "${DATA[1]}" | tr -d '[[:space:]]')
+        echo "   server octavia-${NAME} ${IP}:80 weight 1" >> ${OCTAVIA_CONF_DIR}/haproxy.cfg
     done
 
 }
 
+function configure_rsyslog {
+    sudo cp ${OCTAVIA_DIR}/devstack/etc/rsyslog/10-octavia-log-offloading.conf /etc/rsyslog.d/
+    sudo sed -e "
+        s|%ADMIN_PORT%|${OCTAVIA_AMP_LOG_ADMIN_PORT}|g;
+        s|%TENANT_PORT%|${OCTAVIA_AMP_LOG_TENANT_PORT}|g;
+    " -i /etc/rsyslog.d/10-octavia-log-offloading.conf
+}
+
 function octavia_start {
 
     if  ! ps aux | grep -q [o]-hm0 && [ $OCTAVIA_NODE != 'api' ] ; then
-        sudo dhclient -v o-hm0 -cf $OCTAVIA_DHCLIENT_CONF
+        if [ $SERVICE_IP_VERSION == '6' ] ; then
+                # This is probably out of scope here? Load it from config
+                MGMT_PORT_IP=$(iniget $OCTAVIA_CONF health_manager bind_ip)
+            sudo ip addr add $MGMT_PORT_IP/64 dev o-hm0
+            sudo ip link set o-hm0 up
+        else
+            sudo dhclient -v o-hm0 -cf $OCTAVIA_DHCLIENT_CONF
+        fi
     fi
 
     if [ $OCTAVIA_NODE == 'main' ]; then
@@ -454,9 +624,12 @@ function octavia_start {
         run_process $OCTAVIA_API  "$OCTAVIA_API_BINARY $OCTAVIA_API_ARGS"
     fi
 
+    run_process $OCTAVIA_DRIVER_AGENT "$OCTAVIA_DRIVER_AGENT_BINARY $OCTAVIA_DRIVER_AGENT_ARGS"
     run_process $OCTAVIA_CONSUMER  "$OCTAVIA_CONSUMER_BINARY $OCTAVIA_CONSUMER_ARGS"
     run_process $OCTAVIA_HOUSEKEEPER  "$OCTAVIA_HOUSEKEEPER_BINARY $OCTAVIA_HOUSEKEEPER_ARGS"
     run_process $OCTAVIA_HEALTHMANAGER  "$OCTAVIA_HEALTHMANAGER_BINARY $OCTAVIA_HEALTHMANAGER_ARGS"
+
+    restart_service rsyslog
 }
 
 function octavia_stop {
@@ -466,6 +639,7 @@ function octavia_stop {
     else
         stop_process $OCTAVIA_API
     fi
+    stop_process $OCTAVIA_DRIVER_AGENT
     stop_process $OCTAVIA_CONSUMER
     stop_process $OCTAVIA_HOUSEKEEPER
     stop_process $OCTAVIA_HEALTHMANAGER
@@ -489,30 +663,44 @@ function octavia_stop {
 function octavia_cleanup {
 
     if [ ${OCTAVIA_AMP_IMAGE_NAME}x != x ] ; then
-         rm -rf ${OCTAVIA_AMP_IMAGE_NAME}*
+        rm -rf ${OCTAVIA_AMP_IMAGE_NAME}*
     fi
     if [ ${OCTAVIA_AMP_SSH_KEY_NAME}x != x ] ; then
-         rm -f  ${OCTAVIA_AMP_SSH_KEY_NAME}*
+        rm -f  ${OCTAVIA_AMP_SSH_KEY_NAME}*
     fi
     if [ ${OCTAVIA_SSH_DIR}x != x ] ; then
-         rm -rf ${OCTAVIA_SSH_DIR}
+        rm -rf ${OCTAVIA_SSH_DIR}
     fi
     if [ ${OCTAVIA_CONF_DIR}x != x ] ; then
-         sudo rm -rf ${OCTAVIA_CONF_DIR}
+        sudo rm -rf ${OCTAVIA_CONF_DIR}
+    fi
+    if [ ${OCTAVIA_RUN_DIR}x != x ] ; then
+        sudo rm -rf ${OCTAVIA_RUN_DIR}
     fi
     if [ ${OCTAVIA_AMP_SSH_KEY_PATH}x != x ] ; then
         rm -f ${OCTAVIA_AMP_SSH_KEY_PATH} ${OCTAVIA_AMP_SSH_KEY_PATH}.pub
     fi
     if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
         if [ ${OCTAVIA_AMP_SSH_KEY_NAME}x != x ] ; then
-            openstack keypair delete ${OCTAVIA_AMP_SSH_KEY_NAME}
+            openstack keypair delete ${OCTAVIA_AMP_SSH_KEY_NAME} || true
         fi
     fi
     if [[ "$OCTAVIA_USE_MOD_WSGI" == "True" ]]; then
         _cleanup_octavia_apache_wsgi
     fi
 
+    sudo rm -rf $OCTAVIA_DIR/bin/dual_ca
+    sudo rm -rf $OCTAVIA_DIR/bin/single_ca
+
     sudo rm -rf $NOVA_STATE_PATH $NOVA_AUTH_CACHE_DIR
+
+    if [[ ${OCTAVIA_ENABLE_AMPHORAV2_JOBBOARD} == True ]]; then
+        uninstall_redis
+    fi
+
+    sudo rm -f /etc/rsyslog.d/10-octavia-log-offloading.conf
+    restart_service rsyslog
+
 }
 
 function add_load-balancer_roles {
@@ -525,72 +713,70 @@ function add_load-balancer_roles {
 }
 
 function octavia_init {
-   if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ]  && [ $OCTAVIA_NODE != 'api' ]; then
-       # without the other services enabled apparently we don't have
-       # credentials at this point
-       TOP_DIR=$(cd $(dirname "$0") && pwd)
-       source ${TOP_DIR}/openrc admin admin
-   fi
+    if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ]  && [ $OCTAVIA_NODE != 'api' ]; then
+        # without the other services enabled apparently we don't have
+        # credentials at this point
+#        TOP_DIR=$(cd $(dirname "$0") && pwd)
+        source ${TOP_DIR}/openrc admin admin
+        OCTAVIA_AMP_NETWORK_ID=$(openstack network show lb-mgmt-net -f value -c id)
+        iniset $OCTAVIA_CONF controller_worker amp_boot_network_list ${OCTAVIA_AMP_NETWORK_ID}
+    fi
 
-   if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
-       # things that should only happen on the ha main node / or once
-       if ! openstack keypair show ${OCTAVIA_AMP_SSH_KEY_NAME} ; then
-           openstack keypair create --public-key ${OCTAVIA_AMP_SSH_KEY_PATH}.pub ${OCTAVIA_AMP_SSH_KEY_NAME}
-       fi
+    if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
+        # things that should only happen on the ha main node / or once
+        if ! openstack keypair show ${OCTAVIA_AMP_SSH_KEY_NAME} ; then
+            openstack keypair create --public-key ${OCTAVIA_AMP_SSH_KEY_PATH}.pub ${OCTAVIA_AMP_SSH_KEY_NAME}
+        fi
 
-       # Check if an amphora image is already loaded
-       AMPHORA_IMAGE_NAME=$(openstack image list --property name=${OCTAVIA_AMP_IMAGE_NAME} -f value -c Name)
-       export AMPHORA_IMAGE_NAME
+        # Check if an amphora image is already loaded
+        AMPHORA_IMAGE_NAME=$(openstack image list --property name=${OCTAVIA_AMP_IMAGE_NAME} -f value -c Name)
+        export AMPHORA_IMAGE_NAME
 
-       if [ "$AMPHORA_IMAGE_NAME" == ${OCTAVIA_AMP_IMAGE_NAME} ]; then
-           echo "Found existing amphora image: $AMPHORA_IMAGE_NAME"
-           echo "Skipping amphora image build"
-           export DISABLE_AMP_IMAGE_BUILD=True
-       fi
+        if [ "$AMPHORA_IMAGE_NAME" == ${OCTAVIA_AMP_IMAGE_NAME} ]; then
+            echo "Found existing amphora image: $AMPHORA_IMAGE_NAME"
+            echo "Skipping amphora image build"
+            export DISABLE_AMP_IMAGE_BUILD=True
+        fi
 
-       if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
-           build_octavia_worker_image
-       fi
+        if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
+            build_octavia_worker_image
+        fi
 
-       OCTAVIA_AMP_IMAGE_ID=$(openstack image list -f value --property name=${OCTAVIA_AMP_IMAGE_NAME} -c ID)
+        OCTAVIA_AMP_IMAGE_ID=$(openstack image list -f value --property name=${OCTAVIA_AMP_IMAGE_NAME} -c ID)
 
-       if [ -n "$OCTAVIA_AMP_IMAGE_ID" ]; then
-           openstack image set --tag ${OCTAVIA_AMP_IMAGE_TAG} ${OCTAVIA_AMP_IMAGE_ID}
-       fi
+        if [ -n "$OCTAVIA_AMP_IMAGE_ID" ]; then
+            openstack image set --tag ${OCTAVIA_AMP_IMAGE_TAG} --property hw_architecture='x86_64' --property hw_rng_model=virtio ${OCTAVIA_AMP_IMAGE_ID}
+        fi
 
-       # Create a management network.
-       build_mgmt_network
-       OCTAVIA_AMP_NETWORK_ID=$(openstack network show lb-mgmt-net -f value -c id)
-       iniset $OCTAVIA_CONF controller_worker amp_boot_network_list ${OCTAVIA_AMP_NETWORK_ID}
+        # Create a management network.
+        build_mgmt_network
+        OCTAVIA_AMP_NETWORK_ID=$(openstack network show lb-mgmt-net -f value -c id)
+        iniset $OCTAVIA_CONF controller_worker amp_boot_network_list ${OCTAVIA_AMP_NETWORK_ID}
 
-       create_octavia_accounts
+        create_octavia_accounts
 
-       # Adds service and endpoint
-       if is_service_enabled tempest; then
-           configure_octavia_tempest ${OCTAVIA_AMP_NETWORK_ID}
-       fi
+        add_load-balancer_roles
 
-       add_load-balancer_roles
-   elif [ $OCTAVIA_NODE == 'api' ] ; then
-       create_octavia_accounts
+        configure_rsyslog
+    elif [ $OCTAVIA_NODE == 'api' ] ; then
+        create_octavia_accounts
 
-       # Adds service and endpoint
-       if is_service_enabled tempest; then
-           configure_octavia_tempest ${OCTAVIA_AMP_NETWORK_ID}
-       fi
+        add_load-balancer_roles
+    fi
 
-       add_load-balancer_roles
-   fi
+    if [ $OCTAVIA_NODE != 'api' ] ; then
+        create_mgmt_network_interface
+        create_amphora_flavor
+        configure_lb_mgmt_sec_grp
+    fi
 
-   if [ $OCTAVIA_NODE != 'api' ] ; then
-       create_mgmt_network_interface
-       create_amphora_flavor
-       configure_lb_mgmt_sec_grp
-   fi
+    if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
+        set_octavia_worker_image_owner_id
+    fi
+}
 
-   if ! [ "$DISABLE_AMP_IMAGE_BUILD" == 'True' ]; then
-       set_octavia_worker_image_owner_id
-   fi
+function _configure_tempest {
+    iniset $TEMPEST_CONFIG service_available octavia "True"
 }
 
 # check for service enabled
@@ -610,6 +796,7 @@ if is_service_enabled $OCTAVIA; then
     if [[ "$1" == "stack" && "$2" == "install" ]]; then
         # Perform installation of service source
         echo_summary "Installing octavia"
+        octavia_lib_install
         octavia_install
         octaviaclient_install
 
@@ -626,6 +813,11 @@ if is_service_enabled $OCTAVIA; then
 
         echo_summary "Starting Octavia"
         octavia_start
+    elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
+        if is_service_enabled tempest; then
+            # Configure Tempest for Congress
+            _configure_tempest
+        fi
     fi
 fi
 
@@ -644,3 +836,14 @@ if [[ "$1" == "clean" ]]; then
         octavia_cleanup
     fi
 fi
+
+if [[ $saveenv =~ e ]]; then
+    set -e
+else
+    set +e
+fi
+if [[ $saveenv =~ x ]]; then
+    set -x
+else
+    set +x
+fi
