@@ -24,6 +24,7 @@ from octavia_lib.api.drivers import data_models as driver_dm
 from octavia_lib.api.drivers import exceptions
 from octavia_lib.api.drivers import provider_base as driver_base
 
+from octavia.api.drivers.amphora_driver import availability_zone_schema
 from octavia.api.drivers.amphora_driver import flavor_schema
 from octavia.api.drivers import utils as driver_utils
 from octavia.common import constants as consts
@@ -45,7 +46,7 @@ AMPHORA_SUPPORTED_LB_ALGORITHMS = [
 
 class AmphoraProviderDriver(driver_base.ProviderDriver):
     def __init__(self):
-        super(AmphoraProviderDriver, self).__init__()
+        super().__init__()
         topic = cfg.CONF.oslo_messaging.topic
         self.target = messaging.Target(
             namespace=consts.RPC_NAMESPACE_CONTROLLER_AGENT,
@@ -61,6 +62,19 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                 user_fault_string=msg,
                 operator_fault_string=msg)
 
+    def _validate_alpn_protocols(self, listener):
+        if not listener.alpn_protocols:
+            return
+        supported = consts.AMPHORA_SUPPORTED_ALPN_PROTOCOLS
+        not_supported = set(listener.alpn_protocols) - set(supported)
+        if not_supported:
+            msg = ('Amphora provider does not support %s ALPN protocol(s). '
+                   'Supported: %s'
+                   % (", ".join(not_supported), ", ".join(supported)))
+            raise exceptions.UnsupportedOptionError(
+                user_fault_string=msg,
+                operator_fault_string=msg)
+
     # Load Balancer
     def create_vip_port(self, loadbalancer_id, project_id, vip_dictionary):
         vip_obj = driver_utils.provider_vip_dict_to_vip_obj(vip_dictionary)
@@ -71,8 +85,11 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
         try:
             vip = network_driver.allocate_vip(lb_obj)
         except network_base.AllocateVIPException as e:
-            raise exceptions.DriverError(user_fault_string=e.orig_msg,
-                                         operator_fault_string=e.orig_msg)
+            message = str(e)
+            if getattr(e, 'orig_msg', None) is not None:
+                message = e.orig_msg
+            raise exceptions.DriverError(user_fault_string=message,
+                                         operator_fault_string=message)
 
         LOG.info('Amphora provider created VIP port %s for load balancer %s.',
                  vip.port_id, loadbalancer_id)
@@ -83,8 +100,11 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
     def loadbalancer_create(self, loadbalancer):
         if loadbalancer.flavor == driver_dm.Unset:
             loadbalancer.flavor = None
+        if loadbalancer.availability_zone == driver_dm.Unset:
+            loadbalancer.availability_zone = None
         payload = {consts.LOAD_BALANCER_ID: loadbalancer.loadbalancer_id,
-                   consts.FLAVOR: loadbalancer.flavor}
+                   consts.FLAVOR: loadbalancer.flavor,
+                   consts.AVAILABILITY_ZONE: loadbalancer.availability_zone}
         self.client.cast({}, 'create_load_balancer', **payload)
 
     def loadbalancer_delete(self, loadbalancer, cascade=False):
@@ -116,6 +136,7 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
 
     # Listener
     def listener_create(self, listener):
+        self._validate_alpn_protocols(listener)
         payload = {consts.LISTENER_ID: listener.listener_id}
         self.client.cast({}, 'create_listener', **payload)
 
@@ -125,6 +146,7 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
         self.client.cast({}, 'delete_listener', **payload)
 
     def listener_update(self, old_listener, new_listener):
+        self._validate_alpn_protocols(new_listener)
         listener_dict = new_listener.to_dict()
         if 'admin_state_up' in listener_dict:
             listener_dict['enabled'] = listener_dict.pop('admin_state_up')
@@ -201,9 +223,7 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
                    consts.MEMBER_UPDATES: member_dict}
         self.client.cast({}, 'update_member', **payload)
 
-    def member_batch_update(self, members):
-        # Get a list of existing members
-        pool_id = members[0].pool_id
+    def member_batch_update(self, pool_id, members):
         # The DB should not have updated yet, so we can still use the pool
         db_pool = self.repositories.pool.get(db_apis.get_session(), id=pool_id)
 
@@ -236,10 +256,13 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
             if m.id not in new_member_ids:
                 deleted_members.append(m)
 
-        payload = {'old_member_ids': [m.id for m in deleted_members],
-                   'new_member_ids': [m.member_id for m in new_members],
-                   'updated_members': updated_members}
-        self.client.cast({}, 'batch_update_members', **payload)
+        if deleted_members or new_members or updated_members:
+            payload = {'old_member_ids': [m.id for m in deleted_members],
+                       'new_member_ids': [m.member_id for m in new_members],
+                       'updated_members': updated_members}
+            self.client.cast({}, 'batch_update_members', **payload)
+        else:
+            LOG.info("Member batch update is a noop, returning early.")
 
     def _validate_members(self, db_pool, members):
         if db_pool.protocol == consts.PROTOCOL_UDP:
@@ -384,3 +407,90 @@ class AmphoraProviderDriver(driver_base.ProviderDriver):
             # TODO(johnsom) Fix this to raise a NotFound error
             # when the octavia-lib supports it.
             compute_driver.validate_flavor(compute_flavor)
+
+        amp_image_tag = flavor_dict.get(consts.AMP_IMAGE_TAG, None)
+        if amp_image_tag:
+            image_driver = stevedore_driver.DriverManager(
+                namespace='octavia.image.drivers',
+                name=CONF.controller_worker.image_driver,
+                invoke_on_load=True
+            ).driver
+
+            try:
+                image_driver.get_image_id_by_tag(
+                    amp_image_tag, CONF.controller_worker.amp_image_owner_id)
+            except Exception as e:
+                raise exceptions.NotFound(
+                    user_fault_string='Failed to find an image with tag {} '
+                                      'due to: {}'.format(
+                                          amp_image_tag, str(e)),
+                    operator_fault_string='Failed to find an image with tag '
+                                          '{} due to: {}'.format(
+                                              amp_image_tag, str(e)))
+
+    # Availability Zone
+    def get_supported_availability_zone_metadata(self):
+        """Returns the valid availability zone metadata keys and descriptions.
+
+        This extracts the valid availability zone metadata keys and
+        descriptions from the JSON validation schema and returns it as a
+        dictionary.
+
+        :return: Dictionary of availability zone metadata keys and descriptions
+        :raises DriverError: An unexpected error occurred.
+        """
+        try:
+            props = (
+                availability_zone_schema.SUPPORTED_AVAILABILITY_ZONE_SCHEMA[
+                    'properties'])
+            return {k: v.get('description', '') for k, v in props.items()}
+        except Exception as e:
+            raise exceptions.DriverError(
+                user_fault_string='Failed to get the supported availability '
+                                  'zone metadata due to: {}'.format(str(e)),
+                operator_fault_string='Failed to get the supported '
+                                      'availability zone metadata due to: '
+                                      '{}'.format(str(e)))
+
+    def validate_availability_zone(self, availability_zone_dict):
+        """Validates availability zone profile data.
+
+        This will validate an availability zone profile dataset against the
+        availability zone settings the amphora driver supports.
+
+        :param availability_zone_dict: The availability zone dict to validate.
+        :type availability_zone_dict: dict
+        :return: None
+        :raises DriverError: An unexpected error occurred.
+        :raises UnsupportedOptionError: If the driver does not support
+          one of the availability zone settings.
+        """
+        try:
+            validate(
+                availability_zone_dict,
+                availability_zone_schema.SUPPORTED_AVAILABILITY_ZONE_SCHEMA)
+        except js_exceptions.ValidationError as e:
+            error_object = ''
+            if e.relative_path:
+                error_object = '{} '.format(e.relative_path[0])
+            raise exceptions.UnsupportedOptionError(
+                user_fault_string='{0}{1}'.format(error_object, e.message),
+                operator_fault_string=str(e))
+        except Exception as e:
+            raise exceptions.DriverError(
+                user_fault_string='Failed to validate the availability zone '
+                                  'metadata due to: {}'.format(str(e)),
+                operator_fault_string='Failed to validate the availability '
+                                      'zone metadata due to: {}'.format(str(e))
+            )
+        compute_zone = availability_zone_dict.get(consts.COMPUTE_ZONE, None)
+        if compute_zone:
+            compute_driver = stevedore_driver.DriverManager(
+                namespace='octavia.compute.drivers',
+                name=CONF.controller_worker.compute_driver,
+                invoke_on_load=True
+            ).driver
+
+            # TODO(johnsom) Fix this to raise a NotFound error
+            # when the octavia-lib supports it.
+            compute_driver.validate_availability_zone(compute_zone)
