@@ -12,17 +12,20 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+import ipaddress
 
+from octavia_lib.api.drivers import data_models as driver_dm
 from oslo_config import cfg
 from oslo_db import exception as odb_exceptions
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import strutils
-import pecan
+from pecan import expose as pecan_expose
+from pecan import request as pecan_request
+from sqlalchemy.orm import exc as sa_exception
 from wsme import types as wtypes
 from wsmeext import pecan as wsme_pecan
 
-from octavia.api.drivers import data_models as driver_dm
 from octavia.api.drivers import driver_factory
 from octavia.api.drivers import utils as driver_utils
 from octavia.api.v2.controllers import base
@@ -49,13 +52,13 @@ class LoadBalancersController(base.BaseController):
     RBAC_TYPE = constants.RBAC_LOADBALANCER
 
     def __init__(self):
-        super(LoadBalancersController, self).__init__()
+        super().__init__()
 
     @wsme_pecan.wsexpose(lb_types.LoadBalancerRootResponse, wtypes.text,
                          [wtypes.text], ignore_extra_args=True)
     def get_one(self, id, fields=None):
         """Gets a single load balancer's details."""
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         load_balancer = self._get_db_lb(context.session, id,
                                         show_deleted=False)
 
@@ -77,15 +80,16 @@ class LoadBalancersController(base.BaseController):
                          [wtypes.text], ignore_extra_args=True)
     def get_all(self, project_id=None, fields=None):
         """Lists all load balancers."""
-        pcontext = pecan.request.context
+        pcontext = pecan_request.context
         context = pcontext.get('octavia_context')
 
         query_filter = self._auth_get_all(context, project_id)
 
-        load_balancers, links = self.repositories.load_balancer.get_all(
-            context.session, show_deleted=False,
-            pagination_helper=pcontext.get(constants.PAGINATION_HELPER),
-            **query_filter)
+        load_balancers, links = (
+            self.repositories.load_balancer.get_all_API_list(
+                context.session, show_deleted=False,
+                pagination_helper=pcontext.get(constants.PAGINATION_HELPER),
+                **query_filter))
         result = self._convert_db_to_type(
             load_balancers, [lb_types.LoadBalancerResponse])
         if fields is not None:
@@ -104,11 +108,23 @@ class LoadBalancersController(base.BaseController):
             raise exceptions.LBPendingStateError(
                 state=prov_status, id=id)
 
+    def _test_and_set_failover_prov_status(self, session, id):
+        lb_repo = self.repositories.load_balancer
+        if not lb_repo.set_status_for_failover(session, id,
+                                               constants.PENDING_UPDATE):
+            prov_status = lb_repo.get(session, id=id).provisioning_status
+            LOG.info("Invalid state %(state)s of loadbalancer resource %(id)s",
+                     {"state": prov_status, "id": id})
+            raise exceptions.LBPendingStateError(
+                state=prov_status, id=id)
+
     @staticmethod
-    def _validate_network_and_fill_or_validate_subnet(load_balancer):
+    def _validate_network_and_fill_or_validate_subnet(load_balancer,
+                                                      context=None):
         network = validate.network_exists_optionally_contains_subnet(
             network_id=load_balancer.vip_network_id,
-            subnet_id=load_balancer.vip_subnet_id)
+            subnet_id=load_balancer.vip_subnet_id,
+            context=context)
         if not load_balancer.vip_subnet_id:
             network_driver = utils.get_network_driver()
             if load_balancer.vip_address:
@@ -124,9 +140,29 @@ class LoadBalancersController(base.BaseController):
                         "VIP address specified."
                     ))
             else:
-                # If subnet and IP are not provided, pick the first subnet,
-                # preferring ipv4
-                for subnet_id in network.subnets:
+                # If subnet and IP are not provided, pick the first subnet with
+                # enough available IPs, preferring ipv4
+                if not network.subnets:
+                    raise exceptions.ValidationException(detail=_(
+                        "Supplied network does not contain a subnet."
+                    ))
+                ip_avail = network_driver.get_network_ip_availability(
+                    network)
+                if (CONF.controller_worker.loadbalancer_topology ==
+                        constants.TOPOLOGY_SINGLE):
+                    num_req_ips = 2
+                if (CONF.controller_worker.loadbalancer_topology ==
+                        constants.TOPOLOGY_ACTIVE_STANDBY):
+                    num_req_ips = 3
+                subnets = [subnet_id for subnet_id in network.subnets if
+                           utils.subnet_ip_availability(ip_avail, subnet_id,
+                                                        num_req_ips)]
+                if not subnets:
+                    raise exceptions.ValidationException(detail=_(
+                        "Subnet(s) in the supplied network do not contain "
+                        "enough available IPs."
+                    ))
+                for subnet_id in subnets:
                     # Use the first subnet, in case there are no ipv4 subnets
                     if not load_balancer.vip_subnet_id:
                         load_balancer.vip_subnet_id = subnet_id
@@ -134,14 +170,12 @@ class LoadBalancersController(base.BaseController):
                     if subnet.ip_version == 4:
                         load_balancer.vip_subnet_id = subnet_id
                         break
-                if not load_balancer.vip_subnet_id:
-                    raise exceptions.ValidationException(detail=_(
-                        "Supplied network does not contain a subnet."
-                    ))
 
     @staticmethod
-    def _validate_port_and_fill_or_validate_subnet(load_balancer):
-        port = validate.port_exists(port_id=load_balancer.vip_port_id)
+    def _validate_port_and_fill_or_validate_subnet(load_balancer,
+                                                   context=None):
+        port = validate.port_exists(port_id=load_balancer.vip_port_id,
+                                    context=context)
         validate.check_port_in_use(port)
         load_balancer.vip_network_id = port.network_id
 
@@ -154,27 +188,42 @@ class LoadBalancersController(base.BaseController):
                 isinstance(load_balancer.vip_qos_policy_id, wtypes.UnsetType)):
             load_balancer.vip_qos_policy_id = port_qos_policy_id
 
-        # Identify the subnet for this port
         if load_balancer.vip_subnet_id:
-            validate.subnet_exists(subnet_id=load_balancer.vip_subnet_id)
-        else:
-            if load_balancer.vip_address:
-                for port_fixed_ip in port.fixed_ips:
-                    if port_fixed_ip.ip_address == load_balancer.vip_address:
-                        load_balancer.vip_subnet_id = port_fixed_ip.subnet_id
-                        break
-                if not load_balancer.vip_subnet_id:
-                    raise exceptions.ValidationException(detail=_(
-                        "Specified VIP address not found on the "
-                        "specified VIP port."))
-            elif len(port.fixed_ips) == 1:
-                load_balancer.vip_subnet_id = port.fixed_ips[0].subnet_id
-            else:
+            # If we were provided a subnet_id, validate it exists and that
+            # there is a fixed_ip on the port that matches the provided subnet
+            validate.subnet_exists(subnet_id=load_balancer.vip_subnet_id,
+                                   context=context)
+            for port_fixed_ip in port.fixed_ips:
+                if port_fixed_ip.subnet_id == load_balancer.vip_subnet_id:
+                    load_balancer.vip_address = port_fixed_ip.ip_address
+                    break  # Just pick the first address found in the subnet
+            if not load_balancer.vip_address:
                 raise exceptions.ValidationException(detail=_(
-                    "VIP port's subnet could not be determined. Please "
-                    "specify either a VIP subnet or address."))
+                    "No VIP address found on the specified VIP port within "
+                    "the specified subnet."))
+        elif load_balancer.vip_address:
+            normalized_lb_ip = ipaddress.ip_address(
+                load_balancer.vip_address).compressed
+            for port_fixed_ip in port.fixed_ips:
+                normalized_port_ip = ipaddress.ip_address(
+                    port_fixed_ip.ip_address).compressed
+                if normalized_port_ip == normalized_lb_ip:
+                    load_balancer.vip_subnet_id = port_fixed_ip.subnet_id
+                    break
+            if not load_balancer.vip_subnet_id:
+                raise exceptions.ValidationException(detail=_(
+                    "Specified VIP address not found on the "
+                    "specified VIP port."))
+        elif len(port.fixed_ips) == 1:
+            # User provided only a port, get the subnet and address from it
+            load_balancer.vip_subnet_id = port.fixed_ips[0].subnet_id
+            load_balancer.vip_address = port.fixed_ips[0].ip_address
+        else:
+            raise exceptions.ValidationException(detail=_(
+                "VIP port's subnet could not be determined. Please "
+                "specify either a VIP subnet or address."))
 
-    def _validate_vip_request_object(self, load_balancer):
+    def _validate_vip_request_object(self, load_balancer, context=None):
         allowed_network_objects = []
         if CONF.networking.allow_vip_port_id:
             allowed_network_objects.append('vip_port_id')
@@ -206,10 +255,12 @@ class LoadBalancersController(base.BaseController):
 
         # Validate the port id
         if load_balancer.vip_port_id:
-            self._validate_port_and_fill_or_validate_subnet(load_balancer)
+            self._validate_port_and_fill_or_validate_subnet(load_balancer,
+                                                            context=context)
         # If no port id, validate the network id (and subnet if provided)
         elif load_balancer.vip_network_id:
-            self._validate_network_and_fill_or_validate_subnet(load_balancer)
+            self._validate_network_and_fill_or_validate_subnet(load_balancer,
+                                                               context=context)
         # Validate just the subnet id
         elif load_balancer.vip_subnet_id:
             subnet = validate.subnet_exists(
@@ -218,7 +269,6 @@ class LoadBalancersController(base.BaseController):
         if load_balancer.vip_qos_policy_id:
             validate.qos_policy_exists(
                 qos_policy_id=load_balancer.vip_qos_policy_id)
-        validate.network_allowed_by_config(load_balancer.vip_network_id)
 
     def _create_vip_port_if_not_exist(self, load_balancer_db):
         """Create vip port."""
@@ -235,27 +285,140 @@ class LoadBalancersController(base.BaseController):
                 e.msg = e.orig_msg
             raise e
 
+    def _get_provider(self, session, load_balancer):
+        """Decide on the provider for this load balancer."""
+
+        provider = None
+        if not isinstance(load_balancer.flavor_id, wtypes.UnsetType):
+            try:
+                provider = self.repositories.flavor.get_flavor_provider(
+                    session, load_balancer.flavor_id)
+            except sa_exception.NoResultFound as e:
+                raise exceptions.ValidationException(
+                    detail=_("Invalid flavor_id.")) from e
+
+        # No provider specified and no flavor specified, use conf default
+        if (isinstance(load_balancer.provider, wtypes.UnsetType) and
+                not provider):
+            provider = CONF.api_settings.default_provider_driver
+        # Both provider and flavor specified, they must match
+        elif (not isinstance(load_balancer.provider, wtypes.UnsetType) and
+                provider):
+            if provider != load_balancer.provider:
+                raise exceptions.ProviderFlavorMismatchError(
+                    flav=load_balancer.flavor_id, prov=load_balancer.provider)
+        # No flavor, but provider, use the provider specified
+        elif not provider:
+            provider = load_balancer.provider
+        # Otherwise, use the flavor provider we found above
+
+        return provider
+
+    def _apply_flavor_to_lb_dict(self, lock_session, driver, lb_dict):
+
+        flavor_dict = {}
+        if 'flavor_id' in lb_dict:
+            try:
+                flavor_dict = (
+                    self.repositories.flavor.get_flavor_metadata_dict(
+                        lock_session, lb_dict['flavor_id']))
+            except sa_exception.NoResultFound as e:
+                raise exceptions.ValidationException(
+                    detail=_("Invalid flavor_id.")) from e
+
+        # Make sure the driver will still accept the flavor metadata
+        if flavor_dict:
+            driver_utils.call_provider(driver.name, driver.validate_flavor,
+                                       flavor_dict)
+
+        # Apply the flavor settings to the load balanacer
+        # Use the configuration file settings as defaults
+        lb_dict[constants.TOPOLOGY] = flavor_dict.get(
+            constants.LOADBALANCER_TOPOLOGY,
+            CONF.controller_worker.loadbalancer_topology)
+
+        return flavor_dict
+
+    def _validate_flavor(self, session, load_balancer):
+        if not isinstance(load_balancer.flavor_id, wtypes.UnsetType):
+            flavor = self.repositories.flavor.get(session,
+                                                  id=load_balancer.flavor_id)
+            if not flavor:
+                raise exceptions.ValidationException(
+                    detail=_("Invalid flavor_id."))
+            if not flavor.enabled:
+                raise exceptions.DisabledOption(option='flavor',
+                                                value=load_balancer.flavor_id)
+
+    def _validate_and_return_az_dict(self, lock_session, driver, lb_dict):
+
+        az_dict = {}
+        if 'availability_zone' in lb_dict:
+            try:
+                az = self.repositories.availability_zone.get(
+                    lock_session, name=lb_dict['availability_zone'])
+                az_dict = (
+                    self.repositories.availability_zone
+                    .get_availability_zone_metadata_dict(lock_session, az.name)
+                )
+            except sa_exception.NoResultFound as e:
+                raise exceptions.ValidationException(
+                    detail=_("Invalid availability_zone.")) from e
+
+        # Make sure the driver will still accept the availability zone metadata
+        if az_dict:
+            try:
+                driver_utils.call_provider(driver.name,
+                                           driver.validate_availability_zone,
+                                           az_dict)
+            except NotImplementedError as e:
+                raise exceptions.ProviderNotImplementedError(
+                    prov=driver.name, user_msg="This provider does not support"
+                                               " availability zones.") from e
+
+        return az_dict
+
+    def _validate_availability_zone(self, session, load_balancer):
+        if not isinstance(load_balancer.availability_zone, wtypes.UnsetType):
+            az = self.repositories.availability_zone.get(
+                session, name=load_balancer.availability_zone)
+            if not az:
+                raise exceptions.ValidationException(
+                    detail=_("Invalid availability zone."))
+            if not az.enabled:
+                raise exceptions.DisabledOption(
+                    option='availability_zone',
+                    value=load_balancer.availability_zone)
+
     @wsme_pecan.wsexpose(lb_types.LoadBalancerFullRootResponse,
                          body=lb_types.LoadBalancerRootPOST, status_code=201)
     def post(self, load_balancer):
         """Creates a load balancer."""
         load_balancer = load_balancer.loadbalancer
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
 
         if not load_balancer.project_id and context.project_id:
             load_balancer.project_id = context.project_id
 
         if not load_balancer.project_id:
             raise exceptions.ValidationException(detail=_(
-                "Missing project ID in request where one is required."))
+                "Missing project ID in request where one is required. "
+                "An administrator should check the keystone settings "
+                "in the Octavia configuration."))
 
         self._auth_validate_action(context, load_balancer.project_id,
                                    constants.RBAC_POST)
 
-        self._validate_vip_request_object(load_balancer)
+        self._validate_vip_request_object(load_balancer, context=context)
+
+        self._validate_flavor(context.session, load_balancer)
+
+        self._validate_availability_zone(context.session, load_balancer)
+
+        provider = self._get_provider(context.session, load_balancer)
 
         # Load the driver early as it also provides validation
-        driver = driver_factory.get_driver(load_balancer.provider)
+        driver = driver_factory.get_driver(provider)
 
         lock_session = db_api.get_session(autocommit=False)
         try:
@@ -281,15 +444,31 @@ class LoadBalancersController(base.BaseController):
             listeners = lb_dict.pop('listeners', []) or []
             pools = lb_dict.pop('pools', []) or []
 
-            # TODO(johnsom) Remove flavor from the lb_dict
-            # as it has not been implemented beyond the API yet.
-            # Remove this line when it is implemented.
-            lb_dict.pop('flavor', None)
+            flavor_dict = self._apply_flavor_to_lb_dict(lock_session, driver,
+                                                        lb_dict)
+
+            az_dict = self._validate_and_return_az_dict(lock_session, driver,
+                                                        lb_dict)
+            # Validate the network as soon as we have the AZ data
+            validate.network_allowed_by_config(
+                load_balancer.vip_network_id,
+                valid_networks=az_dict.get(constants.VALID_VIP_NETWORKS))
 
             db_lb = self.repositories.create_load_balancer_and_vip(
                 lock_session, lb_dict, vip_dict)
 
-            # See if the provider driver wants to create the VIP port
+            # Pass the flavor dictionary through for the provider drivers
+            # This is a "virtual" lb_dict item that includes the expanded
+            # flavor dict instead of just the flavor_id we store in the DB.
+            lb_dict['flavor'] = flavor_dict
+
+            # Do the same with the availability_zone dict
+            lb_dict['availability_zone'] = az_dict
+
+            # See if the provider driver wants to manage the VIP port
+            # This will still be called if the user provided a port to
+            # allow drivers to collect any required information about the
+            # VIP port.
             octavia_owned = False
             try:
                 provider_vip_dict = driver_utils.vip_dict_to_provider_dict(
@@ -309,6 +488,10 @@ class LoadBalancersController(base.BaseController):
                 if 'port_id' not in vip_dict or not vip_dict['port_id']:
                     octavia_owned = True
 
+            # Check if the driver claims octavia owns the VIP port.
+            if vip.octavia_owned:
+                octavia_owned = True
+
             self.repositories.vip.update(
                 lock_session, db_lb.id, ip_address=vip.ip_address,
                 port_id=vip.port_id, network_id=vip.network_id,
@@ -330,9 +513,9 @@ class LoadBalancersController(base.BaseController):
                 driver_dm.LoadBalancer.from_dict(driver_lb_dict))
 
             lock_session.commit()
-        except odb_exceptions.DBDuplicateEntry:
+        except odb_exceptions.DBDuplicateEntry as e:
             lock_session.rollback()
-            raise exceptions.IDAlreadyExists()
+            raise exceptions.IDAlreadyExists() from e
         except Exception:
             with excutils.save_and_reraise_exception():
                 lock_session.rollback()
@@ -348,8 +531,8 @@ class LoadBalancersController(base.BaseController):
         pools_required = set()
         # Look through listeners and find any extra pools, and move them to the
         # top level so they are created first.
-        for l in listeners:
-            default_pool = l.get('default_pool')
+        for li in listeners:
+            default_pool = li.get('default_pool')
             pool_name = (
                 default_pool.get('name') if default_pool else None)
             # All pools need to have a name so they can be referenced
@@ -358,31 +541,33 @@ class LoadBalancersController(base.BaseController):
                     detail='Pools must be named when creating a fully '
                            'populated loadbalancer.')
             # If a pool has more than a name, assume it's a full specification
-            # (but use >2 because it will also have "enabled" as default)
-            if default_pool and len(default_pool) > 2:
+            # (but use >3 because it will also have "enabled" and "tls_enabled"
+            # as default)
+            if default_pool and len(default_pool) > 3:
                 pools.append(default_pool)
-                l['default_pool'] = {'name': pool_name}
+                li['default_pool'] = {'name': pool_name}
             # Otherwise, it's a reference and we record it and move on
             elif default_pool:
                 pools_required.add(pool_name)
             # We also need to check policy redirects
-            for policy in l.get('l7policies'):
+            for policy in li.get('l7policies'):
                 redirect_pool = policy.get('redirect_pool')
                 pool_name = (
                     redirect_pool.get('name') if redirect_pool else None)
                 # All pools need to have a name so they can be referenced
-                if default_pool and not pool_name:
+                if redirect_pool and not pool_name:
                     raise exceptions.ValidationException(
                         detail='Pools must be named when creating a fully '
                                'populated loadbalancer.')
                 # If a pool has more than a name, assume it's a full spec
-                # (but use >2 because it will also have "enabled" as default)
-                if redirect_pool and len(redirect_pool) > 2:
+                # (but use >2 because it will also have "enabled" and
+                # "tls_enabled" as default)
+                if redirect_pool and len(redirect_pool) > 3:
                     pool_name = redirect_pool['name']
                     policy['redirect_pool'] = {'name': pool_name}
                     pools.append(redirect_pool)
                 # Otherwise, it's a reference and we record it and move on
-                elif default_pool:
+                elif redirect_pool:
                     pools_required.add(pool_name)
 
         # Make sure all pool names are unique.
@@ -432,8 +617,8 @@ class LoadBalancersController(base.BaseController):
 
         # Now create all of the listeners
         new_lists = []
-        for l in listeners:
-            default_pool = l.pop('default_pool', None)
+        for li in listeners:
+            default_pool = li.pop('default_pool', None)
             # If there's a default pool, replace it with the ID
             if default_pool:
                 pool_name = default_pool['name']
@@ -441,11 +626,11 @@ class LoadBalancersController(base.BaseController):
                 if not pool_id:
                     raise exceptions.SingleCreateDetailsMissing(
                         type='Pool', name=pool_name)
-                l['default_pool_id'] = pool_id
-            l['load_balancer_id'] = db_lb.id
-            l['project_id'] = db_lb.project_id
+                li['default_pool_id'] = pool_id
+            li['load_balancer_id'] = db_lb.id
+            li['project_id'] = db_lb.project_id
             new_lists.append(listener.ListenersController()._graph_create(
-                lock_session, l, pool_name_ids=pool_name_ids))
+                lock_session, li, pool_name_ids=pool_name_ids))
 
         return new_pools, new_lists
 
@@ -455,17 +640,18 @@ class LoadBalancersController(base.BaseController):
     def put(self, id, load_balancer):
         """Updates a load balancer."""
         load_balancer = load_balancer.loadbalancer
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         db_lb = self._get_db_lb(context.session, id, show_deleted=False)
 
         self._auth_validate_action(context, db_lb.project_id,
                                    constants.RBAC_PUT)
 
-        if (load_balancer.vip_qos_policy_id and
-                not isinstance(load_balancer.vip_qos_policy_id,
-                               wtypes.UnsetType) and
-                db_lb.vip.qos_policy_id != load_balancer.vip_qos_policy_id):
-            validate.qos_policy_exists(load_balancer.vip_qos_policy_id)
+        if not isinstance(load_balancer.vip_qos_policy_id, wtypes.UnsetType):
+            network_driver = utils.get_network_driver()
+            validate.qos_extension_enabled(network_driver)
+            if load_balancer.vip_qos_policy_id is not None:
+                if db_lb.vip.qos_policy_id != load_balancer.vip_qos_policy_id:
+                    validate.qos_policy_exists(load_balancer.vip_qos_policy_id)
 
         # Load the driver early as it also provides validation
         driver = driver_factory.get_driver(db_lb.provider)
@@ -483,7 +669,8 @@ class LoadBalancersController(base.BaseController):
 
             # Also prepare the baseline object data
             old_provider_lb = (
-                driver_utils.db_loadbalancer_to_provider_loadbalancer(db_lb))
+                driver_utils.db_loadbalancer_to_provider_loadbalancer(
+                    db_lb, for_delete=True))
 
             # Dispatch to the driver
             LOG.info("Sending update Load Balancer %s to provider "
@@ -511,7 +698,7 @@ class LoadBalancersController(base.BaseController):
     @wsme_pecan.wsexpose(None, wtypes.text, wtypes.text, status_code=204)
     def delete(self, id, cascade=False):
         """Deletes a load balancer."""
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         cascade = strutils.bool_from_string(cascade)
         db_lb = self._get_db_lb(context.session, id, show_deleted=False)
 
@@ -533,11 +720,12 @@ class LoadBalancersController(base.BaseController):
             LOG.info("Sending delete Load Balancer %s to provider %s",
                      id, driver.name)
             provider_loadbalancer = (
-                driver_utils.db_loadbalancer_to_provider_loadbalancer(db_lb))
+                driver_utils.db_loadbalancer_to_provider_loadbalancer(
+                    db_lb, for_delete=True))
             driver_utils.call_provider(driver.name, driver.loadbalancer_delete,
                                        provider_loadbalancer, cascade)
 
-    @pecan.expose()
+    @pecan_expose()
     def _lookup(self, id, *remainder):
         """Overridden pecan _lookup method for custom routing.
 
@@ -557,11 +745,11 @@ class LoadBalancersController(base.BaseController):
         if is_children:
             controller = remainder[0]
             remainder = remainder[1:]
-            if controller == 'status' or controller == 'statuses':
+            if controller in ('status', 'statuses'):
                 return StatusController(lb_id=id), remainder
-            elif controller == 'stats':
+            if controller == 'stats':
                 return StatisticsController(lb_id=id), remainder
-            elif controller == 'failover':
+            if controller == 'failover':
                 return FailoverController(lb_id=id), remainder
         return None
 
@@ -570,13 +758,13 @@ class StatusController(base.BaseController):
     RBAC_TYPE = constants.RBAC_LOADBALANCER
 
     def __init__(self, lb_id):
-        super(StatusController, self).__init__()
+        super().__init__()
         self.id = lb_id
 
     @wsme_pecan.wsexpose(lb_types.StatusRootResponse, wtypes.text,
                          status_code=200)
     def get(self):
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         load_balancer = self._get_db_lb(context.session, self.id,
                                         show_deleted=False)
         if not load_balancer:
@@ -598,13 +786,13 @@ class StatisticsController(base.BaseController, stats.StatsMixin):
     RBAC_TYPE = constants.RBAC_LOADBALANCER
 
     def __init__(self, lb_id):
-        super(StatisticsController, self).__init__()
+        super().__init__()
         self.id = lb_id
 
     @wsme_pecan.wsexpose(lb_types.StatisticsRootResponse, wtypes.text,
                          status_code=200)
     def get(self):
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         load_balancer = self._get_db_lb(context.session, self.id,
                                         show_deleted=False)
         if not load_balancer:
@@ -626,13 +814,13 @@ class StatisticsController(base.BaseController, stats.StatsMixin):
 class FailoverController(LoadBalancersController):
 
     def __init__(self, lb_id):
-        super(FailoverController, self).__init__()
+        super().__init__()
         self.lb_id = lb_id
 
     @wsme_pecan.wsexpose(None, wtypes.text, status_code=202)
     def put(self, **kwargs):
         """Fails over a loadbalancer"""
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         db_lb = self._get_db_lb(context.session, self.lb_id,
                                 show_deleted=False)
 
@@ -643,7 +831,7 @@ class FailoverController(LoadBalancersController):
         driver = driver_factory.get_driver(db_lb.provider)
 
         with db_api.get_lock_session() as lock_session:
-            self._test_lb_status(lock_session, self.lb_id)
+            self._test_and_set_failover_prov_status(lock_session, self.lb_id)
             LOG.info("Sending failover request for load balancer %s to the "
                      "provider %s", self.lb_id, driver.name)
             driver_utils.call_provider(
