@@ -13,14 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from octavia_lib.api.drivers import data_models as driver_dm
 from oslo_db import exception as odb_exceptions
 from oslo_log import log as logging
 from oslo_utils import excutils
-import pecan
+from oslo_utils import strutils
+from pecan import request as pecan_request
 from wsme import types as wtypes
 from wsmeext import pecan as wsme_pecan
 
-from octavia.api.drivers import data_models as driver_dm
 from octavia.api.drivers import driver_factory
 from octavia.api.drivers import utils as driver_utils
 from octavia.api.v2.controllers import base
@@ -40,20 +41,22 @@ class MemberController(base.BaseController):
     RBAC_TYPE = constants.RBAC_MEMBER
 
     def __init__(self, pool_id):
-        super(MemberController, self).__init__()
+        super().__init__()
         self.pool_id = pool_id
 
     @wsme_pecan.wsexpose(member_types.MemberRootResponse, wtypes.text,
                          [wtypes.text], ignore_extra_args=True)
     def get(self, id, fields=None):
         """Gets a single pool member's details."""
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         db_member = self._get_db_member(context.session, id,
                                         show_deleted=False)
 
         self._auth_validate_action(context, db_member.project_id,
                                    constants.RBAC_GET_ONE)
 
+        self._validate_pool_id(id, db_member.pool_id)
+
         result = self._convert_db_to_type(
             db_member, member_types.MemberResponse)
         if fields is not None:
@@ -64,7 +67,7 @@ class MemberController(base.BaseController):
                          ignore_extra_args=True)
     def get_all(self, fields=None):
         """Lists all pool members of a pool."""
-        pcontext = pecan.request.context
+        pcontext = pecan_request.context
         context = pcontext.get('octavia_context')
 
         pool = self._get_db_pool(context.session, self.pool_id,
@@ -73,7 +76,7 @@ class MemberController(base.BaseController):
         self._auth_validate_action(context, pool.project_id,
                                    constants.RBAC_GET_ALL)
 
-        db_members, links = self.repositories.member.get_all(
+        db_members, links = self.repositories.member.get_all_API_list(
             context.session, show_deleted=False,
             pool_id=self.pool_id,
             pagination_helper=pcontext.get(constants.PAGINATION_HELPER))
@@ -87,10 +90,10 @@ class MemberController(base.BaseController):
     def _get_affected_listener_ids(self, session, member=None):
         """Gets a list of all listeners this request potentially affects."""
         if member:
-            listener_ids = [l.id for l in member.pool.listeners]
+            listener_ids = [li.id for li in member.pool.listeners]
         else:
             pool = self._get_db_pool(session, self.pool_id)
-            listener_ids = [l.id for l in pool.listeners]
+            listener_ids = [li.id for li in pool.listeners]
         return listener_ids
 
     def _test_lb_and_listener_and_pool_statuses(self, session, member=None):
@@ -98,6 +101,9 @@ class MemberController(base.BaseController):
         # We need to verify that any listeners referencing this member's
         # pool are also mutable
         pool = self._get_db_pool(session, self.pool_id)
+        # Check the parent is not locked for some reason (ERROR, etc.)
+        if pool.provisioning_status not in constants.MUTABLE_STATUSES:
+            raise exceptions.ImmutableObject(resource='Pool', id=self.pool_id)
         load_balancer_id = pool.load_balancer_id
         if not self.repositories.test_and_set_lb_and_listeners_prov_status(
                 session, load_balancer_id,
@@ -118,30 +124,28 @@ class MemberController(base.BaseController):
             constraint_list = ['uq_member_pool_id_address_protocol_port']
             if ['id'] == de.columns:
                 raise exceptions.IDAlreadyExists()
-            elif (set(column_list) == set(de.columns) or
-                  set(constraint_list) == set(de.columns)):
+            if (set(column_list) == set(de.columns) or
+                    set(constraint_list) == set(de.columns)):
                 raise exceptions.DuplicateMemberEntry(
                     ip_address=member_dict.get('ip_address'),
                     port=member_dict.get('protocol_port'))
-        except odb_exceptions.DBError:
+        except odb_exceptions.DBError as e:
             # TODO(blogan): will have to do separate validation protocol
             # before creation or update since the exception messages
             # do not give any information as to what constraint failed
-            raise exceptions.InvalidOption(value='', option='')
+            raise exceptions.InvalidOption(value='', option='') from e
+
+    def _validate_pool_id(self, member_id, db_member_pool_id):
+        if db_member_pool_id != self.pool_id:
+            raise exceptions.NotFound(resource='Member', id=member_id)
 
     @wsme_pecan.wsexpose(member_types.MemberRootResponse,
                          body=member_types.MemberRootPOST, status_code=201)
     def post(self, member_):
         """Creates a pool member on a pool."""
         member = member_.member
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
 
-        validate.ip_not_reserved(member.address)
-
-        # Validate member subnet
-        if member.subnet_id and not validate.subnet_exists(member.subnet_id):
-            raise exceptions.NotFound(resource='Subnet',
-                                      id=member.subnet_id)
         pool = self.repositories.pool.get(context.session, id=self.pool_id)
         member.project_id, provider = self._get_lb_project_id_provider(
             context.session, pool.load_balancer_id)
@@ -149,6 +153,13 @@ class MemberController(base.BaseController):
         self._auth_validate_action(context, member.project_id,
                                    constants.RBAC_POST)
 
+        validate.ip_not_reserved(member.address)
+
+        # Validate member subnet
+        if (member.subnet_id and
+                not validate.subnet_exists(member.subnet_id, context=context)):
+            raise exceptions.NotFound(resource='Subnet', id=member.subnet_id)
+
         # Load the driver early as it also provides validation
         driver = driver_factory.get_driver(provider)
 
@@ -197,16 +208,31 @@ class MemberController(base.BaseController):
 
         return db_member
 
+    def _set_default_on_none(self, member):
+        """Reset settings to their default values if None/null was passed in
+
+        A None/null value can be passed in to clear a value. PUT values
+        that were not provided by the user have a type of wtypes.UnsetType.
+        If the user is attempting to clear values, they should either
+        be set to None (for example in the name field) or they should be
+        reset to their default values.
+        This method is intended to handle those values that need to be set
+        back to a default value.
+        """
+        if member.backup is None:
+            member.backup = False
+        if member.weight is None:
+            member.weight = constants.DEFAULT_WEIGHT
+
     @wsme_pecan.wsexpose(member_types.MemberRootResponse,
                          wtypes.text, body=member_types.MemberRootPUT,
                          status_code=200)
     def put(self, id, member_):
         """Updates a pool member."""
         member = member_.member
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         db_member = self._get_db_member(context.session, id,
                                         show_deleted=False)
-
         pool = self.repositories.pool.get(context.session,
                                           id=db_member.pool_id)
         project_id, provider = self._get_lb_project_id_provider(
@@ -214,6 +240,10 @@ class MemberController(base.BaseController):
 
         self._auth_validate_action(context, project_id, constants.RBAC_PUT)
 
+        self._validate_pool_id(id, db_member.pool_id)
+
+        self._set_default_on_none(member)
+
         # Load the driver early as it also provides validation
         driver = driver_factory.get_driver(provider)
 
@@ -255,7 +285,7 @@ class MemberController(base.BaseController):
     @wsme_pecan.wsexpose(None, wtypes.text, status_code=204)
     def delete(self, id):
         """Deletes a pool member."""
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         db_member = self._get_db_member(context.session, id,
                                         show_deleted=False)
 
@@ -266,6 +296,8 @@ class MemberController(base.BaseController):
 
         self._auth_validate_action(context, project_id, constants.RBAC_DELETE)
 
+        self._validate_pool_id(id, db_member.pool_id)
+
         # Load the driver early as it also provides validation
         driver = driver_factory.get_driver(provider)
 
@@ -287,14 +319,15 @@ class MemberController(base.BaseController):
 class MembersController(MemberController):
 
     def __init__(self, pool_id):
-        super(MembersController, self).__init__(pool_id)
+        super().__init__(pool_id)
 
     @wsme_pecan.wsexpose(None, wtypes.text,
                          body=member_types.MembersRootPUT, status_code=202)
-    def put(self, members_):
+    def put(self, additive_only=False, members_=None):
         """Updates all members."""
         members = members_.members
-        context = pecan.request.context.get('octavia_context')
+        additive_only = strutils.bool_from_string(additive_only)
+        context = pecan_request.context.get('octavia_context')
 
         db_pool = self._get_db_pool(context.session, self.pool_id)
         old_members = db_pool.members
@@ -305,12 +338,14 @@ class MembersController(MemberController):
         # Check POST+PUT+DELETE since this operation is all of 'CUD'
         self._auth_validate_action(context, project_id, constants.RBAC_POST)
         self._auth_validate_action(context, project_id, constants.RBAC_PUT)
-        self._auth_validate_action(context, project_id, constants.RBAC_DELETE)
+        if not additive_only:
+            self._auth_validate_action(context, project_id,
+                                       constants.RBAC_DELETE)
 
         # Validate member subnets
         for member in members:
             if member.subnet_id and not validate.subnet_exists(
-                    member.subnet_id):
+                    member.subnet_id, context=context):
                 raise exceptions.NotFound(resource='Subnet',
                                           id=member.subnet_id)
 
@@ -320,13 +355,6 @@ class MembersController(MemberController):
         with db_api.get_lock_session() as lock_session:
             self._test_lb_and_listener_and_pool_statuses(lock_session)
 
-            member_count_diff = len(members) - len(old_members)
-            if member_count_diff > 0 and self.repositories.check_quota_met(
-                    context.session, lock_session, data_models.Member,
-                    db_pool.project_id, count=member_count_diff):
-                raise exceptions.QuotaException(
-                    resource=data_models.Member._name())
-
             old_member_uniques = {
                 (m.ip_address, m.protocol_port): m.id for m in old_members}
             new_member_uniques = [
@@ -337,6 +365,7 @@ class MembersController(MemberController):
             updated_members = []
             for m in members:
                 if (m.address, m.protocol_port) not in old_member_uniques:
+                    validate.ip_not_reserved(m.address)
                     new_members.append(m)
                 else:
                     m.id = old_member_uniques[(m.address, m.protocol_port)]
@@ -348,6 +377,16 @@ class MembersController(MemberController):
                 if (m.ip_address, m.protocol_port) not in new_member_uniques:
                     deleted_members.append(m)
 
+            if additive_only:
+                member_count_diff = len(new_members)
+            else:
+                member_count_diff = len(new_members) - len(deleted_members)
+            if member_count_diff > 0 and self.repositories.check_quota_met(
+                    context.session, lock_session, data_models.Member,
+                    db_pool.project_id, count=member_count_diff):
+                raise exceptions.QuotaException(
+                    resource=data_models.Member._name())
+
             provider_members = []
             # Create new members
             for m in new_members:
@@ -360,20 +399,34 @@ class MembersController(MemberController):
             # Update old members
             for m in updated_members:
                 m.provisioning_status = constants.PENDING_UPDATE
+                m.project_id = db_pool.project_id
                 db_member_dict = m.to_dict(render_unsets=False)
                 db_member_dict.pop('id')
                 self.repositories.member.update(
                     lock_session, m.id, **db_member_dict)
+
+                m.pool_id = self.pool_id
                 provider_members.append(
                     driver_utils.db_member_to_provider_member(m))
             # Delete old members
             for m in deleted_members:
-                self.repositories.member.update(
-                    lock_session, m.id,
-                    provisioning_status=constants.PENDING_DELETE)
+                if additive_only:
+                    # Members are appended to the dict and their status remains
+                    # unchanged, because they are logically "untouched".
+                    db_member_dict = m.to_dict(render_unsets=False)
+                    db_member_dict.pop('id')
+                    m.pool_id = self.pool_id
+                    provider_members.append(
+                        driver_utils.db_member_to_provider_member(m))
+                else:
+                    # Members are changed to PENDING_DELETE and not passed.
+                    self.repositories.member.update(
+                        lock_session, m.id,
+                        provisioning_status=constants.PENDING_DELETE)
 
             # Dispatch to the driver
             LOG.info("Sending Pool %s batch member update to provider %s",
                      db_pool.id, driver.name)
             driver_utils.call_provider(
-                driver.name, driver.member_batch_update, provider_members)
+                driver.name, driver.member_batch_update, db_pool.id,
+                provider_members)
