@@ -13,15 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from octavia_lib.api.drivers import data_models as driver_dm
+from octavia_lib.common import constants as lib_consts
 from oslo_config import cfg
 from oslo_db import exception as odb_exceptions
 from oslo_log import log as logging
 from oslo_utils import excutils
-import pecan
+from pecan import expose as pecan_expose
+from pecan import request as pecan_request
 from wsme import types as wtypes
 from wsmeext import pecan as wsme_pecan
 
-from octavia.api.drivers import data_models as driver_dm
 from octavia.api.drivers import driver_factory
 from octavia.api.drivers import utils as driver_utils
 from octavia.api.v2.controllers import base
@@ -45,13 +47,13 @@ class PoolsController(base.BaseController):
     RBAC_TYPE = constants.RBAC_POOL
 
     def __init__(self):
-        super(PoolsController, self).__init__()
+        super().__init__()
 
     @wsme_pecan.wsexpose(pool_types.PoolRootResponse, wtypes.text,
                          [wtypes.text], ignore_extra_args=True)
     def get(self, id, fields=None):
         """Gets a pool's details."""
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         db_pool = self._get_db_pool(context.session, id, show_deleted=False)
 
         self._auth_validate_action(context, db_pool.project_id,
@@ -66,12 +68,12 @@ class PoolsController(base.BaseController):
                          [wtypes.text], ignore_extra_args=True)
     def get_all(self, project_id=None, fields=None):
         """Lists all pools."""
-        pcontext = pecan.request.context
+        pcontext = pecan_request.context
         context = pcontext.get('octavia_context')
 
         query_filter = self._auth_get_all(context, project_id)
 
-        db_pools, links = self.repositories.pool.get_all(
+        db_pools, links = self.repositories.pool.get_all_API_list(
             context.session, show_deleted=False,
             pagination_helper=pcontext.get(constants.PAGINATION_HELPER),
             **query_filter)
@@ -82,7 +84,7 @@ class PoolsController(base.BaseController):
 
     def _get_affected_listener_ids(self, pool):
         """Gets a list of all listeners this request potentially affects."""
-        listener_ids = [l.id for l in pool.listeners]
+        listener_ids = [li.id for li in pool.listeners]
         return listener_ids
 
     def _test_lb_and_listener_statuses(self, session, lb_id, listener_ids):
@@ -98,53 +100,73 @@ class PoolsController(base.BaseController):
             raise exceptions.ImmutableObject(resource=_('Load Balancer'),
                                              id=lb_id)
 
-    def _reset_lb_and_listener_statuses(self, session, lb_id, listener_ids):
-        # Setting LB + listeners back to active because this should be a
-        # recoverable error
-        self.repositories.load_balancer.update(
-            session, lb_id,
-            provisioning_status=constants.ACTIVE)
-        for listener in listener_ids:
-            self.repositories.listener.update(
-                session, listener,
-                provisioning_status=constants.ACTIVE)
-
     def _validate_create_pool(self, lock_session, pool_dict, listener_id=None):
         """Validate creating pool on load balancer.
 
         Update database for load balancer and (optional) listener based on
         provisioning status.
         """
+        # Make sure we have a client CA if they specify a CRL
+        if (pool_dict.get('crl_container_id') and
+                not pool_dict.get('ca_tls_certificate_id')):
+            raise exceptions.ValidationException(detail=_(
+                "A CA certificate reference is required to "
+                "specify a revocation list."))
+
+        tls_certificate_id = pool_dict.get('tls_certificate_id', None)
+        tls_refs = [tls_certificate_id] if tls_certificate_id else []
+        self._validate_tls_refs(tls_refs)
+
+        # Validate the client CA cert and optional client CRL
+        if pool_dict.get('ca_tls_certificate_id'):
+            self._validate_client_ca_and_crl_refs(
+                pool_dict.get('ca_tls_certificate_id'),
+                pool_dict.get('crl_container_id', None))
+
+        # Check TLS cipher prohibit list
+        if 'tls_ciphers' in pool_dict and pool_dict['tls_ciphers']:
+            rejected_ciphers = validate.check_cipher_prohibit_list(
+                pool_dict['tls_ciphers'])
+            if rejected_ciphers:
+                raise exceptions.ValidationException(detail=_(
+                    'The following ciphers have been prohibited by an '
+                    'administrator: ' + ', '.join(rejected_ciphers)))
+
+        if pool_dict['tls_enabled']:
+            # Validate TLS version list
+            validate.check_tls_version_list(pool_dict['tls_versions'])
+            # Validate TLS versions against minimum
+            validate.check_tls_version_min(pool_dict['tls_versions'])
+
         try:
             return self.repositories.create_pool_on_load_balancer(
                 lock_session, pool_dict,
                 listener_id=listener_id)
-        except odb_exceptions.DBDuplicateEntry:
-            raise exceptions.IDAlreadyExists()
-        except odb_exceptions.DBError:
+        except odb_exceptions.DBDuplicateEntry as e:
+            raise exceptions.IDAlreadyExists() from e
+        except odb_exceptions.DBError as e:
             # TODO(blogan): will have to do separate validation protocol
             # before creation or update since the exception messages
             # do not give any information as to what constraint failed
-            raise exceptions.InvalidOption(value='', option='')
+            raise exceptions.InvalidOption(value='', option='') from e
 
     def _is_only_specified_in_request(self, request, **kwargs):
         request_attrs = []
         check_attrs = kwargs['check_exist_attrs']
-        escaped_attrs = ['from_data_model',
+        escaped_attrs = ['from_data_model', 'translate_key_to_data_model',
                          'translate_dict_keys_to_data_model', 'to_dict']
 
         for attr in dir(request):
             if attr.startswith('_') or attr in escaped_attrs:
                 continue
-            else:
-                request_attrs.append(attr)
+            request_attrs.append(attr)
 
         for req_attr in request_attrs:
             if (getattr(request, req_attr) and req_attr not in check_attrs):
                 return False
         return True
 
-    def _validate_pool_request_for_udp(self, request):
+    def _validate_pool_request_for_udp_sctp(self, request):
         if request.session_persistence:
             if (request.session_persistence.type ==
                     constants.SESSION_PERSISTENCE_SOURCE_IP and
@@ -153,22 +175,24 @@ class PoolsController(base.BaseController):
                         check_exist_attrs=['type', 'persistence_timeout',
                                            'persistence_granularity'])):
                 raise exceptions.ValidationException(detail=_(
-                    "session_persistence %s type for UDP protocol "
+                    "session_persistence %s type for UDP and SCTP protocols "
                     "only accepts: type, persistence_timeout, "
                     "persistence_granularity.") % (
                         constants.SESSION_PERSISTENCE_SOURCE_IP))
-            elif request.session_persistence.cookie_name:
+            if request.session_persistence.cookie_name:
                 raise exceptions.ValidationException(detail=_(
                     "Cookie names are not supported for %s pools.") %
-                    constants.PROTOCOL_UDP)
-            elif request.session_persistence.type in [
+                    "/".join((constants.PROTOCOL_UDP,
+                              lib_consts.PROTOCOL_SCTP)))
+            if request.session_persistence.type in [
                 constants.SESSION_PERSISTENCE_HTTP_COOKIE,
                     constants.SESSION_PERSISTENCE_APP_COOKIE]:
                 raise exceptions.ValidationException(detail=_(
                     "Session persistence of type %(type)s is not supported "
                     "for %(protocol)s protocol pools.") % {
                     'type': request.session_persistence.type,
-                    'protocol': constants.PROTOCOL_UDP})
+                    'protocol': "/".join((constants.PROTOCOL_UDP,
+                                          lib_consts.PROTOCOL_SCTP))})
 
     @wsme_pecan.wsexpose(pool_types.PoolRootResponse,
                          body=pool_types.PoolRootPOST, status_code=201)
@@ -183,16 +207,8 @@ class PoolsController(base.BaseController):
         # For some API requests the listener_id will be passed in the
         # pool_dict:
         pool = pool_.pool
-        context = pecan.request.context.get('octavia_context')
-        if pool.protocol == constants.PROTOCOL_UDP:
-            self._validate_pool_request_for_udp(pool)
-        else:
-            if (pool.session_persistence and (
-                    pool.session_persistence.persistence_timeout or
-                    pool.session_persistence.persistence_granularity)):
-                raise exceptions.ValidationException(detail=_(
-                    "persistence_timeout and persistence_granularity "
-                    "is only for UDP protocol pools."))
+        context = pecan_request.context.get('octavia_context')
+        listener = None
         if pool.loadbalancer_id:
             pool.project_id, provider = self._get_lb_project_id_provider(
                 context.session, pool.loadbalancer_id)
@@ -210,6 +226,20 @@ class PoolsController(base.BaseController):
         self._auth_validate_action(context, pool.project_id,
                                    constants.RBAC_POST)
 
+        if pool.listener_id and listener:
+            self._validate_protocol(listener.protocol, pool.protocol)
+
+        if pool.protocol in (constants.PROTOCOL_UDP,
+                             lib_consts.PROTOCOL_SCTP):
+            self._validate_pool_request_for_udp_sctp(pool)
+        else:
+            if (pool.session_persistence and (
+                    pool.session_persistence.persistence_timeout or
+                    pool.session_persistence.persistence_granularity)):
+                raise exceptions.ValidationException(detail=_(
+                    "persistence_timeout and persistence_granularity "
+                    "is only for UDP and SCTP protocol pools."))
+
         if pool.session_persistence:
             sp_dict = pool.session_persistence.to_dict(render_unsets=False)
             validate.check_session_persistence(sp_dict)
@@ -280,12 +310,25 @@ class PoolsController(base.BaseController):
                 resource=data_models.HealthMonitor._name())
 
         # Now possibly create a healthmonitor
-        new_hm = None
         if hm:
-            hm['pool_id'] = db_pool.id
-            hm['project_id'] = db_pool.project_id
+            hm[constants.POOL_ID] = db_pool.id
+            hm[constants.PROJECT_ID] = db_pool.project_id
             new_hm = health_monitor.HealthMonitorController()._graph_create(
                 lock_session, hm)
+            if db_pool.protocol in (constants.PROTOCOL_UDP,
+                                    lib_consts.PROTOCOL_SCTP):
+                health_monitor.HealthMonitorController(
+                )._validate_healthmonitor_request_for_udp_sctp(new_hm,
+                                                               db_pool)
+            else:
+                if new_hm.type in (constants.HEALTH_MONITOR_UDP_CONNECT,
+                                   lib_consts.HEALTH_MONITOR_SCTP):
+                    raise exceptions.ValidationException(detail=_(
+                        "The %(type)s type is only supported for pools of "
+                        "type %(protocol)s.") % {
+                            'type': new_hm.type,
+                            'protocol': '/'.join((constants.PROTOCOL_UDP,
+                                                  lib_consts.PROTOCOL_SCTP))})
             db_pool.health_monitor = new_hm
 
         # Now check quotas for members
@@ -298,6 +341,8 @@ class PoolsController(base.BaseController):
         # Now create members
         new_members = []
         for m in members:
+            validate.ip_not_reserved(m["ip_address"])
+
             m['project_id'] = db_pool.project_id
             new_members.append(
                 member.MembersController(db_pool.id)._graph_create(
@@ -305,25 +350,11 @@ class PoolsController(base.BaseController):
         db_pool.members = new_members
         return db_pool
 
-    @wsme_pecan.wsexpose(pool_types.PoolRootResponse, wtypes.text,
-                         body=pool_types.PoolRootPut, status_code=200)
-    def put(self, id, pool_):
-        """Updates a pool on a load balancer."""
-        pool = pool_.pool
-        context = pecan.request.context.get('octavia_context')
-        db_pool = self._get_db_pool(context.session, id, show_deleted=False)
+    def _validate_pool_PUT(self, pool, db_pool):
 
-        project_id, provider = self._get_lb_project_id_provider(
-            context.session, db_pool.load_balancer_id)
-
-        if (pool.session_persistence and
-                not pool.session_persistence.type and
-                db_pool.session_persistence and
-                db_pool.session_persistence.type):
-            pool.session_persistence.type = db_pool.session_persistence.type
-        self._auth_validate_action(context, project_id, constants.RBAC_PUT)
-        if db_pool.protocol == constants.PROTOCOL_UDP:
-            self._validate_pool_request_for_udp(pool)
+        if db_pool.protocol in (constants.PROTOCOL_UDP,
+                                lib_consts.PROTOCOL_SCTP):
+            self._validate_pool_request_for_udp_sctp(pool)
         else:
             if (pool.session_persistence and (
                     pool.session_persistence.persistence_timeout or
@@ -336,6 +367,80 @@ class PoolsController(base.BaseController):
             sp_dict = pool.session_persistence.to_dict(render_unsets=False)
             validate.check_session_persistence(sp_dict)
 
+        crl_ref = None
+        # If we got a crl_ref and it's not unset, use it
+        if (pool.crl_container_ref and
+                pool.crl_container_ref != wtypes.Unset):
+            crl_ref = pool.crl_container_ref
+        # If we got Unset and a CRL exists in the DB, use the DB crl_ref
+        elif (db_pool.crl_container_id and
+              pool.crl_container_ref == wtypes.Unset):
+            crl_ref = db_pool.crl_container_id
+
+        ca_ref = None
+        db_ca_ref = db_pool.ca_tls_certificate_id
+        if pool.ca_tls_container_ref != wtypes.Unset:
+            if not pool.ca_tls_container_ref and db_ca_ref and crl_ref:
+                raise exceptions.ValidationException(detail=_(
+                    "A CA reference cannot be removed when a "
+                    "certificate revocation list is present."))
+
+            if not pool.ca_tls_container_ref and not db_ca_ref and crl_ref:
+                raise exceptions.ValidationException(detail=_(
+                    "A CA reference is required to "
+                    "specify a certificate revocation list."))
+            if pool.ca_tls_container_ref:
+                ca_ref = pool.ca_tls_container_ref
+        elif db_ca_ref and pool.ca_tls_container_ref == wtypes.Unset:
+            ca_ref = db_ca_ref
+        elif crl_ref and not db_ca_ref:
+            raise exceptions.ValidationException(detail=_(
+                "A CA reference is required to "
+                "specify a certificate revocation list."))
+
+        if pool.tls_container_ref:
+            self._validate_tls_refs([pool.tls_container_ref])
+
+        # Validate the client CA cert and optional client CRL
+        if ca_ref:
+            self._validate_client_ca_and_crl_refs(ca_ref, crl_ref)
+
+        # Check TLS cipher prohibit list
+        if pool.tls_ciphers:
+            rejected_ciphers = validate.check_cipher_prohibit_list(
+                pool.tls_ciphers)
+            if rejected_ciphers:
+                raise exceptions.ValidationException(detail=_(
+                    "The following ciphers have been prohibited by an "
+                    "administrator: " + ', '.join(rejected_ciphers)))
+
+        if pool.tls_versions is not wtypes.Unset:
+            # Validate TLS version list
+            validate.check_tls_version_list(pool.tls_versions)
+            # Validate TLS version against minimum
+            validate.check_tls_version_min(pool.tls_versions)
+
+    @wsme_pecan.wsexpose(pool_types.PoolRootResponse, wtypes.text,
+                         body=pool_types.PoolRootPut, status_code=200)
+    def put(self, id, pool_):
+        """Updates a pool on a load balancer."""
+        pool = pool_.pool
+        context = pecan_request.context.get('octavia_context')
+        db_pool = self._get_db_pool(context.session, id, show_deleted=False)
+
+        project_id, provider = self._get_lb_project_id_provider(
+            context.session, db_pool.load_balancer_id)
+
+        self._auth_validate_action(context, project_id, constants.RBAC_PUT)
+
+        if (pool.session_persistence and
+                not pool.session_persistence.type and
+                db_pool.session_persistence and
+                db_pool.session_persistence.type):
+            pool.session_persistence.type = db_pool.session_persistence.type
+
+        self._validate_pool_PUT(pool, db_pool)
+
         # Load the driver early as it also provides validation
         driver = driver_factory.get_driver(provider)
 
@@ -352,7 +457,7 @@ class PoolsController(base.BaseController):
 
             # Also prepare the baseline object data
             old_provider_pool = driver_utils.db_pool_to_provider_pool(
-                db_pool)
+                db_pool, for_delete=True)
 
             # Dispatch to the driver
             LOG.info("Sending update Pool %s to provider %s", id, driver.name)
@@ -377,17 +482,18 @@ class PoolsController(base.BaseController):
     @wsme_pecan.wsexpose(None, wtypes.text, status_code=204)
     def delete(self, id):
         """Deletes a pool from a load balancer."""
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         db_pool = self._get_db_pool(context.session, id, show_deleted=False)
-        if db_pool.l7policies:
-            raise exceptions.PoolInUseByL7Policy(
-                id=db_pool.id, l7policy_id=db_pool.l7policies[0].id)
 
         project_id, provider = self._get_lb_project_id_provider(
             context.session, db_pool.load_balancer_id)
 
         self._auth_validate_action(context, project_id, constants.RBAC_DELETE)
 
+        if db_pool.l7policies:
+            raise exceptions.PoolInUseByL7Policy(
+                id=db_pool.id, l7policy_id=db_pool.l7policies[0].id)
+
         # Load the driver early as it also provides validation
         driver = driver_factory.get_driver(provider)
 
@@ -401,18 +507,19 @@ class PoolsController(base.BaseController):
 
             LOG.info("Sending delete Pool %s to provider %s", id, driver.name)
             provider_pool = (
-                driver_utils.db_pool_to_provider_pool(db_pool))
+                driver_utils.db_pool_to_provider_pool(db_pool,
+                                                      for_delete=True))
             driver_utils.call_provider(driver.name, driver.pool_delete,
                                        provider_pool)
 
-    @pecan.expose()
+    @pecan_expose()
     def _lookup(self, pool_id, *remainder):
         """Overridden pecan _lookup method for custom routing.
 
         Verifies that the pool passed in the url exists, and if so decides
         which controller, if any, should control be passed.
         """
-        context = pecan.request.context.get('octavia_context')
+        context = pecan_request.context.get('octavia_context')
         if pool_id and remainder and remainder[0] == 'members':
             remainder = remainder[1:]
             db_pool = self.repositories.pool.get(context.session, id=pool_id)
