@@ -14,13 +14,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import datetime
 import re
 
-import six
+from oslo_log import log as logging
 from sqlalchemy.orm import collections
 
 from octavia.common import constants
 
+LOG = logging.getLogger(__name__)
+
 
 class BaseDataModel(object):
     def to_dict(self, calling_classes=None, recurse=False, **kwargs):
@@ -31,7 +34,18 @@ class BaseDataModel(object):
             if attr.startswith('_') or not kwargs.get(attr, True):
                 continue
             value = self.__dict__[attr]
-
+            if attr == 'tags':
+                # tags is a list, it doesn't need recurse
+                ret[attr] = value
+                continue
+            # We need to have json convertible data for storing it in
+            # persistence jobboard backend.
+            if isinstance(value, datetime.datetime):
+                ret[attr] = value.isoformat()
+                continue
+            if isinstance(value, bytes):
+                ret[attr] = value.decode()
+                continue
             if recurse:
                 if isinstance(getattr(self, attr), list):
                     ret[attr] = []
@@ -40,24 +54,32 @@ class BaseDataModel(object):
                             if type(self) not in calling_classes:
                                 ret[attr].append(
                                     item.to_dict(calling_classes=(
-                                        calling_classes + [type(self)])))
+                                        calling_classes + [type(self)]),
+                                        recurse=recurse))
                             else:
+                                # TODO(rm_work): Is the idea that if this list
+                                #  contains ANY BaseDataModel, that all of them
+                                #  are data models, and we may as well quit?
+                                #  Or, were we supposed to append a `None` for
+                                #  each one? I assume the former?
                                 ret[attr] = None
+                                break
                         else:
-                            ret[attr] = item
+                            ret[attr].append(item)
                 elif isinstance(getattr(self, attr), BaseDataModel):
                     if type(self) not in calling_classes:
                         ret[attr] = value.to_dict(
-                            calling_classes=calling_classes + [type(self)])
+                            calling_classes=calling_classes + [type(self)],
+                            recurse=recurse)
                     else:
                         ret[attr] = None
-                elif six.PY2 and isinstance(value, six.text_type):
-                    ret[attr.encode('utf8')] = value.encode('utf8')
                 else:
                     ret[attr] = value
             else:
-                if isinstance(getattr(self, attr), (BaseDataModel, list)):
+                if isinstance(getattr(self, attr), BaseDataModel):
                     ret[attr] = None
+                elif isinstance(getattr(self, attr), list):
+                    ret[attr] = []
                 else:
                     ret[attr] = value
 
@@ -90,19 +112,20 @@ class BaseDataModel(object):
                                       'Listener', 'Amphora', 'L7Policy',
                                       'L7Rule']:
             return obj.__class__.__name__ + obj.id
-        elif obj.__class__.__name__ in ['SessionPersistence', 'HealthMonitor']:
+        if obj.__class__.__name__ in ['SessionPersistence', 'HealthMonitor']:
             return obj.__class__.__name__ + obj.pool_id
-        elif obj.__class__.__name__ in ['ListenerStatistics']:
+        if obj.__class__.__name__ in ['ListenerStatistics']:
             return obj.__class__.__name__ + obj.listener_id + obj.amphora_id
-        elif obj.__class__.__name__ in ['VRRPGroup', 'Vip']:
+        if obj.__class__.__name__ in ['ListenerCidr']:
+            return obj.__class__.__name__ + obj.listener_id + obj.cidr
+        if obj.__class__.__name__ in ['VRRPGroup', 'Vip']:
             return obj.__class__.__name__ + obj.load_balancer_id
-        elif obj.__class__.__name__ in ['AmphoraHealth']:
+        if obj.__class__.__name__ in ['AmphoraHealth']:
             return obj.__class__.__name__ + obj.amphora_id
-        elif obj.__class__.__name__ in ['SNI']:
+        if obj.__class__.__name__ in ['SNI']:
             return (obj.__class__.__name__ +
                     obj.listener_id + obj.tls_container_id)
-        else:
-            raise NotImplementedError
+        raise NotImplementedError
 
     def _find_in_graph(self, key, _visited_nodes=None):
         """Locates an object with the given unique key in the current
@@ -114,26 +137,25 @@ class BaseDataModel(object):
         if mykey in _visited_nodes:
             # Seen this node already, don't traverse further
             return None
-        elif mykey == key:
+        if mykey == key:
             return self
-        else:
-            _visited_nodes.append(mykey)
-            attr_names = [attr_name for attr_name in dir(self)
-                          if not attr_name.startswith('_')]
-            for attr_name in attr_names:
-                attr = getattr(self, attr_name)
-                if isinstance(attr, BaseDataModel):
-                    result = attr._find_in_graph(
-                        key, _visited_nodes=_visited_nodes)
-                    if result is not None:
-                        return result
-                elif isinstance(attr, (collections.InstrumentedList, list)):
-                    for item in attr:
-                        if isinstance(item, BaseDataModel):
-                            result = item._find_in_graph(
-                                key, _visited_nodes=_visited_nodes)
-                            if result is not None:
-                                return result
+        _visited_nodes.append(mykey)
+        attr_names = [attr_name for attr_name in dir(self)
+                      if not attr_name.startswith('_')]
+        for attr_name in attr_names:
+            attr = getattr(self, attr_name)
+            if isinstance(attr, BaseDataModel):
+                result = attr._find_in_graph(
+                    key, _visited_nodes=_visited_nodes)
+                if result is not None:
+                    return result
+            elif isinstance(attr, (collections.InstrumentedList, list)):
+                for item in attr:
+                    if isinstance(item, BaseDataModel):
+                        result = item._find_in_graph(
+                            key, _visited_nodes=_visited_nodes)
+                        if result is not None:
+                            return result
         # If we are here we didn't find it.
         return None
 
@@ -166,7 +188,7 @@ class ListenerStatistics(BaseDataModel):
 
     def __init__(self, listener_id=None, amphora_id=None, bytes_in=0,
                  bytes_out=0, active_connections=0,
-                 total_connections=0, request_errors=0):
+                 total_connections=0, request_errors=0, received_time=0.0):
         self.listener_id = listener_id
         self.amphora_id = amphora_id
         self.bytes_in = bytes_in
@@ -174,6 +196,7 @@ class ListenerStatistics(BaseDataModel):
         self.active_connections = active_connections
         self.total_connections = total_connections
         self.request_errors = request_errors
+        self.received_time = received_time
 
     def get_stats(self):
         stats = {
@@ -185,13 +208,21 @@ class ListenerStatistics(BaseDataModel):
         }
         return stats
 
-    def __iadd__(self, other):
+    def db_fields(self):
+        fields = self.to_dict()
+        fields.pop('received_time')
+        return fields
 
+    def __iadd__(self, other):
         if isinstance(other, ListenerStatistics):
             self.bytes_in += other.bytes_in
             self.bytes_out += other.bytes_out
             self.request_errors += other.request_errors
             self.total_connections += other.total_connections
+        else:
+            raise TypeError(  # noqa: O342
+                "unsupported operand type(s) for +=: '{0}' and '{1}'".format(
+                    type(self), type(other)))
 
         return self
 
@@ -225,7 +256,8 @@ class HealthMonitor(BaseDataModel):
                  rise_threshold=None, http_method=None, url_path=None,
                  expected_codes=None, enabled=None, pool=None, name=None,
                  provisioning_status=None, operating_status=None,
-                 created_at=None, updated_at=None):
+                 created_at=None, updated_at=None, tags=None,
+                 http_version=None, domain_name=None):
         self.id = id
         self.project_id = project_id
         self.pool_id = pool_id
@@ -244,6 +276,9 @@ class HealthMonitor(BaseDataModel):
         self.name = name
         self.created_at = created_at
         self.updated_at = updated_at
+        self.tags = tags
+        self.http_version = http_version
+        self.domain_name = domain_name
 
     def delete(self):
         self.pool.health_monitor = None
@@ -255,7 +290,10 @@ class Pool(BaseDataModel):
                  operating_status=None, members=None, health_monitor=None,
                  session_persistence=None, load_balancer_id=None,
                  load_balancer=None, listeners=None, l7policies=None,
-                 created_at=None, updated_at=None, provisioning_status=None):
+                 created_at=None, updated_at=None, provisioning_status=None,
+                 tags=None, tls_certificate_id=None,
+                 ca_tls_certificate_id=None, crl_container_id=None,
+                 tls_enabled=None, tls_ciphers=None, tls_versions=None):
         self.id = id
         self.project_id = project_id
         self.name = name
@@ -274,6 +312,13 @@ class Pool(BaseDataModel):
         self.created_at = created_at
         self.updated_at = updated_at
         self.provisioning_status = provisioning_status
+        self.tags = tags
+        self.tls_certificate_id = tls_certificate_id
+        self.ca_tls_certificate_id = ca_tls_certificate_id
+        self.crl_container_id = crl_container_id
+        self.tls_enabled = tls_enabled
+        self.tls_ciphers = tls_ciphers
+        self.tls_versions = tls_versions
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -300,7 +345,11 @@ class Pool(BaseDataModel):
                     break
         for pool in self.load_balancer.pools:
             if pool.id == self.id:
-                self.load_balancer.pools.remove(pool)
+                try:
+                    self.load_balancer.pools.remove(pool)
+                except ValueError:
+                    LOG.debug("Pool %s has already been removed from load "
+                              "balancer pools list.", pool.id)
                 break
         for l7policy in self.l7policies:
             if l7policy.redirect_pool_id == self.id:
@@ -319,7 +368,8 @@ class Member(BaseDataModel):
                  protocol_port=None, weight=None, backup=None, enabled=None,
                  subnet_id=None, operating_status=None, pool=None,
                  created_at=None, updated_at=None, provisioning_status=None,
-                 name=None, monitor_address=None, monitor_port=None):
+                 name=None, monitor_address=None, monitor_port=None,
+                 tags=None):
         self.id = id
         self.project_id = project_id
         self.pool_id = pool_id
@@ -337,6 +387,7 @@ class Member(BaseDataModel):
         self.name = name
         self.monitor_address = monitor_address
         self.monitor_port = monitor_port
+        self.tags = tags
 
     def delete(self):
         for mem in self.pool.members:
@@ -356,7 +407,11 @@ class Listener(BaseDataModel):
                  l7policies=None, pools=None, insert_headers=None,
                  created_at=None, updated_at=None,
                  timeout_client_data=None, timeout_member_connect=None,
-                 timeout_member_data=None, timeout_tcp_inspect=None):
+                 timeout_member_data=None, timeout_tcp_inspect=None,
+                 tags=None, client_ca_tls_certificate_id=None,
+                 client_authentication=None, client_crl_container_id=None,
+                 allowed_cidrs=None, tls_ciphers=None, tls_versions=None,
+                 alpn_protocols=None):
         self.id = id
         self.project_id = project_id
         self.name = name
@@ -384,6 +439,14 @@ class Listener(BaseDataModel):
         self.timeout_member_connect = timeout_member_connect
         self.timeout_member_data = timeout_member_data
         self.timeout_tcp_inspect = timeout_tcp_inspect
+        self.tags = tags
+        self.client_ca_tls_certificate_id = client_ca_tls_certificate_id
+        self.client_authentication = client_authentication
+        self.client_crl_container_id = client_crl_container_id
+        self.allowed_cidrs = allowed_cidrs or []
+        self.tls_ciphers = tls_ciphers
+        self.tls_versions = tls_versions
+        self.alpn_protocols = alpn_protocols
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -424,7 +487,8 @@ class LoadBalancer(BaseDataModel):
                  provisioning_status=None, operating_status=None, enabled=None,
                  topology=None, vip=None, listeners=None, amphorae=None,
                  pools=None, vrrp_group=None, server_group_id=None,
-                 created_at=None, updated_at=None, provider=None):
+                 created_at=None, updated_at=None, provider=None, tags=None,
+                 flavor_id=None, availability_zone=None):
 
         self.id = id
         self.project_id = project_id
@@ -443,6 +507,9 @@ class LoadBalancer(BaseDataModel):
         self.created_at = created_at
         self.updated_at = updated_at
         self.provider = provider
+        self.tags = tags or []
+        self.flavor_id = flavor_id
+        self.availability_zone = availability_zone
 
     def update(self, update_dict):
         for key, value in update_dict.items():
@@ -494,6 +561,14 @@ class SNI(BaseDataModel):
         self.listener = listener
         self.tls_container_id = tls_container_id
 
+    # SQLAlchemy kindly attaches the whole listener object so
+    # let's keep this simple by overriding the to_dict for this
+    # object. Otherwise we recurse down the "ghost" listener object.
+    def to_dict(self, **kwargs):
+        return {'tls_container_id': self.tls_container_id,
+                'listener_id': self.listener_id,
+                'position': self.position}
+
 
 class TLSContainer(BaseDataModel):
 
@@ -515,7 +590,7 @@ class Amphora(BaseDataModel):
                  load_balancer=None, role=None, cert_expiration=None,
                  cert_busy=False, vrrp_interface=None, vrrp_id=None,
                  vrrp_priority=None, cached_zone=None, created_at=None,
-                 updated_at=None, image_id=None):
+                 updated_at=None, image_id=None, compute_flavor=None):
         self.id = id
         self.load_balancer_id = load_balancer_id
         self.compute_id = compute_id
@@ -536,6 +611,7 @@ class Amphora(BaseDataModel):
         self.created_at = created_at
         self.updated_at = updated_at
         self.image_id = image_id
+        self.compute_flavor = compute_flavor
 
     def delete(self):
         for amphora in self.load_balancer.amphorae:
@@ -557,7 +633,7 @@ class L7Rule(BaseDataModel):
     def __init__(self, id=None, l7policy_id=None, type=None, enabled=None,
                  compare_type=None, key=None, value=None, l7policy=None,
                  invert=False, provisioning_status=None, operating_status=None,
-                 project_id=None, created_at=None, updated_at=None):
+                 project_id=None, created_at=None, updated_at=None, tags=None):
         self.id = id
         self.l7policy_id = l7policy_id
         self.type = type
@@ -572,6 +648,7 @@ class L7Rule(BaseDataModel):
         self.created_at = created_at
         self.updated_at = updated_at
         self.enabled = enabled
+        self.tags = tags
 
     def delete(self):
         if len(self.l7policy.l7rules) == 1:
@@ -592,7 +669,8 @@ class L7Policy(BaseDataModel):
                  position=None, listener=None, redirect_pool=None,
                  enabled=None, l7rules=None, provisioning_status=None,
                  operating_status=None, project_id=None, created_at=None,
-                 updated_at=None):
+                 updated_at=None, redirect_prefix=None, tags=None,
+                 redirect_http_code=None):
         self.id = id
         self.name = name
         self.description = description
@@ -610,6 +688,9 @@ class L7Policy(BaseDataModel):
         self.project_id = project_id
         self.created_at = created_at
         self.updated_at = updated_at
+        self.redirect_prefix = redirect_prefix
+        self.tags = tags
+        self.redirect_http_code = redirect_http_code
 
     def _conditionally_remove_pool_links(self, pool):
         """Removes links to the given pool from parent objects.
@@ -636,6 +717,7 @@ class L7Policy(BaseDataModel):
                 self._conditionally_remove_pool_links(self.redirect_pool)
                 self.action = constants.L7POLICY_ACTION_REDIRECT_TO_POOL
                 self.redirect_url = None
+                self.redirect_http_code = None
                 pool = self._find_in_graph('Pool' + value)
                 self.redirect_pool = pool
                 if self.l7rules and (self.enabled is True or (
@@ -655,6 +737,7 @@ class L7Policy(BaseDataModel):
                 self._conditionally_remove_pool_links(self.redirect_pool)
                 self.redirect_pool = None
                 self.redirect_pool_id = None
+                self.redirect_http_code = None
             elif key == 'position':
                 self.listener.l7policies.remove(self)
                 self.listener.l7policies.insert(value - 1, self)
@@ -694,19 +777,82 @@ class Quotas(BaseDataModel):
                  pool=None,
                  health_monitor=None,
                  member=None,
+                 l7policy=None,
+                 l7rule=None,
                  in_use_health_monitor=None,
                  in_use_listener=None,
                  in_use_load_balancer=None,
                  in_use_member=None,
-                 in_use_pool=None):
+                 in_use_pool=None,
+                 in_use_l7policy=None,
+                 in_use_l7rule=None):
         self.project_id = project_id
         self.health_monitor = health_monitor
         self.listener = listener
         self.load_balancer = load_balancer
         self.pool = pool
         self.member = member
+        self.l7policy = l7policy
+        self.l7rule = l7rule
         self.in_use_health_monitor = in_use_health_monitor
         self.in_use_listener = in_use_listener
         self.in_use_load_balancer = in_use_load_balancer
         self.in_use_member = in_use_member
         self.in_use_pool = in_use_pool
+        self.in_use_l7policy = in_use_l7policy
+        self.in_use_l7rule = in_use_l7rule
+
+
+class Flavor(BaseDataModel):
+
+    def __init__(self, id=None, name=None,
+                 description=None, enabled=None,
+                 flavor_profile_id=None):
+        self.id = id
+        self.name = name
+        self.description = description
+        self.enabled = enabled
+        self.flavor_profile_id = flavor_profile_id
+
+
+class FlavorProfile(BaseDataModel):
+
+    def __init__(self, id=None, name=None, provider_name=None,
+                 flavor_data=None):
+        self.id = id
+        self.name = name
+        self.provider_name = provider_name
+        self.flavor_data = flavor_data
+
+
+class AvailabilityZone(BaseDataModel):
+
+    def __init__(self, name=None, description=None, enabled=None,
+                 availability_zone_profile_id=None):
+        self.name = name
+        self.description = description
+        self.enabled = enabled
+        self.availability_zone_profile_id = availability_zone_profile_id
+
+
+class AvailabilityZoneProfile(BaseDataModel):
+
+    def __init__(self, id=None, name=None, provider_name=None,
+                 availability_zone_data=None):
+        self.id = id
+        self.name = name
+        self.provider_name = provider_name
+        self.availability_zone_data = availability_zone_data
+
+
+class ListenerCidr(BaseDataModel):
+
+    def __init__(self, listener_id=None, cidr=None):
+        self.listener_id = listener_id
+        self.cidr = cidr
+
+    # SQLAlchemy kindly attaches the whole listener object so
+    # let's keep this simple by overriding the to_dict for this
+    # object. Otherwise we recurse down the "ghost" listener object.
+    def to_dict(self, **kwargs):
+        return {'cidr': self.cidr, 'listener_id': self.listener_id}
