@@ -16,7 +16,7 @@ import os
 import re
 
 import jinja2
-import six
+from octavia_lib.common import constants as lib_consts
 
 from octavia.common.config import cfg
 from octavia.common import constants
@@ -81,9 +81,8 @@ class JinjaTemplater(object):
         self.log_server = log_server
         self.connection_logging = connection_logging
 
-    def build_config(self, host_amphora, listeners, haproxy_versions,
-                     socket_path=None, client_ca_filename=None,
-                     client_crl=None, pool_tls_certs=None):
+    def build_config(self, host_amphora, listeners, tls_certs,
+                     haproxy_versions, socket_path=None):
         """Convert a logical configuration to the HAProxy version
 
         :param host_amphora: The Amphora this configuration is hosted on
@@ -102,10 +101,9 @@ class JinjaTemplater(object):
             feature_compatibility[constants.HTTP_REUSE] = True
 
         return self.render_loadbalancer_obj(
-            host_amphora, listeners, socket_path=socket_path,
-            feature_compatibility=feature_compatibility,
-            client_ca_filename=client_ca_filename, client_crl=client_crl,
-            pool_tls_certs=pool_tls_certs)
+            host_amphora, listeners, tls_certs=tls_certs,
+            socket_path=socket_path,
+            feature_compatibility=feature_compatibility)
 
     def _get_template(self):
         """Returns the specified Jinja configuration template."""
@@ -123,8 +121,8 @@ class JinjaTemplater(object):
 
     def _format_log_string(self, load_balancer, protocol):
         log_format = CONF.haproxy_amphora.user_log_format.replace(
-            '{project_id}', load_balancer.project_id)
-        log_format = log_format.replace('{lb_id}', load_balancer.id)
+            '{{ project_id }}', load_balancer.project_id)
+        log_format = log_format.replace('{{ lb_id }}', load_balancer.id)
 
         # Order of these filters matter.
         # TODO(johnsom) Remove when HAProxy handles the format string
@@ -141,14 +139,13 @@ class JinjaTemplater(object):
         return log_format
 
     def render_loadbalancer_obj(self, host_amphora, listeners,
-                                socket_path=None, feature_compatibility=None,
-                                client_ca_filename=None, client_crl=None,
-                                pool_tls_certs=None):
+                                tls_certs=None, socket_path=None,
+                                feature_compatibility=None):
         """Renders a templated configuration from a load balancer object
 
         :param host_amphora: The Amphora this configuration is hosted on
         :param listener: The listener configuration
-        :param client_ca_filename: The CA certificate for client authorization
+        :param tls_certs: Dict of the TLS certificates for the listener
         :param socket_path: The socket path for Haproxy process
         :return: Rendered configuration
         """
@@ -157,10 +154,8 @@ class JinjaTemplater(object):
             host_amphora,
             listeners[0].load_balancer,
             listeners,
-            feature_compatibility,
-            client_ca_filename=client_ca_filename,
-            client_crl=client_crl,
-            pool_tls_certs=pool_tls_certs)
+            tls_certs,
+            feature_compatibility,)
         if not socket_path:
             socket_path = '%s/%s.sock' % (self.base_amp_path,
                                           listeners[0].load_balancer.id)
@@ -173,11 +168,10 @@ class JinjaTemplater(object):
                  CONF.amphora_agent.administrative_log_facility,
              'user_log_facility': CONF.amphora_agent.user_log_facility,
              'connection_logging': self.connection_logging},
-            constants=constants)
+            constants=constants, lib_consts=lib_consts)
 
     def _transform_loadbalancer(self, host_amphora, loadbalancer, listeners,
-                                feature_compatibility, client_ca_filename=None,
-                                client_crl=None, pool_tls_certs=None):
+                                tls_certs, feature_compatibility):
         """Transforms a load balancer into an object that will
 
            be processed by the templating system
@@ -187,9 +181,7 @@ class JinjaTemplater(object):
             if listener.protocol == constants.PROTOCOL_UDP:
                 continue
             listener_transforms.append(self._transform_listener(
-                listener, feature_compatibility, loadbalancer,
-                client_ca_filename=client_ca_filename, client_crl=client_crl,
-                pool_tls_certs=pool_tls_certs))
+                listener, tls_certs, feature_compatibility, loadbalancer))
 
         ret_value = {
             'id': loadbalancer.id,
@@ -211,8 +203,8 @@ class JinjaTemplater(object):
             if listener.connection_limit and listener.connection_limit > -1:
                 connection_limit_sum += listener.connection_limit
             else:
-                # If *any* listener has no connection limit, global = MAX
-                connection_limit_sum = constants.HAPROXY_MAX_MAXCONN
+                connection_limit_sum += (
+                    CONF.haproxy_amphora.default_connection_limit)
         # If there's a limit between 0 and MAX, set it, otherwise just set MAX
         if 0 < connection_limit_sum < constants.HAPROXY_MAX_MAXCONN:
             ret_value['global_connection_limit'] = connection_limit_sum
@@ -239,9 +231,8 @@ class JinjaTemplater(object):
             'vrrp_priority': amphora.vrrp_priority
         }
 
-    def _transform_listener(self, listener, feature_compatibility,
-                            loadbalancer, client_ca_filename=None,
-                            client_crl=None, pool_tls_certs=None):
+    def _transform_listener(self, listener, tls_certs, feature_compatibility,
+                            loadbalancer):
         """Transforms a listener into an object that will
 
             be processed by the templating system
@@ -271,31 +262,50 @@ class JinjaTemplater(object):
         if listener.connection_limit and listener.connection_limit > -1:
             ret_value['connection_limit'] = listener.connection_limit
         else:
-            ret_value['connection_limit'] = constants.HAPROXY_MAX_MAXCONN
+            ret_value['connection_limit'] = (
+                CONF.haproxy_amphora.default_connection_limit)
 
         if listener.tls_certificate_id:
             ret_value['crt_list_filename'] = os.path.join(
                 CONF.haproxy_amphora.base_cert_dir,
                 loadbalancer.id, '{}.pem'.format(listener.id))
 
-        if listener.client_ca_tls_certificate_id:
-            ret_value['client_ca_tls_path'] = '%s' % (
-                os.path.join(self.base_crt_dir, loadbalancer.id,
-                             client_ca_filename))
-            ret_value['client_auth'] = CLIENT_AUTH_MAP.get(
-                listener.client_authentication)
-        if listener.client_crl_container_id:
-            ret_value['client_crl_path'] = '%s' % (
-                os.path.join(self.base_crt_dir, loadbalancer.id, client_crl))
+        if tls_certs is not None:
+            if listener.client_ca_tls_certificate_id:
+                ret_value['client_ca_tls_path'] = '%s' % (
+                    os.path.join(
+                        self.base_crt_dir, loadbalancer.id,
+                        tls_certs[listener.client_ca_tls_certificate_id]))
+                ret_value['client_auth'] = CLIENT_AUTH_MAP.get(
+                    listener.client_authentication)
+
+            if listener.client_crl_container_id:
+                ret_value['client_crl_path'] = '%s' % (
+                    os.path.join(self.base_crt_dir, loadbalancer.id,
+                                 tls_certs[listener.client_crl_container_id]))
+
+        tls_enabled = False
+        if listener.protocol == constants.PROTOCOL_TERMINATED_HTTPS:
+            tls_enabled = True
+            if listener.tls_ciphers is not None:
+                ret_value['tls_ciphers'] = listener.tls_ciphers
+            if listener.tls_versions is not None:
+                ret_value['tls_versions'] = listener.tls_versions
+            if listener.alpn_protocols is not None:
+                ret_value['alpn_protocols'] = ",".join(listener.alpn_protocols)
 
         pools = []
-        for x in listener.pools:
+        pool_gen = (pool for pool in listener.pools if
+                    pool.provisioning_status != constants.PENDING_DELETE)
+        for pool in pool_gen:
             kwargs = {}
-            if pool_tls_certs and pool_tls_certs.get(x.id):
-                kwargs = {'pool_tls_certs': pool_tls_certs.get(x.id)}
+            if tls_certs is not None and tls_certs.get(pool.id):
+                kwargs = {'pool_tls_certs': tls_certs.get(pool.id)}
             pools.append(self._transform_pool(
-                x, feature_compatibility, **kwargs))
+                pool, feature_compatibility, tls_enabled, **kwargs))
         ret_value['pools'] = pools
+        policy_gen = (policy for policy in listener.l7policies if
+                      policy.provisioning_status != constants.PENDING_DELETE)
         if listener.default_pool:
             for pool in pools:
                 if pool['id'] == listener.default_pool.id:
@@ -303,21 +313,27 @@ class JinjaTemplater(object):
                     break
 
         l7policies = [self._transform_l7policy(
-                      x, feature_compatibility, pool_tls_certs)
-                      for x in listener.l7policies]
+                      x, feature_compatibility, tls_enabled, tls_certs)
+                      for x in policy_gen]
         ret_value['l7policies'] = l7policies
         return ret_value
 
     def _transform_pool(self, pool, feature_compatibility,
-                        pool_tls_certs=None):
+                        listener_tls_enabled, pool_tls_certs=None):
         """Transforms a pool into an object that will
 
             be processed by the templating system
         """
+        proxy_protocol_version = None
+        if pool.protocol == constants.PROTOCOL_PROXY:
+            proxy_protocol_version = 1
+        if pool.protocol == lib_consts.PROTOCOL_PROXYV2:
+            proxy_protocol_version = 2
         ret_value = {
             'id': pool.id,
             'protocol': PROTOCOL_MAP[pool.protocol],
-            'proxy_protocol': pool.protocol == constants.PROTOCOL_PROXY,
+            'proxy_protocol': proxy_protocol_version,
+            'listener_tls_enabled': listener_tls_enabled,
             'lb_algorithm': BALANCE_MAP.get(pool.lb_algorithm, 'roundrobin'),
             'members': [],
             'health_monitor': '',
@@ -331,12 +347,16 @@ class JinjaTemplater(object):
             'crl_path': '',
             'tls_enabled': pool.tls_enabled
         }
+        members_gen = (mem for mem in pool.members if
+                       mem.provisioning_status != constants.PENDING_DELETE)
         members = [self._transform_member(x, feature_compatibility)
-                   for x in pool.members]
+                   for x in members_gen]
         ret_value['members'] = members
-        if pool.health_monitor:
+        health_mon = pool.health_monitor
+        if (health_mon and
+                health_mon.provisioning_status != constants.PENDING_DELETE):
             ret_value['health_monitor'] = self._transform_health_monitor(
-                pool.health_monitor, feature_compatibility)
+                health_mon, feature_compatibility)
         if pool.session_persistence:
             ret_value[
                 'session_persistence'] = self._transform_session_persistence(
@@ -344,6 +364,11 @@ class JinjaTemplater(object):
         if (pool.tls_certificate_id and pool_tls_certs and
                 pool_tls_certs.get('client_cert')):
             ret_value['client_cert'] = pool_tls_certs.get('client_cert')
+        if pool.tls_enabled is True:
+            if pool.tls_ciphers is not None:
+                ret_value['tls_ciphers'] = pool.tls_ciphers
+            if pool.tls_versions is not None:
+                ret_value['tls_versions'] = pool.tls_versions
         if (pool.ca_tls_certificate_id and pool_tls_certs and
                 pool_tls_certs.get('ca_cert')):
             ret_value['ca_cert'] = pool_tls_certs.get('ca_cert')
@@ -390,7 +415,7 @@ class JinjaTemplater(object):
         """
         codes = None
         if monitor.expected_codes:
-            codes = '|'.join(self._expand_expected_codes(
+            codes = '|'.join(octavia_utils.expand_expected_codes(
                 monitor.expected_codes))
         return {
             'id': monitor.id,
@@ -408,7 +433,7 @@ class JinjaTemplater(object):
         }
 
     def _transform_l7policy(self, l7policy, feature_compatibility,
-                            pool_tls_certs=None):
+                            listener_tls_enabled, tls_certs=None):
         """Transforms an L7 policy into an object that will
 
             be processed by the templating system
@@ -420,14 +445,17 @@ class JinjaTemplater(object):
             'redirect_prefix': l7policy.redirect_prefix,
             'enabled': l7policy.enabled
         }
-        if l7policy.redirect_pool:
+        if (l7policy.redirect_pool and
+            l7policy.redirect_pool.provisioning_status !=
+                constants.PENDING_DELETE):
             kwargs = {}
-            if pool_tls_certs and pool_tls_certs.get(
+            if tls_certs is not None and tls_certs.get(
                     l7policy.redirect_pool.id):
                 kwargs = {'pool_tls_certs':
-                          pool_tls_certs.get(l7policy.redirect_pool.id)}
+                          tls_certs.get(l7policy.redirect_pool.id)}
             ret_value['redirect_pool'] = self._transform_pool(
-                l7policy.redirect_pool, feature_compatibility, **kwargs)
+                l7policy.redirect_pool, feature_compatibility,
+                listener_tls_enabled, **kwargs)
         else:
             ret_value['redirect_pool'] = None
         if (l7policy.action in [constants.L7POLICY_ACTION_REDIRECT_TO_URL,
@@ -436,8 +464,10 @@ class JinjaTemplater(object):
             ret_value['redirect_http_code'] = l7policy.redirect_http_code
         else:
             ret_value['redirect_http_code'] = None
+        rule_gen = (rule for rule in l7policy.l7rules if rule.enabled and
+                    rule.provisioning_status != constants.PENDING_DELETE)
         l7rules = [self._transform_l7rule(x, feature_compatibility)
-                   for x in l7policy.l7rules if x.enabled]
+                   for x in rule_gen]
         ret_value['l7rules'] = l7rules
         return ret_value
 
@@ -467,25 +497,3 @@ class JinjaTemplater(object):
         # Spaces next
         value = re.sub(' ', '\\ ', value)
         return value
-
-    @staticmethod
-    def _expand_expected_codes(codes):
-        """Expand the expected code string in set of codes.
-
-        200-204 -> 200, 201, 202, 204
-        200, 203 -> 200, 203
-        """
-
-        retval = set()
-        for code in codes.replace(',', ' ').split(' '):
-            code = code.strip()
-
-            if not code:
-                continue
-            if '-' in code:
-                low, hi = code.split('-')[:2]
-                retval.update(
-                    str(i) for i in six.moves.xrange(int(low), int(hi) + 1))
-            else:
-                retval.add(code)
-        return sorted(retval)
