@@ -25,7 +25,7 @@ import re
 import netaddr
 from oslo_config import cfg
 import rfc3986
-import six
+from wsme import types as wtypes
 
 from octavia.common import constants
 from octavia.common import exceptions
@@ -44,8 +44,8 @@ def url(url, require_scheme=True):
         if require_scheme:
             if p_url.scheme != 'http' and p_url.scheme != 'https':
                 raise exceptions.InvalidURL(url=url)
-    except Exception:
-        raise exceptions.InvalidURL(url=url)
+    except Exception as e:
+        raise exceptions.InvalidURL(url=url) from e
     return True
 
 
@@ -63,8 +63,8 @@ def url_path(url_path):
 
         if invalid_path:
             raise exceptions.InvalidURLPath(url_path=url_path)
-    except Exception:
-        raise exceptions.InvalidURLPath(url_path=url_path)
+    except Exception as e:
+        raise exceptions.InvalidURLPath(url_path=url_path) from e
     return True
 
 
@@ -106,6 +106,8 @@ def regex(regex):
 # L7 rules must be internally consistent.
 def l7rule_data(l7rule):
     """Raises an error if the l7rule given is invalid in some way."""
+    if not l7rule.value:
+        raise exceptions.InvalidL7Rule(msg=_('L7 rule type requires a value'))
     if l7rule.type == constants.L7RULE_TYPE_HEADER:
         if not l7rule.key:
             raise exceptions.InvalidL7Rule(msg='L7 rule type requires a key')
@@ -161,11 +163,72 @@ def l7rule_data(l7rule):
             raise exceptions.InvalidL7Rule(msg='invalid comparison type '
                                            'for rule type')
 
+    elif l7rule.type in [constants.L7RULE_TYPE_SSL_CONN_HAS_CERT,
+                         constants.L7RULE_TYPE_SSL_VERIFY_RESULT,
+                         constants.L7RULE_TYPE_SSL_DN_FIELD]:
+        validate_l7rule_ssl_types(l7rule)
+
     else:
         raise exceptions.InvalidL7Rule(msg='invalid rule type')
     return True
 
 
+def validate_l7rule_ssl_types(l7rule):
+    if not l7rule.type or l7rule.type not in [
+       constants.L7RULE_TYPE_SSL_CONN_HAS_CERT,
+       constants.L7RULE_TYPE_SSL_VERIFY_RESULT,
+       constants.L7RULE_TYPE_SSL_DN_FIELD]:
+        return
+
+    rule_type = None if l7rule.type == wtypes.Unset else l7rule.type
+    req_key = None if l7rule.key == wtypes.Unset else l7rule.key
+    req_value = None if l7rule.value == wtypes.Unset else l7rule.value
+    compare_type = (None if l7rule.compare_type == wtypes.Unset else
+                    l7rule.compare_type)
+    msg = None
+    if rule_type == constants.L7RULE_TYPE_SSL_CONN_HAS_CERT:
+        # key and value are not allowed
+        if req_key:
+            # log error or raise
+            msg = 'L7rule type {0} does not use the "key" field.'.format(
+                rule_type)
+        elif req_value.lower() != 'true':
+            msg = 'L7rule value {0} is not a boolean True string.'.format(
+                req_value)
+        elif compare_type != constants.L7RULE_COMPARE_TYPE_EQUAL_TO:
+            msg = 'L7rule type {0} only supports the {1} compare type.'.format(
+                rule_type, constants.L7RULE_COMPARE_TYPE_EQUAL_TO)
+
+    if rule_type == constants.L7RULE_TYPE_SSL_VERIFY_RESULT:
+        if req_key:
+            # log or raise req_key not used
+            msg = 'L7rule type {0} does not use the "key" field.'.format(
+                rule_type)
+        elif not req_value.isdigit() or int(req_value) < 0:
+            # log or raise req_value must be int
+            msg = 'L7rule type {0} needs a int value, which is >= 0'.format(
+                rule_type)
+        elif compare_type != constants.L7RULE_COMPARE_TYPE_EQUAL_TO:
+            msg = 'L7rule type {0} only supports the {1} compare type.'.format(
+                rule_type, constants.L7RULE_COMPARE_TYPE_EQUAL_TO)
+
+    if rule_type == constants.L7RULE_TYPE_SSL_DN_FIELD:
+        dn_regex = re.compile(constants.DISTINGUISHED_NAME_FIELD_REGEX)
+        if compare_type == constants.L7RULE_COMPARE_TYPE_REGEX:
+            regex(l7rule.value)
+
+        if not req_key or not req_value:
+            # log or raise key and value must be specified.
+            msg = 'L7rule type {0} needs to specify a key and a value.'.format(
+                rule_type)
+        # log or raise the key must be splited by '-'
+        elif not dn_regex.match(req_key):
+            msg = ('Invalid L7rule distinguished name field.')
+
+    if msg:
+        raise exceptions.InvalidL7Rule(msg=msg)
+
+
 def sanitize_l7policy_api_args(l7policy, create=False):
     """Validate and make consistent L7Policy API arguments.
 
@@ -194,30 +257,50 @@ def sanitize_l7policy_api_args(l7policy, create=False):
                 raise exceptions.InvalidL7PolicyArgs(
                     msg='redirect_pool_id or redirect_pool must not be None')
             l7policy.update({'redirect_url': None})
+        elif l7policy['action'] == constants.L7POLICY_ACTION_REDIRECT_PREFIX:
+            if not l7policy.get('redirect_prefix'):
+                raise exceptions.InvalidL7PolicyArgs(
+                    msg='redirect_prefix must not be None')
         else:
             raise exceptions.InvalidL7PolicyAction(
                 action=l7policy['action'])
-    if ((l7policy.get('redirect_pool_id') or
-            l7policy.get('redirect_pool')) and l7policy.get('redirect_url')):
+    if ((l7policy.get('redirect_pool_id') or l7policy.get('redirect_pool')) and
+            (l7policy.get('redirect_url') or l7policy.get('redirect_prefix'))):
         raise exceptions.InvalidL7PolicyArgs(
-            msg='Cannot specify redirect_pool_id and redirect_url '
-                'at the same time')
+            msg='Cannot specify redirect_pool_id and redirect_url or '
+                'redirect_prefix at the same time')
     if l7policy.get('redirect_pool_id'):
         l7policy.update({
             'action': constants.L7POLICY_ACTION_REDIRECT_TO_POOL})
         l7policy.update({'redirect_url': None})
         l7policy.pop('redirect_pool', None)
+        l7policy.update({'redirect_prefix': None})
+        l7policy.update({'redirect_http_code': None})
     if l7policy.get('redirect_pool'):
         l7policy.update({
             'action': constants.L7POLICY_ACTION_REDIRECT_TO_POOL})
         l7policy.update({'redirect_url': None})
         l7policy.pop('redirect_pool_id', None)
+        l7policy.update({'redirect_prefix': None})
+        l7policy.update({'redirect_http_code': None})
     if l7policy.get('redirect_url'):
         url(l7policy['redirect_url'])
         l7policy.update({
             'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL})
         l7policy.update({'redirect_pool_id': None})
+        l7policy.update({'redirect_prefix': None})
         l7policy.pop('redirect_pool', None)
+        if not l7policy.get('redirect_http_code'):
+            l7policy.update({'redirect_http_code': 302})
+    if l7policy.get('redirect_prefix'):
+        url(l7policy['redirect_prefix'])
+        l7policy.update({
+            'action': constants.L7POLICY_ACTION_REDIRECT_PREFIX})
+        l7policy.update({'redirect_pool_id': None})
+        l7policy.update({'redirect_url': None})
+        l7policy.pop('redirect_pool', None)
+        if not l7policy.get('redirect_http_code'):
+            l7policy.update({'redirect_http_code': 302})
 
     # If we are creating, we need an action at this point
     if create and 'action' not in l7policy.keys():
@@ -229,13 +312,13 @@ def sanitize_l7policy_api_args(l7policy, create=False):
     return l7policy
 
 
-def port_exists(port_id):
+def port_exists(port_id, context=None):
     """Raises an exception when a port does not exist."""
     network_driver = utils.get_network_driver()
     try:
-        port = network_driver.get_port(port_id)
-    except Exception:
-        raise exceptions.InvalidSubresource(resource='Port', id=port_id)
+        port = network_driver.get_port(port_id, context=context)
+    except Exception as e:
+        raise exceptions.InvalidSubresource(resource='Port', id=port_id) from e
     return port
 
 
@@ -248,36 +331,46 @@ def check_port_in_use(port):
     return False
 
 
-def subnet_exists(subnet_id):
+def subnet_exists(subnet_id, context=None):
     """Raises an exception when a subnet does not exist."""
     network_driver = utils.get_network_driver()
     try:
-        subnet = network_driver.get_subnet(subnet_id)
-    except Exception:
-        raise exceptions.InvalidSubresource(resource='Subnet', id=subnet_id)
+        subnet = network_driver.get_subnet(subnet_id, context=context)
+    except Exception as e:
+        raise exceptions.InvalidSubresource(
+            resource='Subnet', id=subnet_id) from e
     return subnet
 
 
 def qos_policy_exists(qos_policy_id):
     network_driver = utils.get_network_driver()
+    qos_extension_enabled(network_driver)
     try:
         qos_policy = network_driver.get_qos_policy(qos_policy_id)
-    except Exception:
-        raise exceptions.InvalidSubresource(resource='qos_policy',
-                                            id=qos_policy_id)
+    except Exception as e:
+        raise exceptions.InvalidSubresource(
+            resource='qos_policy', id=qos_policy_id) from e
     return qos_policy
 
 
-def network_exists_optionally_contains_subnet(network_id, subnet_id=None):
+def qos_extension_enabled(network_driver):
+    if not network_driver.qos_enabled():
+        raise exceptions.ValidationException(detail=_(
+            "VIP QoS policy is not allowed in this deployment."))
+
+
+def network_exists_optionally_contains_subnet(network_id, subnet_id=None,
+                                              context=None):
     """Raises an exception when a network does not exist.
 
     If a subnet is provided, also validate the network contains that subnet.
     """
     network_driver = utils.get_network_driver()
     try:
-        network = network_driver.get_network(network_id)
-    except Exception:
-        raise exceptions.InvalidSubresource(resource='Network', id=network_id)
+        network = network_driver.get_network(network_id, context=context)
+    except Exception as e:
+        raise exceptions.InvalidSubresource(
+            resource='Network', id=network_id) from e
     if subnet_id:
         if not network.subnets or subnet_id not in network.subnets:
             raise exceptions.InvalidSubresource(resource='Subnet',
@@ -285,10 +378,12 @@ def network_exists_optionally_contains_subnet(network_id, subnet_id=None):
     return network
 
 
-def network_allowed_by_config(network_id):
-    if CONF.networking.valid_vip_networks:
-        valid_networks = map(str.lower, CONF.networking.valid_vip_networks)
-        if network_id not in valid_networks:
+def network_allowed_by_config(network_id, valid_networks=None):
+    if CONF.networking.valid_vip_networks and not valid_networks:
+        valid_networks = CONF.networking.valid_vip_networks
+    if valid_networks:
+        valid_networks = map(str.lower, valid_networks)
+        if network_id.lower() not in valid_networks:
             raise exceptions.ValidationException(detail=_(
                 'Supplied VIP network_id is not allowed by the configuration '
                 'of this deployment.'))
@@ -321,14 +416,120 @@ def check_session_persistence(SP_dict):
                 '"APP_COOKIE" session persistence type.'))
     except exceptions.ValidationException:
         raise
-    except Exception:
+    except Exception as e:
         raise exceptions.ValidationException(detail=_(
-            'Invalid session_persistence provided.'))
+            'Invalid session_persistence provided.')) from e
 
 
 def ip_not_reserved(ip_address):
     ip_address = (
-        ipaddress.ip_address(six.text_type(ip_address)).exploded.upper())
+        ipaddress.ip_address(ip_address).exploded.upper())
     if ip_address in CONF.networking.reserved_ips:
         raise exceptions.InvalidOption(value=ip_address,
                                        option='member address')
+
+
+def is_flavor_spares_compatible(flavor):
+    if flavor:
+        # If a compute flavor is specified, the flavor is not spares compatible
+        if flavor.get(constants.COMPUTE_FLAVOR, None):
+            return False
+    return True
+
+
+def check_cipher_prohibit_list(cipherstring):
+    ciphers = cipherstring.split(':')
+    prohibit_list = CONF.api_settings.tls_cipher_prohibit_list.split(':')
+    rejected = []
+    for cipher in ciphers:
+        if cipher in prohibit_list:
+            rejected.append(cipher)
+    return rejected
+
+
+def check_default_ciphers_prohibit_list_conflict():
+    listener_rejected = check_cipher_prohibit_list(
+        CONF.api_settings.default_listener_ciphers)
+    if listener_rejected:
+        raise exceptions.ValidationException(
+            detail=_('Default listener ciphers conflict with prohibit list. '
+                     'Conflicting ciphers: ' + ', '.join(listener_rejected)))
+
+    pool_rejected = check_cipher_prohibit_list(
+        CONF.api_settings.default_pool_ciphers)
+    if pool_rejected:
+        raise exceptions.ValidationException(
+            detail=_('Default pool ciphers conflict with prohibit list. '
+                     'Conflicting ciphers: ' + ', '.join(pool_rejected)))
+
+
+def check_tls_version_list(versions):
+    if versions == []:
+        raise exceptions.ValidationException(
+            detail=_('Empty TLS version list. Either specify at least one TLS '
+                     'version or remove this parameter to use the default.'))
+
+    # Unset action
+    if versions is None:
+        return
+
+    invalid_versions = [v for v in versions
+                        if v not in constants.TLS_ALL_VERSIONS]
+    if invalid_versions:
+        raise exceptions.ValidationException(
+            detail=_('Invalid TLS versions: ' + ', '.join(invalid_versions)))
+
+
+def check_tls_version_min(versions, message=None):
+    """Checks a TLS version string against the configured minimum."""
+
+    if not CONF.api_settings.minimum_tls_version:
+        return
+
+    if not message:
+        message = _("Requested TLS versions are less than the minimum: ")
+
+    min_ver_index = constants.TLS_ALL_VERSIONS.index(
+        CONF.api_settings.minimum_tls_version)
+
+    rejected = []
+    for ver in versions:
+        if constants.TLS_ALL_VERSIONS.index(ver) < min_ver_index:
+            rejected.append(ver)
+    if rejected:
+        raise exceptions.ValidationException(detail=(
+            message + ', '.join(rejected) + " < " +
+            CONF.api_settings.minimum_tls_version))
+
+
+def check_default_tls_versions_min_conflict():
+    if not CONF.api_settings.minimum_tls_version:
+        return
+
+    listener_message = _("Default listener TLS versions are less than the "
+                         "minimum: ")
+    pool_message = _("Default pool TLS versions are less than the minimum: ")
+
+    check_tls_version_min(CONF.api_settings.default_listener_tls_versions,
+                          message=listener_message)
+
+    check_tls_version_min(CONF.api_settings.default_pool_tls_versions,
+                          message=pool_message)
+
+
+def check_alpn_protocols(protocols):
+    if protocols == []:
+        raise exceptions.ValidationException(
+            detail=_('Empty ALPN protocol list. Either specify at least one '
+                     'ALPN protocol or remove this parameter to use the '
+                     'default.'))
+
+    # Unset action
+    if protocols is None:
+        return
+
+    invalid_protocols = [p for p in protocols
+                         if p not in constants.SUPPORTED_ALPN_PROTOCOLS]
+    if invalid_protocols:
+        raise exceptions.ValidationException(
+            detail=_('Invalid ALPN protocol: ' + ', '.join(invalid_protocols)))
