@@ -1,5 +1,6 @@
 #    Copyright 2014 Rackspace
 #    Copyright 2016 Blue Box, an IBM Company
+#    Copyright 2017 Walmart Stores Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -13,14 +14,20 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+
 from oslo_db.sqlalchemy import models
 import sqlalchemy as sa
 from sqlalchemy.ext import orderinglist
 from sqlalchemy import orm
 from sqlalchemy.orm import validates
 from sqlalchemy.sql import func
+from sqlalchemy_utils import ScalarListType
 
 from octavia.api.v2.types import amphora
+from octavia.api.v2.types import availability_zone_profile
+from octavia.api.v2.types import availability_zones
+from octavia.api.v2.types import flavor_profile
+from octavia.api.v2.types import flavors
 from octavia.api.v2.types import health_monitor
 from octavia.api.v2.types import l7policy
 from octavia.api.v2.types import l7rule
@@ -29,6 +36,7 @@ from octavia.api.v2.types import load_balancer
 from octavia.api.v2.types import member
 from octavia.api.v2.types import pool
 from octavia.api.v2.types import quotas
+from octavia.common import constants
 from octavia.common import data_models
 from octavia.db import base_models
 from octavia.i18n import _
@@ -133,9 +141,7 @@ class SessionPersistence(base_models.BASE):
     persistence_timeout = sa.Column(sa.Integer(), nullable=True)
     persistence_granularity = sa.Column(sa.String(64), nullable=True)
     pool = orm.relationship("Pool", uselist=False,
-                            backref=orm.backref("session_persistence",
-                                                uselist=False,
-                                                cascade="delete"))
+                            back_populates="session_persistence")
 
 
 class ListenerStatistics(base_models.BASE):
@@ -169,9 +175,24 @@ class ListenerStatistics(base_models.BASE):
                                'Current value is %(value)d.') % data)
         return value
 
+    def __iadd__(self, other):
+        if isinstance(other, (ListenerStatistics,
+                              data_models.ListenerStatistics)):
+            self.bytes_in += other.bytes_in
+            self.bytes_out += other.bytes_out
+            self.request_errors += other.request_errors
+            self.total_connections += other.total_connections
+        else:
+            raise TypeError(  # noqa: O342
+                "unsupported operand type(s) for +=: '{0}' and '{1}'".format(
+                    type(self), type(other)))
+
+        return self
+
 
 class Member(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
-             models.TimestampMixin, base_models.NameMixin):
+             models.TimestampMixin, base_models.NameMixin,
+             base_models.TagMixin):
 
     __data_model__ = data_models.Member
 
@@ -206,14 +227,20 @@ class Member(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
                       name="fk_member_operating_status_name"),
         nullable=False)
     enabled = sa.Column(sa.Boolean(), nullable=False)
-    pool = orm.relationship("Pool", backref=orm.backref("members",
-                                                        uselist=True,
-                                                        cascade="delete"))
+    pool = orm.relationship("Pool", back_populates="members")
+
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==Member.id)'
+    )
 
 
 class HealthMonitor(base_models.BASE, base_models.IdMixin,
                     base_models.ProjectMixin, models.TimestampMixin,
-                    base_models.NameMixin):
+                    base_models.NameMixin, base_models.TagMixin):
 
     __data_model__ = data_models.HealthMonitor
 
@@ -244,9 +271,8 @@ class HealthMonitor(base_models.BASE, base_models.IdMixin,
     expected_codes = sa.Column(sa.String(64), nullable=True)
     enabled = sa.Column(sa.Boolean, nullable=False)
     pool = orm.relationship("Pool", uselist=False,
-                            backref=orm.backref("health_monitor",
-                                                uselist=False,
-                                                cascade="delete"))
+                            back_populates="health_monitor")
+
     provisioning_status = sa.Column(
         sa.String(16),
         sa.ForeignKey("provisioning_status.name",
@@ -257,10 +283,19 @@ class HealthMonitor(base_models.BASE, base_models.IdMixin,
         sa.ForeignKey("operating_status.name",
                       name="fk_health_monitor_operating_status_name"),
         nullable=False)
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==HealthMonitor.id)'
+    )
+    http_version = sa.Column(sa.Float, nullable=True)
+    domain_name = sa.Column(sa.String(255), nullable=True)
 
 
 class Pool(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
-           models.TimestampMixin, base_models.NameMixin):
+           models.TimestampMixin, base_models.NameMixin, base_models.TagMixin):
 
     __data_model__ = data_models.Pool
 
@@ -292,10 +327,32 @@ class Pool(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
         sa.String(36),
         sa.ForeignKey("load_balancer.id", name="fk_pool_load_balancer_id"),
         nullable=True)
+    health_monitor = orm.relationship("HealthMonitor", uselist=False,
+                                      cascade="delete", back_populates="pool")
     load_balancer = orm.relationship("LoadBalancer", uselist=False,
-                                     backref=orm.backref("pools",
-                                                         uselist=True,
-                                                         cascade="delete"))
+                                     back_populates="pools")
+    members = orm.relationship("Member", uselist=True, cascade="delete",
+                               back_populates="pool")
+    session_persistence = orm.relationship(
+        "SessionPersistence", uselist=False, cascade="delete",
+        back_populates="pool")
+    _default_listeners = orm.relationship("Listener", uselist=True,
+                                          back_populates="default_pool")
+    l7policies = orm.relationship("L7Policy", uselist=True,
+                                  back_populates="redirect_pool")
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==Pool.id)'
+    )
+    tls_certificate_id = sa.Column(sa.String(255), nullable=True)
+    ca_tls_certificate_id = sa.Column(sa.String(255), nullable=True)
+    crl_container_id = sa.Column(sa.String(255), nullable=True)
+    tls_enabled = sa.Column(sa.Boolean, default=False, nullable=False)
+    tls_ciphers = sa.Column(sa.String(2048), nullable=True)
+    tls_versions = sa.Column(ScalarListType(), nullable=True)
 
     # This property should be a unique list of any listeners that reference
     # this pool as its default_pool and any listeners referenced by enabled
@@ -305,19 +362,19 @@ class Pool(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
     @property
     def listeners(self):
         _listeners = self._default_listeners[:]
-        _l_ids = [l.id for l in _listeners]
+        _l_ids = [li.id for li in _listeners]
         l7_listeners = [p.listener for p in self.l7policies
                         if len(p.l7rules) > 0 and p.enabled is True]
-        for l in l7_listeners:
-            if l.id not in _l_ids:
-                _listeners.append(l)
-                _l_ids.append(l.id)
+        for li in l7_listeners:
+            if li.id not in _l_ids:
+                _listeners.append(li)
+                _l_ids.append(li.id)
         return _listeners
 
 
 class LoadBalancer(base_models.BASE, base_models.IdMixin,
                    base_models.ProjectMixin, models.TimestampMixin,
-                   base_models.NameMixin):
+                   base_models.NameMixin, base_models.TagMixin):
 
     __data_model__ = data_models.LoadBalancer
 
@@ -342,10 +399,30 @@ class LoadBalancer(base_models.BASE, base_models.IdMixin,
         nullable=True)
     enabled = sa.Column(sa.Boolean, nullable=False)
     amphorae = orm.relationship("Amphora", uselist=True,
-                                backref=orm.backref("load_balancer",
-                                                    uselist=False))
+                                back_populates="load_balancer")
     server_group_id = sa.Column(sa.String(36), nullable=True)
     provider = sa.Column(sa.String(64), nullable=True)
+    vip = orm.relationship('Vip', cascade='delete', uselist=False,
+                           backref=orm.backref('load_balancer', uselist=False))
+    pools = orm.relationship('Pool', cascade='delete', uselist=True,
+                             back_populates="load_balancer")
+    listeners = orm.relationship('Listener', cascade='delete', uselist=True,
+                                 back_populates='load_balancer')
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==LoadBalancer.id)'
+    )
+    flavor_id = sa.Column(
+        sa.String(36),
+        sa.ForeignKey("flavor.id", name="fk_lb_flavor_id"), nullable=True)
+    availability_zone = sa.Column(
+        sa.String(255),
+        sa.ForeignKey("availability_zone.name",
+                      name="fk_load_balancer_availability_zone_name"),
+        nullable=True)
 
 
 class VRRPGroup(base_models.BASE):
@@ -387,16 +464,13 @@ class Vip(base_models.BASE):
     port_id = sa.Column(sa.String(36), nullable=True)
     subnet_id = sa.Column(sa.String(36), nullable=True)
     network_id = sa.Column(sa.String(36), nullable=True)
-    load_balancer = orm.relationship("LoadBalancer", uselist=False,
-                                     backref=orm.backref("vip", uselist=False,
-                                                         cascade="delete"))
     qos_policy_id = sa.Column(sa.String(36), nullable=True)
     octavia_owned = sa.Column(sa.Boolean(), nullable=True)
 
 
 class Listener(base_models.BASE, base_models.IdMixin,
                base_models.ProjectMixin, models.TimestampMixin,
-               base_models.NameMixin):
+               base_models.NameMixin, base_models.TagMixin):
 
     __data_model__ = data_models.Listener
 
@@ -405,8 +479,9 @@ class Listener(base_models.BASE, base_models.IdMixin,
     __v2_wsme__ = listener.ListenerResponse
 
     __table_args__ = (
-        sa.UniqueConstraint('load_balancer_id', 'protocol_port',
-                            name='uq_listener_load_balancer_id_protocol_port'),
+        sa.UniqueConstraint(
+            'load_balancer_id', 'protocol', 'protocol_port',
+            name='uq_listener_load_balancer_id_protocol_port'),
     )
 
     description = sa.Column(sa.String(255), nullable=True)
@@ -420,7 +495,7 @@ class Listener(base_models.BASE, base_models.IdMixin,
         sa.String(36),
         sa.ForeignKey("load_balancer.id", name="fk_listener_load_balancer_id"),
         nullable=True)
-    tls_certificate_id = sa.Column(sa.String(36), nullable=True)
+    tls_certificate_id = sa.Column(sa.String(255), nullable=True)
     default_pool_id = sa.Column(
         sa.String(36),
         sa.ForeignKey("pool.id", name="fk_listener_pool_id"),
@@ -437,12 +512,17 @@ class Listener(base_models.BASE, base_models.IdMixin,
         nullable=False)
     enabled = sa.Column(sa.Boolean(), nullable=False)
     load_balancer = orm.relationship("LoadBalancer", uselist=False,
-                                     backref=orm.backref("listeners",
-                                                         uselist=True,
-                                                         cascade="delete"))
+                                     back_populates="listeners")
     default_pool = orm.relationship("Pool", uselist=False,
-                                    backref=orm.backref("_default_listeners",
-                                                        uselist=True))
+                                    back_populates="_default_listeners")
+    sni_containers = orm.relationship(
+        'SNI', cascade='all,delete-orphan',
+        uselist=True, backref=orm.backref('listener', uselist=False))
+
+    l7policies = orm.relationship(
+        'L7Policy', uselist=True, order_by='L7Policy.position',
+        collection_class=orderinglist.ordering_list('position', count_from=1),
+        cascade='delete', back_populates='listener')
 
     peer_port = sa.Column(sa.Integer(), nullable=True)
     insert_headers = sa.Column(sa.PickleType())
@@ -450,6 +530,24 @@ class Listener(base_models.BASE, base_models.IdMixin,
     timeout_member_connect = sa.Column(sa.Integer, nullable=True)
     timeout_member_data = sa.Column(sa.Integer, nullable=True)
     timeout_tcp_inspect = sa.Column(sa.Integer, nullable=True)
+    client_ca_tls_certificate_id = sa.Column(sa.String(255), nullable=True)
+    client_authentication = sa.Column(
+        sa.String(10),
+        sa.ForeignKey("client_authentication_mode.name",
+                      name="fk_listener_client_authentication_mode_name"),
+        nullable=False, default=constants.CLIENT_AUTH_NONE)
+    client_crl_container_id = sa.Column(sa.String(255), nullable=True)
+    tls_ciphers = sa.Column(sa.String(2048), nullable=True)
+    tls_versions = sa.Column(ScalarListType(), nullable=True)
+    alpn_protocols = sa.Column(ScalarListType(), nullable=True)
+
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==Listener.id)'
+    )
 
     # This property should be a unique list of the default_pool and anything
     # referenced by enabled L7Policies with at least one rule that also
@@ -471,6 +569,10 @@ class Listener(base_models.BASE, base_models.IdMixin,
                 _p_ids.append(p.id)
         return _pools
 
+    allowed_cidrs = orm.relationship(
+        'ListenerCidr', cascade='all,delete-orphan',
+        uselist=True, backref=orm.backref('listener', uselist=False))
+
 
 class SNI(base_models.BASE):
 
@@ -480,17 +582,12 @@ class SNI(base_models.BASE):
     __table_args__ = (
         sa.PrimaryKeyConstraint('listener_id', 'tls_container_id'),
     )
-
     listener_id = sa.Column(
         sa.String(36),
         sa.ForeignKey("listener.id", name="fk_sni_listener_id"),
         nullable=False)
     tls_container_id = sa.Column(sa.String(128), nullable=False)
     position = sa.Column(sa.Integer(), nullable=True)
-    listener = orm.relationship("Listener", uselist=False,
-                                backref=orm.backref("sni_containers",
-                                                    uselist=True,
-                                                    cascade="delete"))
 
 
 class Amphora(base_models.BASE, base_models.IdMixin, models.TimestampMixin):
@@ -528,6 +625,9 @@ class Amphora(base_models.BASE, base_models.IdMixin, models.TimestampMixin):
     vrrp_priority = sa.Column(sa.Integer(), nullable=True)
     cached_zone = sa.Column(sa.String(255), nullable=True)
     image_id = sa.Column(sa.String(36), nullable=True)
+    load_balancer = orm.relationship("LoadBalancer", uselist=False,
+                                     back_populates='amphorae')
+    compute_flavor = sa.Column(sa.String(255), nullable=True)
 
 
 class AmphoraHealth(base_models.BASE):
@@ -543,7 +643,7 @@ class AmphoraHealth(base_models.BASE):
 
 
 class L7Rule(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
-             models.TimestampMixin):
+             models.TimestampMixin, base_models.TagMixin):
 
     __data_model__ = data_models.L7Rule
 
@@ -572,9 +672,7 @@ class L7Rule(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
     invert = sa.Column(sa.Boolean(), default=False, nullable=False)
     enabled = sa.Column(sa.Boolean(), nullable=False)
     l7policy = orm.relationship("L7Policy", uselist=False,
-                                backref=orm.backref("l7rules",
-                                                    uselist=True,
-                                                    cascade="delete"))
+                                back_populates="l7rules")
     provisioning_status = sa.Column(
         sa.String(16),
         sa.ForeignKey("provisioning_status.name",
@@ -585,10 +683,18 @@ class L7Rule(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
         sa.ForeignKey("operating_status.name",
                       name="fk_l7rule_operating_status_name"),
         nullable=False)
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==L7Rule.id)'
+    )
 
 
 class L7Policy(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
-               models.TimestampMixin, base_models.NameMixin):
+               models.TimestampMixin, base_models.NameMixin,
+               base_models.TagMixin):
 
     __data_model__ = data_models.L7Policy
 
@@ -614,20 +720,18 @@ class L7Policy(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
     redirect_url = sa.Column(
         sa.String(255),
         nullable=True)
+    redirect_prefix = sa.Column(
+        sa.String(255),
+        nullable=True)
+    redirect_http_code = sa.Column(sa.Integer, nullable=True)
     position = sa.Column(sa.Integer, nullable=False)
     enabled = sa.Column(sa.Boolean(), nullable=False)
-    listener = orm.relationship(
-        "Listener", uselist=False,
-        backref=orm.backref(
-            "l7policies",
-            uselist=True,
-            order_by="L7Policy.position",
-            collection_class=orderinglist.ordering_list('position',
-                                                        count_from=1),
-            cascade="delete"))
+    listener = orm.relationship("Listener", uselist=False,
+                                back_populates="l7policies")
     redirect_pool = orm.relationship("Pool", uselist=False,
-                                     backref=orm.backref("l7policies",
-                                                         uselist=True))
+                                     back_populates="l7policies")
+    l7rules = orm.relationship("L7Rule", uselist=True, cascade="delete",
+                               back_populates="l7policy")
     provisioning_status = sa.Column(
         sa.String(16),
         sa.ForeignKey("provisioning_status.name",
@@ -638,6 +742,13 @@ class L7Policy(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
         sa.ForeignKey("operating_status.name",
                       name="fk_l7policy_operating_status_name"),
         nullable=False)
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==L7Policy.id)'
+    )
 
 
 class Quotas(base_models.BASE):
@@ -654,8 +765,114 @@ class Quotas(base_models.BASE):
     load_balancer = sa.Column(sa.Integer(), nullable=True)
     member = sa.Column(sa.Integer(), nullable=True)
     pool = sa.Column(sa.Integer(), nullable=True)
+    l7policy = sa.Column(sa.Integer(), nullable=True)
+    l7rule = sa.Column(sa.Integer(), nullable=True)
     in_use_health_monitor = sa.Column(sa.Integer(), nullable=True)
     in_use_listener = sa.Column(sa.Integer(), nullable=True)
     in_use_load_balancer = sa.Column(sa.Integer(), nullable=True)
     in_use_member = sa.Column(sa.Integer(), nullable=True)
     in_use_pool = sa.Column(sa.Integer(), nullable=True)
+    in_use_l7policy = sa.Column(sa.Integer(), nullable=True)
+    in_use_l7rule = sa.Column(sa.Integer(), nullable=True)
+
+
+class FlavorProfile(base_models.BASE, base_models.IdMixin,
+                    base_models.NameMixin):
+
+    __data_model__ = data_models.FlavorProfile
+
+    __tablename__ = "flavor_profile"
+
+    __v2_wsme__ = flavor_profile.FlavorProfileResponse
+
+    provider_name = sa.Column(sa.String(255), nullable=False)
+    flavor_data = sa.Column(sa.String(4096), nullable=False)
+
+
+class Flavor(base_models.BASE,
+             base_models.IdMixin,
+             base_models.NameMixin):
+
+    __data_model__ = data_models.Flavor
+
+    __tablename__ = "flavor"
+
+    __v2_wsme__ = flavors.FlavorResponse
+
+    __table_args__ = (
+        sa.UniqueConstraint('name',
+                            name='uq_flavor_name'),
+    )
+
+    description = sa.Column(sa.String(255), nullable=True)
+    enabled = sa.Column(sa.Boolean(), nullable=False)
+    flavor_profile_id = sa.Column(
+        sa.String(36),
+        sa.ForeignKey("flavor_profile.id",
+                      name="fk_flavor_flavor_profile_id"),
+        nullable=False)
+
+
+class AvailabilityZoneProfile(base_models.BASE, base_models.IdMixin,
+                              base_models.NameMixin):
+
+    __data_model__ = data_models.AvailabilityZoneProfile
+
+    __tablename__ = "availability_zone_profile"
+
+    __v2_wsme__ = availability_zone_profile.AvailabilityZoneProfileResponse
+
+    provider_name = sa.Column(sa.String(255), nullable=False)
+    availability_zone_data = sa.Column(sa.String(4096), nullable=False)
+
+
+class AvailabilityZone(base_models.BASE,
+                       base_models.NameMixin):
+
+    __data_model__ = data_models.AvailabilityZone
+
+    __tablename__ = "availability_zone"
+
+    __v2_wsme__ = availability_zones.AvailabilityZoneResponse
+
+    __table_args__ = (
+        sa.PrimaryKeyConstraint('name'),
+    )
+
+    description = sa.Column(sa.String(255), nullable=True)
+    enabled = sa.Column(sa.Boolean(), nullable=False)
+    availability_zone_profile_id = sa.Column(
+        sa.String(36),
+        sa.ForeignKey("availability_zone_profile.id",
+                      name="fk_az_az_profile_id"),
+        nullable=False)
+
+
+class ClientAuthenticationMode(base_models.BASE):
+
+    __tablename__ = "client_authentication_mode"
+
+    name = sa.Column(sa.String(10), primary_key=True, nullable=False)
+
+
+class SparesPool(base_models.BASE):
+
+    __tablename__ = "spares_pool"
+
+    updated_at = sa.Column(sa.DateTime, primary_key=True, nullable=True)
+
+
+class ListenerCidr(base_models.BASE):
+
+    __data_model__ = data_models.ListenerCidr
+
+    __tablename__ = "listener_cidr"
+    __table_args__ = (
+        sa.PrimaryKeyConstraint('listener_id', 'cidr'),
+    )
+
+    listener_id = sa.Column(
+        sa.String(36),
+        sa.ForeignKey("listener.id", name="fk_listener_cidr_listener_id"),
+        nullable=False)
+    cidr = sa.Column(sa.String(64), nullable=False)
