@@ -12,7 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
@@ -32,6 +33,22 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
     BASE_PATH = '/v2'
     BASE_PATH_v2_0 = '/v2.0'
 
+    # /lbaas/flavors
+    FLAVORS_PATH = '/flavors'
+    FLAVOR_PATH = FLAVORS_PATH + '/{flavor_id}'
+
+    # /lbaas/flavorprofiles
+    FPS_PATH = '/flavorprofiles'
+    FP_PATH = FPS_PATH + '/{fp_id}'
+
+    # /lbaas/availabilityzones
+    AZS_PATH = '/availabilityzones'
+    AZ_PATH = AZS_PATH + '/{az_name}'
+
+    # /lbaas/availabilityzoneprofiles
+    AZPS_PATH = '/availabilityzoneprofiles'
+    AZP_PATH = AZPS_PATH + '/{azp_id}'
+
     # /lbaas/loadbalancers
     LBS_PATH = '/lbaas/loadbalancers'
     LB_PATH = LBS_PATH + '/{lb_id}'
@@ -68,15 +85,21 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
     AMPHORAE_PATH = '/octavia/amphorae'
     AMPHORA_PATH = AMPHORAE_PATH + '/{amphora_id}'
     AMPHORA_FAILOVER_PATH = AMPHORA_PATH + '/failover'
+    AMPHORA_STATS_PATH = AMPHORA_PATH + '/stats'
+    AMPHORA_CONFIG_PATH = AMPHORA_PATH + '/config'
 
     PROVIDERS_PATH = '/lbaas/providers'
+    FLAVOR_CAPABILITIES_PATH = (
+        PROVIDERS_PATH + '/{provider}/flavor_capabilities')
+    AVAILABILITY_ZONE_CAPABILITIES_PATH = (
+        PROVIDERS_PATH + '/{provider}/availability_zone_capabilities')
 
     NOT_AUTHORIZED_BODY = {
         'debuginfo': None, 'faultcode': 'Client',
         'faultstring': 'Policy does not allow this request to be performed.'}
 
     def setUp(self):
-        super(BaseAPITest, self).setUp()
+        super().setUp()
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         self.conf.config(group="controller_worker",
                          network_driver='network_noop_driver')
@@ -88,6 +111,7 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
                          enabled_provider_drivers={
                              'amphora': 'Amp driver.',
                              'noop_driver': 'NoOp driver.',
+                             'noop_driver-alt': 'NoOp driver alt alisas.',
                              'octavia': 'Octavia driver.'})
         self.lb_repo = repositories.LoadBalancerRepository()
         self.listener_repo = repositories.ListenerRepository()
@@ -98,6 +122,8 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
         self.l7rule_repo = repositories.L7RuleRepository()
         self.health_monitor_repo = repositories.HealthMonitorRepository()
         self.amphora_repo = repositories.AmphoraRepository()
+        self.flavor_repo = repositories.FlavorRepository()
+        self.flavor_profile_repo = repositories.FlavorProfileRepository()
         patcher2 = mock.patch('octavia.certificates.manager.barbican.'
                               'BarbicanCertManager')
         self.cert_manager_mock = patcher2.start()
@@ -119,8 +145,10 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
         self.addCleanup(check_quota_met_true_mock.stop)
 
     def _make_app(self):
+        # Note: we need to set argv=() to stop the wsgi setup_app from
+        # pulling in the testing tool sys.argv
         return pecan.testing.load_test_app({'app': pconfig.app,
-                                            'wsme': pconfig.wsme})
+                                            'wsme': pconfig.wsme}, argv=())
 
     def _get_full_path(self, path):
         return ''.join([self.BASE_PATH, path])
@@ -182,6 +210,39 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
                                 expect_errors=expect_errors)
         return response
 
+    def create_flavor(self, name, description, flavor_profile_id, enabled):
+        req_dict = {'name': name, 'description': description,
+                    'flavor_profile_id': flavor_profile_id,
+                    'enabled': enabled}
+        body = {'flavor': req_dict}
+        response = self.post(self.FLAVORS_PATH, body)
+        return response.json.get('flavor')
+
+    def create_flavor_profile(self, name, provider_name, flavor_data):
+        req_dict = {'name': name, 'provider_name': provider_name,
+                    constants.FLAVOR_DATA: flavor_data}
+        body = {'flavorprofile': req_dict}
+        response = self.post(self.FPS_PATH, body)
+        return response.json.get('flavorprofile')
+
+    def create_availability_zone(self, name, description,
+                                 availability_zone_profile_id, enabled):
+        req_dict = {
+            'name': name, 'description': description,
+            'availability_zone_profile_id': availability_zone_profile_id,
+            'enabled': enabled}
+        body = {'availability_zone': req_dict}
+        response = self.post(self.AZS_PATH, body)
+        return response.json.get('availability_zone')
+
+    def create_availability_zone_profile(self, name, provider_name,
+                                         availability_zone_data):
+        req_dict = {'name': name, 'provider_name': provider_name,
+                    constants.AVAILABILITY_ZONE_DATA: availability_zone_data}
+        body = {'availability_zone_profile': req_dict}
+        response = self.post(self.AZPS_PATH, body)
+        return response.json.get('availability_zone_profile')
+
     def create_load_balancer(self, vip_subnet_id,
                              **optionals):
         req_dict = {'vip_subnet_id': vip_subnet_id,
@@ -318,6 +379,9 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
         response = self.put(path, body, status=202)
         return response.json
 
+    # NOTE: This method should be used cautiously. On load balancers with a
+    # significant amount of children resources, it will update the status for
+    # each and every resource and thus taking a lot of DB time.
     def _set_lb_and_children_statuses(self, lb_id, prov_status, op_status,
                                       autodetect=True):
         self.set_object_status(self.lb_repo, lb_id,
@@ -387,6 +451,9 @@ class BaseAPITest(base_db_test.OctaviaDBTestBase):
                                        provisioning_status=hm_prov,
                                        operating_status=op_status)
 
+    # NOTE: This method should be used cautiously. On load balancers with a
+    # significant amount of children resources, it will update the status for
+    # each and every resource and thus taking a lot of DB time.
     def set_lb_status(self, lb_id, status=None):
         explicit_status = True if status is not None else False
         if not explicit_status:
