@@ -12,7 +12,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
 
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
@@ -22,6 +22,7 @@ from octavia.common import constants
 import octavia.common.context
 from octavia.common import data_models
 from octavia.common import exceptions
+from octavia.tests.common import constants as c_const
 from octavia.tests.functional.api.v2 import base
 
 
@@ -32,7 +33,7 @@ class TestL7Policy(base.BaseAPITest):
     root_tag_links = 'l7policies_links'
 
     def setUp(self):
-        super(TestL7Policy, self).setUp()
+        super().setUp()
         self.lb = self.create_load_balancer(uuidutils.generate_uuid())
         self.lb_id = self.lb.get('loadbalancer').get('id')
         self.project_id = self.lb.get('loadbalancer').get('project_id')
@@ -51,7 +52,8 @@ class TestL7Policy(base.BaseAPITest):
     def test_get(self):
         api_l7policy = self.create_l7policy(
             self.listener_id,
-            constants.L7POLICY_ACTION_REJECT).get(self.root_tag)
+            constants.L7POLICY_ACTION_REJECT,
+            tags=['test_tag']).get(self.root_tag)
         response = self.get(self.L7POLICY_PATH.format(
             l7policy_id=api_l7policy.get('id'))).json.get(self.root_tag)
         self.assertEqual(api_l7policy, response)
@@ -120,12 +122,14 @@ class TestL7Policy(base.BaseAPITest):
     def test_get_all(self):
         api_l7policy = self.create_l7policy(self.listener_id,
                                             constants.L7POLICY_ACTION_REJECT,
+                                            tags=['test_tag']
                                             ).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         policies = self.get(self.L7POLICIES_PATH).json.get(self.root_tag_list)
         self.assertIsInstance(policies, list)
         self.assertEqual(1, len(policies))
         self.assertEqual(api_l7policy.get('id'), policies[0].get('id'))
+        self.assertEqual(api_l7policy['tags'], policies[0]['tags'])
 
     def test_get_all_admin(self):
         project_id = uuidutils.generate_uuid()
@@ -220,6 +224,58 @@ class TestL7Policy(base.BaseAPITest):
         self.assertIn((api_l7p_c.get('id'), api_l7p_c.get('action')),
                       policy_id_actions)
 
+    def test_get_all_unscoped_token(self):
+        project_id = uuidutils.generate_uuid()
+        lb1 = self.create_load_balancer(uuidutils.generate_uuid(), name='lb1',
+                                        project_id=project_id)
+        lb1_id = lb1.get('loadbalancer').get('id')
+        self.set_lb_status(lb1_id)
+        listener1 = self.create_listener(constants.PROTOCOL_HTTP, 80,
+                                         lb1_id)
+        listener1_id = listener1.get('listener').get('id')
+        self.set_lb_status(lb1_id)
+        pool1 = self.create_pool(lb1_id, constants.PROTOCOL_HTTP,
+                                 constants.LB_ALGORITHM_ROUND_ROBIN)
+        pool1_id = pool1.get('pool').get('id')
+        self.set_lb_status(lb1_id)
+        self.create_l7policy(
+            listener1_id,
+            constants.L7POLICY_ACTION_REJECT).get(self.root_tag)
+        self.set_lb_status(lb1_id)
+        self.create_l7policy(
+            listener1_id, constants.L7POLICY_ACTION_REDIRECT_TO_POOL,
+            position=2, redirect_pool_id=pool1_id).get(self.root_tag)
+        self.set_lb_status(lb1_id)
+        self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+            redirect_url='http://localhost/').get(self.root_tag)
+        self.set_lb_status(lb1_id)
+
+        auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
+        self.conf.config(group='api_settings',
+                         auth_strategy=constants.KEYSTONE)
+        with mock.patch.object(octavia.common.context.Context, 'project_id',
+                               None):
+            override_credentials = {
+                'service_user_id': None,
+                'user_domain_id': None,
+                'is_admin_project': True,
+                'service_project_domain_id': None,
+                'service_project_id': None,
+                'roles': ['load-balancer_member'],
+                'user_id': None,
+                'is_admin': False,
+                'service_user_domain_id': None,
+                'project_domain_id': None,
+                'service_roles': [],
+                'project_id': None}
+            with mock.patch(
+                    "oslo_context.context.RequestContext.to_policy_values",
+                    return_value=override_credentials):
+                result = self.get(self.L7POLICIES_PATH, status=403).json
+        self.conf.config(group='api_settings', auth_strategy=auth_strategy)
+        self.assertEqual(self.NOT_AUTHORIZED_BODY, result)
+
     def test_get_all_non_admin_global_observer(self):
         project_id = uuidutils.generate_uuid()
         lb1 = self.create_load_balancer(uuidutils.generate_uuid(), name='lb1',
@@ -438,7 +494,8 @@ class TestL7Policy(base.BaseAPITest):
         links = middle[self.root_tag_links]
         self.assertEqual(1, len(objs))
         self.assertEqual(2, len(links))
-        self.assertItemsEqual(['previous', 'next'], [l['rel'] for l in links])
+        self.assertCountEqual(['previous', 'next'],
+                              [link['rel'] for link in links])
 
     def test_get_all_fields_filter(self):
         self.create_l7policy(
@@ -498,6 +555,92 @@ class TestL7Policy(base.BaseAPITest):
         self.assertEqual(policy1['id'],
                          l7pos['l7policies'][0]['id'])
 
+    def test_get_all_tags_filter(self):
+        policy1 = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REJECT,
+            tags=['test_tag1', 'test_tag2']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        policy2 = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REDIRECT_TO_POOL,
+            position=2, redirect_pool_id=self.pool_id,
+            tags=['test_tag2', 'test_tag3']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        policy3 = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+            redirect_url='http://localhost/',
+            tags=['test_tag4', 'test_tag5']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(2, len(policies))
+        self.assertEqual(
+            [policy1.get('id'), policy2.get('id')],
+            [policy.get('id') for policy in policies]
+        )
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'tags': ['test_tag2', 'test_tag3']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(1, len(policies))
+        self.assertEqual(
+            [policy2.get('id')],
+            [policy.get('id') for policy in policies]
+        )
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'tags-any': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(2, len(policies))
+        self.assertEqual(
+            [policy1.get('id'), policy2.get('id')],
+            [policy.get('id') for policy in policies]
+        )
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'not-tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(1, len(policies))
+        self.assertEqual(
+            [policy3.get('id')],
+            [policy.get('id') for policy in policies]
+        )
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'not-tags-any': ['test_tag2', 'test_tag4']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(0, len(policies))
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'tags': 'test_tag2',
+                    'tags-any': ['test_tag1', 'test_tag3']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(2, len(policies))
+        self.assertEqual(
+            [policy1.get('id'), policy2.get('id')],
+            [policy.get('id') for policy in policies]
+        )
+
+        policies = self.get(
+            self.L7POLICIES_PATH,
+            params={'tags': 'test_tag2', 'not-tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(policies, list)
+        self.assertEqual(0, len(policies))
+
     def test_empty_get_all(self):
         response = self.get(self.L7POLICIES_PATH).json.get(self.root_tag_list)
         self.assertIsInstance(response, list)
@@ -614,6 +757,61 @@ class TestL7Policy(base.BaseAPITest):
             l7policy_prov_status=constants.PENDING_CREATE,
             l7policy_op_status=constants.OFFLINE)
 
+    def test_create_with_redirect_http_code(self):
+        action_key_values = {
+            constants.L7POLICY_ACTION_REDIRECT_PREFIX: {
+                'redirect_prefix': 'https://example.com',
+                'redirect_http_code': 302},
+            constants.L7POLICY_ACTION_REDIRECT_TO_URL: {
+                'redirect_url': 'http://www.example.com',
+                'redirect_http_code': 301}}
+        count = 1
+        # First, test with redirect actions
+        for action in [constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+                       constants.L7POLICY_ACTION_REDIRECT_PREFIX]:
+            api_l7policy = self.create_l7policy(
+                self.listener_id, action,
+                **action_key_values[action]).get(self.root_tag)
+            self.assertEqual(action, api_l7policy['action'])
+            self.assertEqual(count, api_l7policy['position'])
+            self.assertIsNone(api_l7policy.get('redirect_pool_id'))
+            if api_l7policy.get('redirect_url'):
+                self.assertEqual(action_key_values[action]['redirect_url'],
+                                 api_l7policy['redirect_url'])
+            elif api_l7policy.get('redirect_prefix'):
+                self.assertEqual(action_key_values[action]['redirect_prefix'],
+                                 api_l7policy['redirect_prefix'])
+            self.assertEqual(action_key_values[action]['redirect_http_code'],
+                             api_l7policy['redirect_http_code'])
+            self.assert_correct_status(
+                lb_id=self.lb_id, listener_id=self.listener_id,
+                l7policy_id=api_l7policy.get('id'),
+                lb_prov_status=constants.PENDING_UPDATE,
+                listener_prov_status=constants.PENDING_UPDATE,
+                l7policy_prov_status=constants.PENDING_CREATE,
+                l7policy_op_status=constants.OFFLINE)
+            self.set_lb_status(self.lb_id)
+            count += 1
+
+        # test with redirect_pool action
+        api_l7policy = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REDIRECT_TO_POOL,
+            redirect_pool_id=self.pool_id,
+            redirect_http_code=308).get(self.root_tag)
+        self.assertEqual(constants.L7POLICY_ACTION_REDIRECT_TO_POOL,
+                         api_l7policy['action'])
+        self.assertEqual(self.pool_id, api_l7policy.get('redirect_pool_id'))
+        self.assertIsNone(api_l7policy.get('redirect_url'))
+        self.assertIsNone(api_l7policy.get('redirect_prefix'))
+        self.assertIsNone(api_l7policy.get('redirect_http_code'))
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            l7policy_id=api_l7policy.get('id'),
+            lb_prov_status=constants.PENDING_UPDATE,
+            listener_prov_status=constants.PENDING_UPDATE,
+            l7policy_prov_status=constants.PENDING_CREATE,
+            l7policy_op_status=constants.OFFLINE)
+
     def test_bad_create(self):
         l7policy = {'listener_id': self.listener_id,
                     'name': 'test1'}
@@ -626,27 +824,21 @@ class TestL7Policy(base.BaseAPITest):
             'redirect_pool_id': uuidutils.generate_uuid()}
         self.post(self.L7POLICIES_PATH, self._build_body(l7policy), status=404)
 
-    def test_bad_create_redirect_to_udp_pool(self):
-        udp_pool_id = self.create_pool(
-            self.lb_id,
-            constants.PROTOCOL_UDP,
-            constants.LB_ALGORITHM_ROUND_ROBIN).get('pool').get('id')
-        l7policy = {
-            'action': constants.L7POLICY_ACTION_REDIRECT_TO_POOL,
-            'listener_id': self.listener_id,
-            'redirect_pool_id': udp_pool_id}
-        res = self.post(self.L7POLICIES_PATH, self._build_body(l7policy),
-                        status=400, expect_errors=True)
-        expect_error_msg = ("Validation failure: %s protocol pool can not be "
-                            "assigned to l7policy.") % constants.PROTOCOL_UDP
-        self.assertEqual(expect_error_msg, res.json['faultstring'])
-
     def test_bad_create_redirect_to_url(self):
         l7policy = {'listener_id': self.listener_id,
                     'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
                     'redirect_url': 'bad url'}
         self.post(self.L7POLICIES_PATH, self._build_body(l7policy), status=400)
 
+    def test_bad_create_with_redirect_http_code(self):
+        for test_code in [1, '', 'HTTPCODE']:
+            l7policy = {'listener_id': self.listener_id,
+                        'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+                        'redirect_url': 'http://www.example.com',
+                        'redirect_http_code': test_code}
+            self.post(self.L7POLICIES_PATH, self._build_body(l7policy),
+                      status=400)
+
     @mock.patch('octavia.api.drivers.utils.call_provider')
     def test_create_with_bad_provider(self, mock_provider):
         mock_provider.side_effect = exceptions.ProviderDriverError(
@@ -669,16 +861,19 @@ class TestL7Policy(base.BaseAPITest):
     def test_update(self):
         api_l7policy = self.create_l7policy(self.listener_id,
                                             constants.L7POLICY_ACTION_REJECT,
+                                            tags=['old_tag']
                                             ).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         new_l7policy = {
             'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
-            'redirect_url': 'http://www.example.com'}
+            'redirect_url': 'http://www.example.com',
+            'tags': ['new_tag']}
         response = self.put(self.L7POLICY_PATH.format(
             l7policy_id=api_l7policy.get('id')),
             self._build_body(new_l7policy)).json.get(self.root_tag)
         self.assertEqual(constants.L7POLICY_ACTION_REDIRECT_TO_URL,
                          response.get('action'))
+        self.assertEqual(['new_tag'], response['tags'])
         self.assert_correct_status(
             lb_id=self.lb_id, listener_id=self.listener_id,
             l7policy_id=api_l7policy.get('id'),
@@ -779,27 +974,6 @@ class TestL7Policy(base.BaseAPITest):
             l7policy_id=api_l7policy.get('id')),
             self._build_body(new_l7policy), status=400)
 
-    def test_bad_update_redirect_to_udp_pool(self):
-        api_l7policy = self.create_l7policy(self.listener_id,
-                                            constants.L7POLICY_ACTION_REJECT,
-                                            ).get(self.root_tag)
-        self.set_lb_status(self.lb_id)
-        udp_pool_id = self.create_pool(
-            self.lb_id,
-            constants.PROTOCOL_UDP,
-            constants.LB_ALGORITHM_ROUND_ROBIN).get('pool').get('id')
-        self.set_lb_status(self.lb_id)
-        new_l7policy = {
-            'action': constants.L7POLICY_ACTION_REDIRECT_TO_POOL,
-            'redirect_pool_id': udp_pool_id}
-        res = self.put(self.L7POLICY_PATH.format(
-            l7policy_id=api_l7policy.get('id')),
-            self._build_body(new_l7policy),
-            status=400, expect_errors=True)
-        expect_error_msg = ("Validation failure: %s protocol pool can not be "
-                            "assigned to l7policy.") % constants.PROTOCOL_UDP
-        self.assertEqual(expect_error_msg, res.json['faultstring'])
-
     def test_bad_update_redirect_to_url(self):
         api_l7policy = self.create_l7policy(self.listener_id,
                                             constants.L7POLICY_ACTION_REJECT,
@@ -866,6 +1040,61 @@ class TestL7Policy(base.BaseAPITest):
         self.put(self.L7POLICY_PATH.format(l7policy_id=api_l7policy.get('id')),
                  self._build_body(new_l7policy))
 
+    def test_update_with_redirect_http_code(self):
+        # test from non exist
+        api_l7policy = self.create_l7policy(self.listener_id,
+                                            constants.L7POLICY_ACTION_REJECT,
+                                            ).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_l7policy = {
+            'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+            'redirect_url': 'http://www.example.com',
+            'redirect_http_code': 308}
+        response = self.put(self.L7POLICY_PATH.format(
+            l7policy_id=api_l7policy.get('id')),
+            self._build_body(new_l7policy)).json.get(self.root_tag)
+        self.assertEqual(constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+                         response.get('action'))
+        self.assertEqual(308, response.get('redirect_http_code'))
+        self.set_lb_status(self.lb_id)
+
+        # test from exist to new
+        api_l7policy = self.create_l7policy(
+            self.listener_id, constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+            redirect_url='http://www.example.com',
+            redirect_http_code=302).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_l7policy = {
+            'redirect_http_code': 308}
+        response = self.put(self.L7POLICY_PATH.format(
+            l7policy_id=api_l7policy.get('id')),
+            self._build_body(new_l7policy)).json.get(self.root_tag)
+        self.assertEqual(constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+                         response.get('action'))
+        self.assertEqual(308, response.get('redirect_http_code'))
+        self.set_lb_status(self.lb_id)
+
+        # test from exist to null
+        new_l7policy = {
+            'redirect_http_code': None}
+        response = self.put(self.L7POLICY_PATH.format(
+            l7policy_id=api_l7policy.get('id')),
+            self._build_body(new_l7policy)).json.get(self.root_tag)
+        self.assertIsNone(response.get('redirect_http_code'))
+
+    def test_bad_update_with_redirect_http_code(self):
+        api_l7policy = self.create_l7policy(self.listener_id,
+                                            constants.L7POLICY_ACTION_REJECT,
+                                            ).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_l7policy = {
+            'action': constants.L7POLICY_ACTION_REDIRECT_TO_URL,
+            'redirect_url': 'http://www.example.com',
+            'redirect_http_code': ''}
+        self.put(self.L7POLICY_PATH.format(
+            l7policy_id=api_l7policy.get('id')),
+            self._build_body(new_l7policy), status=400).json.get(self.root_tag)
+
     def test_delete(self):
         api_l7policy = self.create_l7policy(
             self.listener_id,
@@ -1087,3 +1316,130 @@ class TestL7Policy(base.BaseAPITest):
         self.delete(self.L7POLICY_PATH.format(
                     l7policy_id=l7policy.get('id')),
                     status=404)
+
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_listener_pool_protocol_map_post(self, mock_cert_data):
+        cert = data_models.TLSContainer(certificate='cert')
+        mock_cert_data.return_value = {'sni_certs': [cert]}
+        valid_map = constants.VALID_LISTENER_POOL_PROTOCOL_MAP
+        port = 1
+        l7policy = {'action': constants.L7POLICY_ACTION_REDIRECT_TO_POOL}
+        for listener_proto in valid_map:
+            for pool_proto in valid_map[listener_proto]:
+                port = port + 1
+                opts = {}
+                if listener_proto == constants.PROTOCOL_TERMINATED_HTTPS:
+                    opts['sni_container_refs'] = [uuidutils.generate_uuid()]
+                listener = self.create_listener(
+                    listener_proto, port, self.lb_id, **opts).get('listener')
+                self.set_object_status(self.lb_repo, self.lb_id)
+                pool = self.create_pool(
+                    self.lb_id, pool_proto,
+                    constants.LB_ALGORITHM_ROUND_ROBIN).get('pool')
+
+                l7policy['listener_id'] = listener.get('id')
+                l7policy['redirect_pool_id'] = pool.get('id')
+                self.set_object_status(self.lb_repo, self.lb_id)
+                self.post(self.L7POLICIES_PATH,
+                          self._build_body(l7policy), status=201)
+                self.set_object_status(self.lb_repo, self.lb_id)
+
+        invalid_map = c_const.INVALID_LISTENER_POOL_PROTOCOL_MAP
+        port = 100
+        for listener_proto in invalid_map:
+            opts = {}
+            if listener_proto == constants.PROTOCOL_TERMINATED_HTTPS:
+                opts['sni_container_refs'] = [uuidutils.generate_uuid()]
+            listener = self.create_listener(
+                listener_proto, port, self.lb_id, **opts).get('listener')
+            self.set_object_status(self.lb_repo, self.lb_id)
+            port = port + 1
+            for pool_proto in invalid_map[listener_proto]:
+                if pool_proto == constants.PROTOCOL_TERMINATED_HTTPS:
+                    pool = self.create_pool(
+                        self.lb_id, pool_proto,
+                        constants.LB_ALGORITHM_ROUND_ROBIN, status=400)
+                    self.assertIn("Invalid input", pool['faultstring'])
+                else:
+                    pool = self.create_pool(
+                        self.lb_id, pool_proto,
+                        constants.LB_ALGORITHM_ROUND_ROBIN).get('pool')
+                    self.set_object_status(self.lb_repo, self.lb_id)
+
+                    l7policy['listener_id'] = listener.get('id')
+                    l7policy['redirect_pool_id'] = pool.get('id')
+                    expect_error_msg = (
+                        "Validation failure: The pool protocol '%s' is "
+                        "invalid while the listener protocol is '%s'.") % (
+                        pool_proto, listener_proto)
+                    res = self.post(self.L7POLICIES_PATH,
+                                    self._build_body(l7policy), status=400)
+                    self.assertEqual(expect_error_msg, res.json['faultstring'])
+                    self.assert_correct_status(lb_id=self.lb_id)
+
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_listener_pool_protocol_map_put(self, mock_cert_data):
+        cert = data_models.TLSContainer(certificate='cert')
+        mock_cert_data.return_value = {'sni_certs': [cert]}
+        valid_map = constants.VALID_LISTENER_POOL_PROTOCOL_MAP
+        port = 1
+        new_l7policy = {'action': constants.L7POLICY_ACTION_REDIRECT_TO_POOL}
+        for listener_proto in valid_map:
+            for pool_proto in valid_map[listener_proto]:
+                port = port + 1
+                opts = {}
+                if listener_proto == constants.PROTOCOL_TERMINATED_HTTPS:
+                    opts['sni_container_refs'] = [uuidutils.generate_uuid()]
+                listener = self.create_listener(
+                    listener_proto, port, self.lb_id, **opts).get('listener')
+                self.set_object_status(self.lb_repo, self.lb_id)
+                pool = self.create_pool(
+                    self.lb_id, pool_proto,
+                    constants.LB_ALGORITHM_ROUND_ROBIN).get('pool')
+                self.set_object_status(self.lb_repo, self.lb_id)
+                l7policy = self.create_l7policy(
+                    listener.get('id'),
+                    constants.L7POLICY_ACTION_REJECT).get(self.root_tag)
+                self.set_object_status(self.lb_repo, self.lb_id)
+                new_l7policy['redirect_pool_id'] = pool.get('id')
+
+                self.put(
+                    self.L7POLICY_PATH.format(l7policy_id=l7policy.get('id')),
+                    self._build_body(new_l7policy), status=200)
+                self.set_object_status(self.lb_repo, self.lb_id)
+
+        invalid_map = c_const.INVALID_LISTENER_POOL_PROTOCOL_MAP
+        port = 100
+        for listener_proto in invalid_map:
+            opts = {}
+            if listener_proto == constants.PROTOCOL_TERMINATED_HTTPS:
+                opts['sni_container_refs'] = [uuidutils.generate_uuid()]
+            listener = self.create_listener(
+                listener_proto, port, self.lb_id, **opts).get('listener')
+            self.set_object_status(self.lb_repo, self.lb_id)
+            port = port + 1
+            for pool_proto in invalid_map[listener_proto]:
+                if pool_proto == constants.PROTOCOL_TERMINATED_HTTPS:
+                    pool = self.create_pool(
+                        self.lb_id, pool_proto,
+                        constants.LB_ALGORITHM_ROUND_ROBIN, status=400)
+                    self.assertIn("Invalid input", pool['faultstring'])
+                else:
+                    pool = self.create_pool(
+                        self.lb_id, pool_proto,
+                        constants.LB_ALGORITHM_ROUND_ROBIN).get('pool')
+                    self.set_object_status(self.lb_repo, self.lb_id)
+                    l7policy = self.create_l7policy(
+                        listener.get('id'),
+                        constants.L7POLICY_ACTION_REJECT).get(self.root_tag)
+                    self.set_object_status(self.lb_repo, self.lb_id)
+                    new_l7policy['redirect_pool_id'] = pool.get('id')
+                    expect_error_msg = (
+                        "Validation failure: The pool protocol '%s' is "
+                        "invalid while the listener protocol is '%s'.") % (
+                        pool_proto, listener_proto)
+                    res = self.put(self.L7POLICY_PATH.format(
+                        l7policy_id=l7policy.get('id')),
+                        self._build_body(new_l7policy), status=400)
+                    self.assertEqual(expect_error_msg, res.json['faultstring'])
+                    self.assert_correct_status(lb_id=self.lb_id)
