@@ -12,17 +12,19 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
+from unittest import mock
+
+from octavia_lib.api.drivers import data_models as driver_dm
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
 
-from octavia.api.drivers import data_models as driver_dm
 from octavia.api.drivers import utils as driver_utils
 from octavia.common import constants
 import octavia.common.context
 from octavia.common import data_models
 from octavia.common import exceptions
+from octavia.db import repositories
 from octavia.network import base as network_base
 from octavia.tests.functional.api.v2 import base
 
@@ -34,7 +36,7 @@ class TestMember(base.BaseAPITest):
     root_tag_links = 'members_links'
 
     def setUp(self):
-        super(TestMember, self).setUp()
+        super().setUp()
         vip_subnet_id = uuidutils.generate_uuid()
         self.lb = self.create_load_balancer(vip_subnet_id)
         self.lb_id = self.lb.get('loadbalancer').get('id')
@@ -61,6 +63,7 @@ class TestMember(base.BaseAPITest):
         self.members_path_listener = self.MEMBERS_PATH.format(
             pool_id=self.pool_with_listener_id)
         self.member_path_listener = self.members_path_listener + '/{member_id}'
+        self.pool_repo = repositories.PoolRepository()
 
     def test_get(self):
         api_member = self.create_member(
@@ -69,6 +72,7 @@ class TestMember(base.BaseAPITest):
             member_id=api_member.get('id'))).json.get(self.root_tag)
         self.assertEqual(api_member, response)
         self.assertEqual(api_member.get('name'), '')
+        self.assertEqual([], api_member['tags'])
 
     def test_get_authorized(self):
         api_member = self.create_member(
@@ -128,10 +132,12 @@ class TestMember(base.BaseAPITest):
 
     def test_get_all(self):
         api_m_1 = self.create_member(
-            self.pool_id, '192.0.2.1', 80).get(self.root_tag)
+            self.pool_id, '192.0.2.1', 80,
+            tags=['test_tag1']).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         api_m_2 = self.create_member(
-            self.pool_id, '192.0.2.2', 80).get(self.root_tag)
+            self.pool_id, '192.0.2.2', 80,
+            tags=['test_tag2']).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         # Original objects didn't have the updated operating/provisioning
         # status that exists in the DB.
@@ -206,6 +212,45 @@ class TestMember(base.BaseAPITest):
         for m in [api_m_1, api_m_2]:
             self.assertIn(m, response)
 
+    def test_get_all_unscoped_token(self):
+        api_m_1 = self.create_member(
+            self.pool_id, '192.0.2.1', 80).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        api_m_2 = self.create_member(
+            self.pool_id, '192.0.2.2', 80).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        # Original objects didn't have the updated operating/provisioning
+        # status that exists in the DB.
+        for m in [api_m_1, api_m_2]:
+            m['operating_status'] = constants.ONLINE
+            m['provisioning_status'] = constants.ACTIVE
+            m.pop('updated_at')
+
+        self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
+        auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
+        self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
+        with mock.patch.object(octavia.common.context.Context, 'project_id',
+                               None):
+            override_credentials = {
+                'service_user_id': None,
+                'user_domain_id': None,
+                'is_admin_project': True,
+                'service_project_domain_id': None,
+                'service_project_id': None,
+                'roles': ['load-balancer_member'],
+                'user_id': None,
+                'is_admin': False,
+                'service_user_domain_id': None,
+                'project_domain_id': None,
+                'service_roles': [],
+                'project_id': None}
+            with mock.patch(
+                    "oslo_context.context.RequestContext.to_policy_values",
+                    return_value=override_credentials):
+                result = self.get(self.members_path, status=403).json
+        self.conf.config(group='api_settings', auth_strategy=auth_strategy)
+        self.assertEqual(self.NOT_AUTHORIZED_BODY, result)
+
     def test_get_all_not_authorized(self):
         api_m_1 = self.create_member(
             self.pool_id, '192.0.2.1', 80).get(self.root_tag)
@@ -288,7 +333,8 @@ class TestMember(base.BaseAPITest):
         links = middle[self.root_tag_links]
         self.assertEqual(1, len(objs))
         self.assertEqual(2, len(links))
-        self.assertItemsEqual(['previous', 'next'], [l['rel'] for l in links])
+        self.assertCountEqual(['previous', 'next'],
+                              [link['rel'] for link in links])
 
     def test_get_all_fields_filter(self):
         self.create_member(self.pool_id, '192.0.2.1', 80, name='member1')
@@ -342,6 +388,102 @@ class TestMember(base.BaseAPITest):
         self.assertEqual(mem1['id'],
                          members['members'][0]['id'])
 
+    def test_get_all_tags_filter(self):
+        mem1 = self.create_member(
+            self.pool_id,
+            '192.0.2.1',
+            80,
+            name='member1',
+            tags=['test_tag1', 'test_tag2']
+        ).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        mem2 = self.create_member(
+            self.pool_id,
+            '192.0.2.2',
+            80,
+            name='member2',
+            tags=['test_tag2', 'test_tag3']
+        ).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        mem3 = self.create_member(
+            self.pool_id,
+            '192.0.2.3',
+            80,
+            name='member3',
+            tags=['test_tag4', 'test_tag5']
+        ).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+
+        mems = self.get(
+            self.members_path,
+            params={'tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(mems, list)
+        self.assertEqual(2, len(mems))
+        self.assertEqual(
+            [mem1.get('id'), mem2.get('id')],
+            [mem.get('id') for mem in mems]
+        )
+
+        mems = self.get(
+            self.members_path,
+            params={'tags': ['test_tag2', 'test_tag3']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(mems, list)
+        self.assertEqual(1, len(mems))
+        self.assertEqual(
+            [mem2.get('id')],
+            [mem.get('id') for mem in mems]
+        )
+
+        mems = self.get(
+            self.members_path,
+            params={'tags-any': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(mems, list)
+        self.assertEqual(2, len(mems))
+        self.assertEqual(
+            [mem1.get('id'), mem2.get('id')],
+            [mem.get('id') for mem in mems]
+        )
+
+        mems = self.get(
+            self.members_path,
+            params={'not-tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(mems, list)
+        self.assertEqual(1, len(mems))
+        self.assertEqual(
+            [mem3.get('id')],
+            [mem.get('id') for mem in mems]
+        )
+
+        mems = self.get(
+            self.members_path,
+            params={'not-tags-any': ['test_tag2', 'test_tag4']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(mems, list)
+        self.assertEqual(0, len(mems))
+
+        mems = self.get(
+            self.members_path,
+            params={'tags': 'test_tag2',
+                    'tags-any': ['test_tag1', 'test_tag3']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(mems, list)
+        self.assertEqual(2, len(mems))
+        self.assertEqual(
+            [mem1.get('id'), mem2.get('id')],
+            [mem.get('id') for mem in mems]
+        )
+
+        mems = self.get(
+            self.members_path,
+            params={'tags': 'test_tag2', 'not-tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(mems, list)
+        self.assertEqual(0, len(mems))
+
     def test_empty_get_all(self):
         response = self.get(self.members_path).json.get(self.root_tag_list)
         self.assertIsInstance(response, list)
@@ -393,11 +535,13 @@ class TestMember(base.BaseAPITest):
                     return_value=override_credentials):
 
                 api_member = self.create_member(
-                    self.pool_id, '192.0.2.1', 80).get(self.root_tag)
+                    self.pool_id, '192.0.2.1', 80,
+                    tags=['test_tag']).get(self.root_tag)
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
 
         self.assertEqual('192.0.2.1', api_member['address'])
         self.assertEqual(80, api_member['protocol_port'])
+        self.assertEqual(['test_tag'], api_member['tags'])
         self.assertIsNotNone(api_member['created_at'])
         self.assertIsNone(api_member['updated_at'])
         self.assert_correct_status(
@@ -426,6 +570,23 @@ class TestMember(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, api_member)
 
+    def test_create_pool_in_error(self):
+        project_id = uuidutils.generate_uuid()
+        lb1 = self.create_load_balancer(uuidutils.generate_uuid(), name='lb1',
+                                        project_id=project_id)
+        lb1_id = lb1.get('loadbalancer').get('id')
+        self.set_lb_status(lb1_id)
+        pool1 = self.create_pool(
+            lb1_id, constants.PROTOCOL_HTTP,
+            constants.LB_ALGORITHM_ROUND_ROBIN).get('pool')
+        pool1_id = pool1.get('id')
+        self.set_lb_status(lb1_id)
+        self.set_object_status(self.pool_repo, pool1_id,
+                               provisioning_status=constants.ERROR)
+        api_member = self.create_member(pool1_id, '192.0.2.1', 80, status=409)
+        ref_msg = 'Pool %s is immutable and cannot be updated.' % pool1_id
+        self.assertEqual(ref_msg, api_member.get('faultstring'))
+
     # TODO(rm_work) Remove after deprecation of project_id in POST (R series)
     def test_create_with_project_id_is_ignored(self):
         pid = uuidutils.generate_uuid()
@@ -462,12 +623,18 @@ class TestMember(base.BaseAPITest):
         mock_driver.name = 'noop_driver'
         mock_get_driver.return_value = mock_driver
 
-        member1 = {'address': '192.0.2.1', 'protocol_port': 80}
-        member2 = {'address': '192.0.2.2', 'protocol_port': 80}
-        member3 = {'address': '192.0.2.3', 'protocol_port': 80}
-        member4 = {'address': '192.0.2.4', 'protocol_port': 80}
-        member5 = {'address': '192.0.2.5', 'protocol_port': 80}
-        member6 = {'address': '192.0.2.6', 'protocol_port': 80}
+        member1 = {'address': '192.0.2.1', 'protocol_port': 80,
+                   'project_id': self.project_id}
+        member2 = {'address': '192.0.2.2', 'protocol_port': 80,
+                   'project_id': self.project_id}
+        member3 = {'address': '192.0.2.3', 'protocol_port': 80,
+                   'project_id': self.project_id}
+        member4 = {'address': '192.0.2.4', 'protocol_port': 80,
+                   'project_id': self.project_id}
+        member5 = {'address': '192.0.2.5', 'protocol_port': 80,
+                   'project_id': self.project_id}
+        member6 = {'address': '192.0.2.6', 'protocol_port': 80,
+                   'project_id': self.project_id}
         members = [member1, member2, member3, member4]
         for m in members:
             self.create_member(pool_id=self.pool_id, **m)
@@ -494,7 +661,6 @@ class TestMember(base.BaseAPITest):
             ('192.0.2.6', 80, 'PENDING_CREATE'),
         ]
 
-        member_ids = {}
         provider_creates = []
         provider_updates = []
         for rm in returned_members:
@@ -502,16 +668,15 @@ class TestMember(base.BaseAPITest):
                 (rm['address'],
                  rm['protocol_port'],
                  rm['provisioning_status']), expected_members)
-            member_ids[(rm['address'], rm['protocol_port'])] = rm['id']
 
             provider_dict = driver_utils.member_dict_to_provider_dict(rm)
             # Adjust for API response
+            provider_dict['pool_id'] = self.pool_id
             if rm['provisioning_status'] == 'PENDING_UPDATE':
                 del provider_dict['name']
                 del provider_dict['subnet_id']
                 provider_updates.append(driver_dm.Member(**provider_dict))
             elif rm['provisioning_status'] == 'PENDING_CREATE':
-                provider_dict['pool_id'] = self.pool_id
                 provider_dict['name'] = None
                 provider_creates.append(driver_dm.Member(**provider_dict))
         # Order matters here
@@ -519,7 +684,7 @@ class TestMember(base.BaseAPITest):
 
         mock_provider.assert_called_once_with(u'noop_driver',
                                               mock_driver.member_batch_update,
-                                              provider_creates)
+                                              self.pool_id, provider_creates)
 
     @mock.patch('octavia.api.drivers.driver_factory.get_driver')
     @mock.patch('octavia.api.drivers.utils.call_provider')
@@ -528,8 +693,10 @@ class TestMember(base.BaseAPITest):
         mock_driver.name = 'noop_driver'
         mock_get_driver.return_value = mock_driver
 
-        member5 = {'address': '192.0.2.5', 'protocol_port': 80}
-        member6 = {'address': '192.0.2.6', 'protocol_port': 80}
+        member5 = {'address': '192.0.2.5', 'protocol_port': 80,
+                   'tags': ['test_tag1']}
+        member6 = {'address': '192.0.2.6', 'protocol_port': 80,
+                   'tags': ['test_tag2']}
 
         req_dict = [member5, member6]
         body = {self.root_tag_list: req_dict}
@@ -540,18 +707,17 @@ class TestMember(base.BaseAPITest):
         ).json.get(self.root_tag_list)
 
         expected_members = [
-            ('192.0.2.5', 80, 'PENDING_CREATE'),
-            ('192.0.2.6', 80, 'PENDING_CREATE'),
+            ('192.0.2.5', 80, 'PENDING_CREATE', ['test_tag1']),
+            ('192.0.2.6', 80, 'PENDING_CREATE', ['test_tag2']),
         ]
 
-        member_ids = {}
         provider_members = []
         for rm in returned_members:
             self.assertIn(
                 (rm['address'],
                  rm['protocol_port'],
-                 rm['provisioning_status']), expected_members)
-            member_ids[(rm['address'], rm['protocol_port'])] = rm['id']
+                 rm['provisioning_status'],
+                 rm['tags']), expected_members)
 
             provider_dict = driver_utils.member_dict_to_provider_dict(rm)
             # Adjust for API response
@@ -561,7 +727,7 @@ class TestMember(base.BaseAPITest):
 
         mock_provider.assert_called_once_with(u'noop_driver',
                                               mock_driver.member_batch_update,
-                                              provider_members)
+                                              self.pool_id, provider_members)
 
     def test_create_batch_members_with_bad_subnet(self):
         subnet_id = uuidutils.generate_uuid()
@@ -581,6 +747,90 @@ class TestMember(base.BaseAPITest):
             err_msg = 'Subnet ' + subnet_id + ' not found.'
             self.assertEqual(response.get('faultstring'), err_msg)
 
+    def test_create_batch_members_with_invalid_address(self):
+        # 169.254.169.254 is the default invalid member address
+        member5 = {'address': '169.254.169.254',
+                   'protocol_port': 80}
+
+        req_dict = [member5]
+        body = {self.root_tag_list: req_dict}
+        path = self.MEMBERS_PATH.format(pool_id=self.pool_id)
+
+        response = self.put(path, body, status=400).json
+        err_msg = ("169.254.169.254 is not a valid option for member address")
+        self.assertEqual(err_msg, response.get('faultstring'))
+
+    @mock.patch('octavia.api.drivers.driver_factory.get_driver')
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_additive_only_batch_members(self, mock_provider, mock_get_driver):
+        mock_driver = mock.MagicMock()
+        mock_driver.name = 'noop_driver'
+        mock_get_driver.return_value = mock_driver
+
+        member1 = {'address': '192.0.2.1', 'protocol_port': 80}
+        member2 = {'address': '192.0.2.2', 'protocol_port': 80}
+        member3 = {'address': '192.0.2.3', 'protocol_port': 80}
+        member4 = {'address': '192.0.2.4', 'protocol_port': 80}
+        member5 = {'address': '192.0.2.5', 'protocol_port': 80}
+        member6 = {'address': '192.0.2.6', 'protocol_port': 80}
+        members = [member1, member2, member3, member4]
+        for m in members:
+            self.create_member(pool_id=self.pool_id, **m)
+            self.set_lb_status(self.lb_id)
+
+        # We are only concerned about the batch update, so clear out the
+        # create members calls above.
+        mock_provider.reset_mock()
+
+        req_dict = [member1, member2, member5, member6]
+        body = {self.root_tag_list: req_dict}
+        path = self.MEMBERS_PATH.format(pool_id=self.pool_id)
+        path = "{}?additive_only=True".format(path)
+        self.put(path, body, status=202)
+        returned_members = self.get(
+            self.MEMBERS_PATH.format(pool_id=self.pool_id)
+        ).json.get(self.root_tag_list)
+
+        # Members 1+2 should be updated, 3+4 left alone, and 5+6 created
+        expected_members = [
+            ('192.0.2.1', 80, 'PENDING_UPDATE'),
+            ('192.0.2.2', 80, 'PENDING_UPDATE'),
+            ('192.0.2.3', 80, 'ACTIVE'),
+            ('192.0.2.4', 80, 'ACTIVE'),
+            ('192.0.2.5', 80, 'PENDING_CREATE'),
+            ('192.0.2.6', 80, 'PENDING_CREATE'),
+        ]
+
+        provider_creates = []
+        provider_updates = []
+        provider_ignored = []
+        for rm in returned_members:
+            self.assertIn(
+                (rm['address'],
+                 rm['protocol_port'],
+                 rm['provisioning_status']), expected_members)
+
+            provider_dict = driver_utils.member_dict_to_provider_dict(rm)
+            # Adjust for API response
+            provider_dict['pool_id'] = self.pool_id
+            if rm['provisioning_status'] == 'PENDING_UPDATE':
+                del provider_dict['name']
+                del provider_dict['subnet_id']
+                provider_updates.append(driver_dm.Member(**provider_dict))
+            elif rm['provisioning_status'] == 'PENDING_CREATE':
+                provider_dict['name'] = None
+                provider_creates.append(driver_dm.Member(**provider_dict))
+            elif rm['provisioning_status'] == 'ACTIVE':
+                provider_dict['name'] = None
+                provider_ignored.append(driver_dm.Member(**provider_dict))
+        # Order matters here
+        provider_creates += provider_updates
+        provider_creates += provider_ignored
+
+        mock_provider.assert_called_once_with(u'noop_driver',
+                                              mock_driver.member_batch_update,
+                                              self.pool_id, provider_creates)
+
     @mock.patch('octavia.api.drivers.driver_factory.get_driver')
     @mock.patch('octavia.api.drivers.utils.call_provider')
     def test_update_batch_members(self, mock_provider, mock_get_driver):
@@ -588,8 +838,10 @@ class TestMember(base.BaseAPITest):
         mock_driver.name = 'noop_driver'
         mock_get_driver.return_value = mock_driver
 
-        member1 = {'address': '192.0.2.1', 'protocol_port': 80}
-        member2 = {'address': '192.0.2.2', 'protocol_port': 80}
+        member1 = {'address': '192.0.2.1', 'protocol_port': 80,
+                   'project_id': self.project_id}
+        member2 = {'address': '192.0.2.2', 'protocol_port': 80,
+                   'project_id': self.project_id}
         members = [member1, member2]
         for m in members:
             self.create_member(pool_id=self.pool_id, **m)
@@ -612,24 +864,23 @@ class TestMember(base.BaseAPITest):
             ('192.0.2.2', 80, 'PENDING_UPDATE'),
         ]
 
-        member_ids = {}
         provider_members = []
         for rm in returned_members:
             self.assertIn(
                 (rm['address'],
                  rm['protocol_port'],
                  rm['provisioning_status']), expected_members)
-            member_ids[(rm['address'], rm['protocol_port'])] = rm['id']
 
             provider_dict = driver_utils.member_dict_to_provider_dict(rm)
             # Adjust for API response
+            provider_dict['pool_id'] = self.pool_id
             del provider_dict['name']
             del provider_dict['subnet_id']
             provider_members.append(driver_dm.Member(**provider_dict))
 
         mock_provider.assert_called_once_with(u'noop_driver',
                                               mock_driver.member_batch_update,
-                                              provider_members)
+                                              self.pool_id, provider_members)
 
     @mock.patch('octavia.api.drivers.driver_factory.get_driver')
     @mock.patch('octavia.api.drivers.utils.call_provider')
@@ -662,18 +913,38 @@ class TestMember(base.BaseAPITest):
             ('192.0.2.4', 80, 'PENDING_DELETE'),
         ]
 
-        member_ids = {}
         provider_members = []
         for rm in returned_members:
             self.assertIn(
                 (rm['address'],
                  rm['protocol_port'],
                  rm['provisioning_status']), expected_members)
-            member_ids[(rm['address'], rm['protocol_port'])] = rm['id']
 
         mock_provider.assert_called_once_with(u'noop_driver',
                                               mock_driver.member_batch_update,
-                                              provider_members)
+                                              self.pool_id, provider_members)
+
+    @mock.patch('octavia.api.drivers.driver_factory.get_driver')
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_delete_batch_members_already_empty(self, mock_provider,
+                                                mock_get_driver):
+        mock_driver = mock.MagicMock()
+        mock_driver.name = 'noop_driver'
+        mock_get_driver.return_value = mock_driver
+
+        req_dict = []
+        body = {self.root_tag_list: req_dict}
+        path = self.MEMBERS_PATH.format(pool_id=self.pool_id)
+        self.put(path, body, status=202)
+        returned_members = self.get(
+            self.MEMBERS_PATH.format(pool_id=self.pool_id)
+        ).json.get(self.root_tag_list)
+
+        self.assertEqual([], returned_members)
+
+        mock_provider.assert_called_once_with(u'noop_driver',
+                                              mock_driver.member_batch_update,
+                                              self.pool_id, [])
 
     def test_create_with_attached_listener(self):
         api_member = self.create_member(
@@ -811,9 +1082,9 @@ class TestMember(base.BaseAPITest):
         new_name = "name2"
         api_member = self.create_member(
             self.pool_with_listener_id, '192.0.2.1', 80,
-            name=old_name).get(self.root_tag)
+            name=old_name, tags=['old_tag']).get(self.root_tag)
         self.set_lb_status(self.lb_id)
-        new_member = {'name': new_name}
+        new_member = {'name': new_name, 'tags': ['new_tag']}
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
@@ -852,6 +1123,7 @@ class TestMember(base.BaseAPITest):
             member_prov_status=constants.PENDING_UPDATE)
         self.set_lb_status(self.lb_id)
         self.assertEqual(new_name, response.get('name'))
+        self.assertEqual(['new_tag'], response['tags'])
         self.assertEqual(api_member.get('created_at'),
                          response.get('created_at'))
         self.assert_correct_status(
@@ -936,6 +1208,26 @@ class TestMember(base.BaseAPITest):
         self.assertIn('Provider \'bad_driver\' reports error: broken',
                       response.json.get('faultstring'))
 
+    def test_update_unset_defaults(self):
+        old_name = "name1"
+        api_member = self.create_member(
+            self.pool_with_listener_id, '192.0.2.1', 80,
+            name=old_name, backup=True, monitor_address='192.0.2.2',
+            monitor_port=8888, weight=10).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        unset_params = {'name': None, 'backup': None, 'monitor_address': None,
+                        'monitor_port': None, 'weight': None}
+        member_path = self.member_path_listener.format(
+            member_id=api_member.get('id'))
+        response = self.put(member_path, self._build_body(unset_params))
+        response = response.json.get(self.root_tag)
+
+        self.assertFalse(response['backup'])
+        self.assertIsNone(response['monitor_address'])
+        self.assertIsNone(response['monitor_port'])
+        self.assertEqual('', response['name'])
+        self.assertEqual(constants.DEFAULT_WEIGHT, response['weight'])
+
     def test_delete(self):
         api_member = self.create_member(
             self.pool_with_listener_id, '192.0.2.1', 80).get(self.root_tag)
@@ -1043,6 +1335,23 @@ class TestMember(base.BaseAPITest):
         self.delete(self.member_path.format(
             member_id=uuidutils.generate_uuid()), status=404)
 
+    def test_delete_mismatch_pool(self):
+        # Create a pool that will not have the member, but is valid.
+        self.pool = self.create_pool(self.lb_id, constants.PROTOCOL_HTTP,
+                                     constants.LB_ALGORITHM_ROUND_ROBIN)
+        bad_pool_id = self.pool.get('pool').get('id')
+        self.set_lb_status(self.lb_id)
+        # Create a member on our reference pool
+        api_member = self.create_member(
+            self.pool_with_listener_id, '192.0.2.1', 80).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        # Attempt to delete the member using the wrong pool in the path
+        member_path = self.MEMBERS_PATH.format(
+            pool_id=bad_pool_id) + '/' + api_member['id']
+        result = self.delete(member_path, status=404).json
+        ref_msg = 'Member %s not found.' % api_member['id']
+        self.assertEqual(ref_msg, result.get('faultstring'))
+
     @mock.patch('octavia.api.drivers.utils.call_provider')
     def test_delete_with_bad_provider(self, mock_provider):
         api_member = self.create_member(
