@@ -13,8 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import random
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
@@ -33,7 +33,7 @@ class TestQuotas(base.BaseAPITest):
     root_tag_links = 'quotas_links'
 
     def setUp(self):
-        super(TestQuotas, self).setUp()
+        super().setUp()
         conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         conf.config(
             group="quotas",
@@ -54,6 +54,14 @@ class TestQuotas(base.BaseAPITest):
             group="quotas",
             default_health_monitor_quota=random.randrange(
                 constants.QUOTA_UNLIMITED, 9000))
+        conf.config(
+            group="quotas",
+            default_l7policy_quota=random.randrange(
+                constants.QUOTA_UNLIMITED, 9000))
+        conf.config(
+            group="quotas",
+            default_l7rule_quota=random.randrange(
+                constants.QUOTA_UNLIMITED, 9000))
 
         self.project_id = uuidutils.generate_uuid()
 
@@ -65,13 +73,17 @@ class TestQuotas(base.BaseAPITest):
                         'pool': CONF.quotas.default_pool_quota,
                         'health_monitor':
                         CONF.quotas.default_health_monitor_quota,
-                        'member': CONF.quotas.default_member_quota}
+                        'member': CONF.quotas.default_member_quota,
+                        'l7policy': CONF.quotas.default_l7policy_quota,
+                        'l7rule': CONF.quotas.default_l7rule_quota}
         self.assertEqual(expected['load_balancer'], observed['load_balancer'])
         self.assertEqual(expected['listener'], observed['listener'])
         self.assertEqual(expected['pool'], observed['pool'])
         self.assertEqual(expected['health_monitor'],
                          observed['health_monitor'])
         self.assertEqual(expected['member'], observed['member'])
+        self.assertEqual(expected['l7policy'], observed['l7policy'])
+        self.assertEqual(expected['l7rule'], observed['l7rule'])
 
     def test_get_all_quotas_no_quotas(self):
         response = self.get(self.QUOTAS_PATH)
@@ -83,12 +95,14 @@ class TestQuotas(base.BaseAPITest):
         project_id2 = uuidutils.generate_uuid()
         quota_path1 = self.QUOTA_PATH.format(project_id=project_id1)
         quota1 = {'load_balancer': constants.QUOTA_UNLIMITED, 'listener': 30,
-                  'pool': 30, 'health_monitor': 30, 'member': 30}
+                  'pool': 30, 'health_monitor': 30, 'member': 30,
+                  'l7policy': 30, 'l7rule': 30}
         body1 = {'quota': quota1}
         self.put(quota_path1, body1, status=202)
         quota_path2 = self.QUOTA_PATH.format(project_id=project_id2)
         quota2 = {'load_balancer': 50, 'listener': 50, 'pool': 50,
-                  'health_monitor': 50, 'member': 50}
+                  'health_monitor': 50, 'member': 50, 'l7policy': 50,
+                  'l7rule': 50}
         body2 = {'quota': quota2}
         self.put(quota_path2, body2, status=202)
 
@@ -111,12 +125,14 @@ class TestQuotas(base.BaseAPITest):
         project_id2 = uuidutils.generate_uuid()
         quota_path1 = self.QUOTA_PATH.format(project_id=project_id1)
         quota1 = {'load_balancer': constants.QUOTA_UNLIMITED, 'listener': 30,
-                  'pool': 30, 'health_monitor': 30, 'member': 30}
+                  'pool': 30, 'health_monitor': 30, 'member': 30,
+                  'l7policy': 30, 'l7rule': 30}
         body1 = {'quota': quota1}
         self.put(quota_path1, body1, status=202)
         quota_path2 = self.QUOTA_PATH.format(project_id=project_id2)
         quota2 = {'loadbalancer': 50, 'listener': 50, 'pool': 50,
-                  'healthmonitor': 50, 'member': 50}
+                  'healthmonitor': 50, 'member': 50, 'l7policy': 50,
+                  'l7rule': 50}
         body2 = {'quota': quota2}
         self.put(quota_path2, body2, status=202)
 
@@ -139,12 +155,14 @@ class TestQuotas(base.BaseAPITest):
         project_id2 = uuidutils.generate_uuid()
         quota_path1 = self.QUOTA_PATH.format(project_id=project_id1)
         quota1 = {'load_balancer': constants.QUOTA_UNLIMITED, 'listener': 30,
-                  'pool': 30, 'health_monitor': 30, 'member': 30}
+                  'pool': 30, 'health_monitor': 30, 'member': 30,
+                  'l7policy': 30, 'l7rule': 30}
         body1 = {'quota': quota1}
         self.put(quota_path1, body1, status=202)
         quota_path2 = self.QUOTA_PATH.format(project_id=project_id2)
         quota2 = {'load_balancer': 50, 'listener': 50, 'pool': 50,
-                  'health_monitor': 50, 'member': 50}
+                  'health_monitor': 50, 'member': 50, 'l7policy': 50,
+                  'l7rule': 50}
         body2 = {'quota': quota2}
         self.put(quota_path2, body2, status=202)
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
@@ -152,7 +170,7 @@ class TestQuotas(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
         with mock.patch.object(octavia.common.context.Context, 'project_id',
                                uuidutils.generate_uuid()):
-                response = self.get(self.QUOTAS_PATH, status=403)
+            response = self.get(self.QUOTAS_PATH, status=403)
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, response.json)
 
@@ -160,7 +178,8 @@ class TestQuotas(base.BaseAPITest):
         project_id1 = uuidutils.generate_uuid()
         quota_path1 = self.QUOTA_PATH.format(project_id=project_id1)
         quota1 = {'load_balancer': constants.QUOTA_UNLIMITED, 'listener': 30,
-                  'pool': 30, 'health_monitor': 30, 'member': 30}
+                  'pool': 30, 'health_monitor': 30, 'member': 30,
+                  'l7policy': 30, 'l7rule': 30}
         body1 = {'quota': quota1}
         self.put(quota_path1, body1, status=202)
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
@@ -193,12 +212,14 @@ class TestQuotas(base.BaseAPITest):
         project_id2 = uuidutils.generate_uuid()
         quota_path1 = self.QUOTA_PATH.format(project_id=project_id1)
         quota1 = {'load_balancer': constants.QUOTA_UNLIMITED, 'listener': 30,
-                  'pool': 30, 'health_monitor': 30, 'member': 30}
+                  'pool': 30, 'health_monitor': 30, 'member': 30,
+                  'l7policy': 30, 'l7rule': 30}
         body1 = {'quota': quota1}
         self.put(quota_path1, body1, status=202)
         quota_path2 = self.QUOTA_PATH.format(project_id=project_id2)
         quota2 = {'load_balancer': 50, 'listener': 50, 'pool': 50,
-                  'health_monitor': 50, 'member': 50}
+                  'health_monitor': 50, 'member': 50, 'l7policy': 50,
+                  'l7rule': 50}
         body2 = {'quota': quota2}
         self.put(quota_path2, body2, status=202)
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
@@ -241,8 +262,8 @@ class TestQuotas(base.BaseAPITest):
         ).get(self.root_tag)
         quotas = self.get(self.QUOTAS_PATH).json.get(self.root_tag_list)
         self.assertEqual(3, len(quotas))
-        quota_lb_member_quotas = [(l.get('load_balancer'), l.get('member'))
-                                  for l in quotas]
+        quota_lb_member_quotas = [(lb.get('load_balancer'), lb.get('member'))
+                                  for lb in quotas]
         self.assertIn((quota1.get('load_balancer'), quota1.get('member')),
                       quota_lb_member_quotas)
         self.assertIn((quota2.get('load_balancer'), quota2.get('member')),
@@ -287,8 +308,8 @@ class TestQuotas(base.BaseAPITest):
                 quotas = quotas.json.get(self.root_tag_list)
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(3, len(quotas))
-        quota_lb_member_quotas = [(l.get('load_balancer'), l.get('member'))
-                                  for l in quotas]
+        quota_lb_member_quotas = [(lb.get('load_balancer'), lb.get('member'))
+                                  for lb in quotas]
         self.assertIn((quota1.get('load_balancer'), quota1.get('member')),
                       quota_lb_member_quotas)
         self.assertIn((quota2.get('load_balancer'), quota2.get('member')),
@@ -333,8 +354,8 @@ class TestQuotas(base.BaseAPITest):
                 quotas = quotas.json.get(self.root_tag_list)
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(3, len(quotas))
-        quota_lb_member_quotas = [(l.get('load_balancer'), l.get('member'))
-                                  for l in quotas]
+        quota_lb_member_quotas = [(lb.get('load_balancer'), lb.get('member'))
+                                  for lb in quotas]
         self.assertIn((quota1.get('load_balancer'), quota1.get('member')),
                       quota_lb_member_quotas)
         self.assertIn((quota2.get('load_balancer'), quota2.get('member')),
@@ -382,8 +403,8 @@ class TestQuotas(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
 
         self.assertEqual(1, len(quotas))
-        quota_lb_member_quotas = [(l.get('load_balancer'), l.get('member'))
-                                  for l in quotas]
+        quota_lb_member_quotas = [(lb.get('load_balancer'), lb.get('member'))
+                                  for lb in quotas]
         self.assertIn((quota3.get('load_balancer'), quota3.get('member')),
                       quota_lb_member_quotas)
 
@@ -427,8 +448,8 @@ class TestQuotas(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
 
         self.assertEqual(1, len(quotas))
-        quota_lb_member_quotas = [(l.get('load_balancer'), l.get('member'))
-                                  for l in quotas]
+        quota_lb_member_quotas = [(lb.get('load_balancer'), lb.get('member'))
+                                  for lb in quotas]
         self.assertIn((quota3.get('load_balancer'), quota3.get('member')),
                       quota_lb_member_quotas)
 
@@ -601,10 +622,10 @@ class TestQuotas(base.BaseAPITest):
         self.assertEqual(3, len(quotas_desc))
         self.assertEqual(3, len(quotas_asc))
 
-        quota_lb_member_desc = [(l.get('load_balancer'), l.get('member'))
-                                for l in quotas_desc]
-        quota_lb_member_asc = [(l.get('load_balancer'), l.get('member'))
-                               for l in quotas_asc]
+        quota_lb_member_desc = [(lb.get('load_balancer'), lb.get('member'))
+                                for lb in quotas_desc]
+        quota_lb_member_asc = [(lb.get('load_balancer'), lb.get('member'))
+                               for lb in quotas_asc]
         self.assertEqual(quota_lb_member_asc,
                          list(reversed(quota_lb_member_desc)))
 
@@ -650,7 +671,8 @@ class TestQuotas(base.BaseAPITest):
         links = middle[self.root_tag_links]
         self.assertEqual(1, len(objs))
         self.assertEqual(2, len(links))
-        self.assertItemsEqual(['previous', 'next'], [l['rel'] for l in links])
+        self.assertCountEqual(['previous', 'next'],
+                              [link['rel'] for link in links])
 
     def test_get_default_quotas(self):
         response = self.get(self.QUOTA_DEFAULT_PATH.format(
@@ -700,7 +722,8 @@ class TestQuotas(base.BaseAPITest):
     def test_custom_quotas(self):
         quota_path = self.QUOTA_PATH.format(project_id=self.project_id)
         body = {'quota': {'load_balancer': 30, 'listener': 30, 'pool': 30,
-                          'health_monitor': 30, 'member': 30}}
+                          'health_monitor': 30, 'member': 30,
+                          'l7policy': 30, 'l7rule': 30}}
         self.put(quota_path, body, status=202)
         response = self.get(quota_path)
         quota_dict = response.json
@@ -709,7 +732,8 @@ class TestQuotas(base.BaseAPITest):
     def test_custom_quotas_quota_admin(self):
         quota_path = self.QUOTA_PATH.format(project_id=self.project_id)
         body = {'quota': {'load_balancer': 30, 'listener': 30, 'pool': 30,
-                          'health_monitor': 30, 'member': 30}}
+                          'health_monitor': 30, 'member': 30, 'l7policy': 30,
+                          'l7rule': 30}}
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
@@ -740,7 +764,8 @@ class TestQuotas(base.BaseAPITest):
     def test_custom_quotas_not_Authorized_member(self):
         quota_path = self.QUOTA_PATH.format(project_id=self.project_id)
         body = {'quota': {'load_balancer': 30, 'listener': 30, 'pool': 30,
-                          'health_monitor': 30, 'member': 30}}
+                          'health_monitor': 30, 'member': 30, 'l7policy': 30,
+                          'l7rule': 30}}
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
@@ -769,11 +794,12 @@ class TestQuotas(base.BaseAPITest):
     def test_custom_partial_quotas(self):
         quota_path = self.QUOTA_PATH.format(project_id=self.project_id)
         body = {'quota': {'load_balancer': 30, 'listener': None, 'pool': 30,
-                          'health_monitor': 30, 'member': 30}}
+                          'health_monitor': 30, 'member': 30, 'l7policy': 30,
+                          'l7rule': 30}}
         expected_body = {'quota': {
             'load_balancer': 30,
             'listener': CONF.quotas.default_listener_quota, 'pool': 30,
-            'health_monitor': 30, 'member': 30}}
+            'health_monitor': 30, 'member': 30, 'l7policy': 30, 'l7rule': 30}}
         self.put(quota_path, body, status=202)
         response = self.get(quota_path)
         quota_dict = response.json
@@ -783,11 +809,12 @@ class TestQuotas(base.BaseAPITest):
     def test_custom_missing_quotas(self):
         quota_path = self.QUOTA_PATH.format(project_id=self.project_id)
         body = {'quota': {'load_balancer': 30, 'pool': 30,
-                          'health_monitor': 30, 'member': 30}}
+                          'health_monitor': 30, 'member': 30,
+                          'l7policy': 30, 'l7rule': 30}}
         expected_body = {'quota': {
             'load_balancer': 30,
             'listener': CONF.quotas.default_listener_quota, 'pool': 30,
-            'health_monitor': 30, 'member': 30}}
+            'health_monitor': 30, 'member': 30, 'l7policy': 30, 'l7rule': 30}}
         self.put(quota_path, body, status=202)
         response = self.get(quota_path)
         quota_dict = response.json
@@ -797,7 +824,8 @@ class TestQuotas(base.BaseAPITest):
     def test_delete_custom_quotas(self):
         quota_path = self.QUOTA_PATH.format(project_id=self.project_id)
         body = {'quota': {'load_balancer': 30, 'listener': 30, 'pool': 30,
-                          'health_monitor': 30, 'member': 30}}
+                          'health_monitor': 30, 'member': 30, 'l7policy': 30,
+                          'l7rule': 30}}
         self.put(quota_path, body, status=202)
         response = self.get(quota_path)
         quota_dict = response.json
@@ -810,7 +838,8 @@ class TestQuotas(base.BaseAPITest):
     def test_delete_custom_quotas_admin(self):
         quota_path = self.QUOTA_PATH.format(project_id=self.project_id)
         body = {'quota': {'load_balancer': 30, 'listener': 30, 'pool': 30,
-                          'health_monitor': 30, 'member': 30}}
+                          'health_monitor': 30, 'member': 30, 'l7policy': 30,
+                          'l7rule': 30}}
         self.put(quota_path, body, status=202)
         response = self.get(quota_path)
         quota_dict = response.json
@@ -845,7 +874,8 @@ class TestQuotas(base.BaseAPITest):
     def test_delete_quotas_not_Authorized_member(self):
         quota_path = self.QUOTA_PATH.format(project_id=self.project_id)
         body = {'quota': {'load_balancer': 30, 'listener': 30, 'pool': 30,
-                          'health_monitor': 30, 'member': 30}}
+                          'health_monitor': 30, 'member': 30, 'l7policy': 30,
+                          'l7rule': 30}}
         self.put(quota_path, body, status=202)
         response = self.get(quota_path)
         quota_dict = response.json
