@@ -11,8 +11,8 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+from unittest import mock
 
-import mock
 from oslo_utils import uuidutils
 
 from octavia.amphorae.drivers.noop_driver import driver
@@ -25,31 +25,17 @@ from octavia.tests.unit import base
 FAKE_UUID_1 = uuidutils.generate_uuid()
 
 
-class TestLoggingUpdate(base.TestCase):
-    def setUp(self):
-        super(TestLoggingUpdate, self).setUp()
-        self.mixin = driver.LoggingUpdate()
-
-    def test_update_stats(self):
-        self.mixin.update_stats('test update stats')
-        self.assertEqual('test update stats', self.mixin.stats)
-
-    def test_update_health(self):
-        self.mixin.update_health('test update health')
-        self.assertEqual('test update health', self.mixin.health)
-
-
 class TestNoopAmphoraLoadBalancerDriver(base.TestCase):
     FAKE_UUID_1 = uuidutils.generate_uuid()
 
     def setUp(self):
-        super(TestNoopAmphoraLoadBalancerDriver, self).setUp()
+        super().setUp()
         self.driver = driver.NoopAmphoraLoadBalancerDriver()
         self.listener = data_models.Listener()
         self.listener.id = uuidutils.generate_uuid()
         self.listener.protocol_port = 80
         self.vip = data_models.Vip()
-        self.vip.ip_address = "10.0.0.1"
+        self.vip.ip_address = "192.51.100.1"
         self.amphora = data_models.Amphora()
         self.amphora.id = self.FAKE_UUID_1
         self.load_balancer = data_models.LoadBalancer(
@@ -95,6 +81,14 @@ class TestNoopAmphoraLoadBalancerDriver(base.TestCase):
                          self.driver.driver.amphoraconfig[(
                              self.load_balancer.id, '321')])
 
+    def test_reload(self):
+        mock_amphora = mock.MagicMock()
+        mock_amphora.id = '321'
+        self.driver.reload(self.load_balancer, amphora=mock_amphora)
+        self.assertEqual((self.load_balancer, mock_amphora, 'reload'),
+                         self.driver.driver.amphoraconfig[(
+                             self.load_balancer.id, '321')])
+
     def test_delete(self):
         self.driver.delete(self.listener)
         self.assertEqual((self.listener, self.vip, 'delete'),
@@ -152,3 +146,12 @@ class TestNoopAmphoraLoadBalancerDriver(base.TestCase):
              'update_amphora_agent_config'),
             self.driver.driver.amphoraconfig[(
                 self.amphora.id, self.agent_config)])
+
+    def test_get_interface_from_ip(self):
+        result = self.driver.get_interface_from_ip(self.amphora,
+                                                   '198.51.100.99')
+        self.assertEqual('noop0', result)
+
+        result = self.driver.get_interface_from_ip(self.amphora,
+                                                   '198.51.100.9')
+        self.assertIsNone(result)
