@@ -11,13 +11,12 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_config import fixture as oslo_fixture
 from oslo_utils import uuidutils
 
-from octavia.api.v1.types import load_balancer as lb_types
 import octavia.common.constants as constants
 import octavia.common.exceptions as exceptions
 import octavia.common.validate as validate
@@ -31,7 +30,7 @@ class TestValidations(base.TestCase):
     # functional tests elsewhere (ex. repository tests)
 
     def setUp(self):
-        super(TestValidations, self).setUp()
+        super().setUp()
         self.conf = oslo_fixture.Config(cfg.CONF)
 
     def test_validate_url(self):
@@ -299,53 +298,50 @@ class TestValidations(base.TestCase):
             self.assertEqual(validate.subnet_exists(subnet_id), subnet)
 
     def test_network_exists_with_bad_network(self):
-        vip = lb_types.VIP()
-        vip.network_id = uuidutils.generate_uuid()
+        network_id = uuidutils.generate_uuid()
         with mock.patch(
                 'octavia.common.utils.get_network_driver') as net_mock:
             net_mock.return_value.get_network = mock.Mock(
                 side_effect=network_base.NetworkNotFound('Network not found'))
             self.assertRaises(
                 exceptions.InvalidSubresource,
-                validate.network_exists_optionally_contains_subnet, vip)
+                validate.network_exists_optionally_contains_subnet, network_id)
 
     def test_network_exists_with_valid_network(self):
-        vip = lb_types.VIP()
-        vip.network_id = uuidutils.generate_uuid()
-        network = network_models.Network(id=vip.network_id)
+        network_id = uuidutils.generate_uuid()
+        network = network_models.Network(id=network_id)
         with mock.patch(
                 'octavia.common.utils.get_network_driver') as net_mock:
             net_mock.return_value.get_network.return_value = network
             self.assertEqual(
-                validate.network_exists_optionally_contains_subnet(vip),
+                validate.network_exists_optionally_contains_subnet(network_id),
                 network)
 
     def test_network_exists_with_valid_subnet(self):
-        vip = lb_types.VIP()
-        vip.network_id = uuidutils.generate_uuid()
-        vip.subnet_id = uuidutils.generate_uuid()
+        network_id = uuidutils.generate_uuid()
+        subnet_id = uuidutils.generate_uuid()
         network = network_models.Network(
-            id=vip.network_id,
-            subnets=[vip.subnet_id])
+            id=network_id,
+            subnets=[subnet_id])
         with mock.patch(
                 'octavia.common.utils.get_network_driver') as net_mock:
             net_mock.return_value.get_network.return_value = network
             self.assertEqual(
-                validate.network_exists_optionally_contains_subnet(vip),
+                validate.network_exists_optionally_contains_subnet(
+                    network_id, subnet_id),
                 network)
 
     def test_network_exists_with_bad_subnet(self):
-        vip = lb_types.VIP()
-        vip.network_id = uuidutils.generate_uuid()
-        vip.subnet_id = uuidutils.generate_uuid()
-        network = network_models.Network(id=vip.network_id)
+        network_id = uuidutils.generate_uuid()
+        subnet_id = uuidutils.generate_uuid()
+        network = network_models.Network(id=network_id)
         with mock.patch(
                 'octavia.common.utils.get_network_driver') as net_mock:
             net_mock.return_value.get_network.return_value = network
             self.assertRaises(
                 exceptions.InvalidSubresource,
                 validate.network_exists_optionally_contains_subnet,
-                vip.network_id, vip.subnet_id)
+                network_id, subnet_id)
 
     def test_network_allowed_by_config(self):
         net_id1 = uuidutils.generate_uuid()
@@ -374,6 +370,18 @@ class TestValidations(base.TestCase):
                               validate.qos_policy_exists,
                               qos_policy_id)
 
+    def test_qos_extension_enabled(self):
+        network_driver = mock.Mock()
+        network_driver.qos_enabled.return_value = True
+        self.assertIsNone(validate.qos_extension_enabled(network_driver))
+
+    def test_qos_extension_disabled(self):
+        network_driver = mock.Mock()
+        network_driver.qos_enabled.return_value = False
+        self.assertRaises(exceptions.ValidationException,
+                          validate.qos_extension_enabled,
+                          network_driver)
+
     def test_check_session_persistence(self):
         valid_cookie_name_dict = {'type': 'APP_COOKIE',
                                   'cookie_name': 'chocolate_chip'}
@@ -442,3 +450,80 @@ class TestValidations(base.TestCase):
         self.assertRaises(exceptions.InvalidOption,
                           validate.ip_not_reserved,
                           '2001:0DB8::5')
+
+    def test_is_flavor_spares_compatible(self):
+        not_compat_flavor = {constants.COMPUTE_FLAVOR: 'chocolate'}
+        compat_flavor = {constants.LOADBALANCER_TOPOLOGY:
+                         constants.TOPOLOGY_SINGLE}
+
+        self.assertTrue(validate.is_flavor_spares_compatible(None))
+        self.assertTrue(validate.is_flavor_spares_compatible(compat_flavor))
+        self.assertFalse(
+            validate.is_flavor_spares_compatible(not_compat_flavor))
+
+    def test_check_default_ciphers_prohibit_list_conflict(self):
+        self.conf.config(group='api_settings',
+                         tls_cipher_prohibit_list='PSK-AES128-CBC-SHA')
+        self.conf.config(group='api_settings',
+                         default_listener_ciphers='ECDHE-ECDSA-AES256-SHA:'
+                         'PSK-AES128-CBC-SHA:TLS_AES_256_GCM_SHA384')
+
+        self.assertRaises(
+            exceptions.ValidationException,
+            validate.check_default_ciphers_prohibit_list_conflict)
+
+    def test_check_tls_version_list(self):
+        # Test valid list
+        validate.check_tls_version_list(['TLSv1.1', 'TLSv1.2', 'TLSv1.3'])
+        # Test invalid list
+        self.assertRaises(
+            exceptions.ValidationException,
+            validate.check_tls_version_list,
+            ['SSLv3', 'TLSv1.0'])
+        # Test empty list
+        self.assertRaises(
+            exceptions.ValidationException,
+            validate.check_tls_version_list,
+            [])
+
+    def test_check_tls_version_min(self):
+        self.conf.config(group="api_settings", minimum_tls_version='TLSv1.2')
+
+        # Test valid list
+        validate.check_tls_version_min(['TLSv1.2', 'TLSv1.3'])
+
+        # Test invalid list
+        self.assertRaises(exceptions.ValidationException,
+                          validate.check_tls_version_min,
+                          ['TLSv1', 'TLSv1.1', 'TLSv1.2'])
+
+    def test_check_default_tls_versions_min_conflict(self):
+        self.conf.config(group="api_settings", minimum_tls_version='TLSv1.2')
+
+        # Test conflict in listener default
+        self.conf.config(group="api_settings", default_listener_tls_versions=[
+                         'SSLv3', 'TLSv1.2'])
+        self.assertRaises(exceptions.ValidationException,
+                          validate.check_default_tls_versions_min_conflict)
+
+        # Test conflict in pool default
+        self.conf.config(group="api_settings", default_listener_tls_versions=[
+                         'TLSv1.2'])
+        self.conf.config(group="api_settings", default_pool_tls_versions=[
+                         'TLSv1', 'TLSv1.3'])
+        self.assertRaises(exceptions.ValidationException,
+                          validate.check_default_tls_versions_min_conflict)
+
+    def test_check_alpn_protocols(self):
+        # Test valid list
+        validate.check_alpn_protocols(['h2', 'http/1.1', 'http/1.0'])
+        # Test invalid list
+        self.assertRaises(
+            exceptions.ValidationException,
+            validate.check_alpn_protocols,
+            ['httpie', 'foobar/1.2.3'])
+        # Test empty list
+        self.assertRaises(
+            exceptions.ValidationException,
+            validate.check_alpn_protocols,
+            [])
